unit EditMaps;

{
 projet ADK-ISO (c)2002-2003 Paul TOTH <tothpaul@free.fr>

 http://www.web-synergy.net/naug-land/

}

interface

uses
 Windows,Classes,SysUtils,ADKUtils,ADKImages,ADKSprites,ADKMaps,Dialogs;

type
 TEditMap=class;
 TArray=class;

 TEditItem=class
 private
  fMap:TEditMap;
 public
  constructor Create(AMap:TEditMap);
  function Copy:TEditItem; virtual;
  function Past(AMap:TEditMap; dx,dy:integer):integer; virtual;
  property Map:TEditMap read fMap;
 end;

 TEditSprite=class(TEditItem)
 public
  procedure Show;
  procedure Hide;
 end;

 TEditLight=class;

 TEditImage=class(TEditItem)
 private
  fArray  :TArray;
  fLight  :TEditLight;
  function GetText:string;
  procedure SetText(Value:string);
 public
  Mask    :integer;
  Data    :TImageData106;
  destructor Destroy; override;
  procedure SaveToStream(AStream:TStream);
  procedure LoadFromStream105(AStream:TStream);
  procedure LoadFromStream(AStream:TStream);
  procedure SetArray(Width,Height,StepX,StepY:integer);
  procedure SetLight(Left,Top,Radius,Color:integer);
  procedure RemoveLight;
  procedure RemoveArray;
  procedure SetForeground(Value:boolean);
  function GetPoint(x,y:integer):boolean;
  property GetArray:TArray read fArray;
  property GetLigth:TEditLight read fLight;
  function Copy:TEditItem; override;
  function Past(AMap:TEditMap; dx,dy:integer):integer; override;
  property Text:string read GetText write SetText;
 end;

 TEditLight=class(TEditItem)
 public
  Data :TLightSource;
  Image:TEditImage;
  procedure SaveToStream(AStream:TStream);
  procedure LoadFromStream(AStream:TStream);
 end;

 TArray=class(TEditItem)
  Data:TArrayData106;
  Index105:integer;
  procedure SaveToStream(AStream:TStream);
  procedure LoadFromStream105(AStream:TStream);
  procedure LoadFromStream(AStream:TStream);
  property Index :integer read Index105 write Index105;
  property Width :integer read Data.Width write Data.Width;
  property Height:integer read Data.Height write Data.Height;
  property StepX :integer read Data.Step.X write Data.Step.X;
  property StepY :integer read Data.Step.Y write Data.Step.Y;
 end;
(*
 TTeleport=class(TEditItem)
  Index :integer; // image ou animation (voir TImage.Flags)
  Active:boolean;
  Target:TPoint;
 end;

 TPortal=class(TEditItem)
  Index  :integer; // image ou animation (voir TImage.Flags)
  Active :boolean;
  MapName:string;
  Target :TPoint;
 end;

 TSequence=class(TEditItem)
  First :integer; // premire image
  Count :integer; // nombre d'images
  Times :array {Count} of integer; // nombre de ms entre deux images
 end;

 TAnimation=class(TEditItem)
  Active:boolean; // animation active
  Index:integer;  // image actuelle
  Sequence:TSequence;
 end;
*)
 TEditMap=class(TStreamedObject)
 private
  fHeader    :THeader107;
  fMap       :array of TCell;
  fImages    :TObjectList;
   fArrays    :TObjectList;
  fLights    :TObjectList;
//   fAnimations:TObjectList;
//   fPortals   :TObjectList;
//   fTeleports :TObjectList;
  fItems     :TObjectList;
  fImageLib  :TADKImageLib;
  fFlagNames :TStringList;
  fTextList  :TStringList;
  EOnResize:TNotifyEvent;
  fNotFlags:integer;
  function GetCell(x,y:integer):PCell;
  function GetImageCount:integer;
  function GetImage(Index:integer):TEditImage;
  function GetItemCount:integer;
  function GetItem(Index:integer):TEditImage;
  procedure SaveStrings(AStream:TStream; List:TStringList; var Info:TArrayInfo);
  procedure LoadStrings(Stream:TStream; List:TStringList; const Info:TArrayInfo);
 protected
  Client:TRect; // client Rect to draw image inside
  ClientWidth:integer;
  List:TList;
  Creeper:TADKSprite;
  fShowSprite:boolean;
  fSpritePos:TPoint;
  fSpriteFrame:integer;
  procedure DrawSprite(Sprite:TADKSprite;AScrollX,AScrollY:integer);
  procedure DrawImage(img:TEditImage;ox,oy:integer);
  procedure LoadFromStream105(AStream:TStream);
  procedure LoadFromStream106(AStream:TStream);
  procedure LoadFromStream(AStream:TStream); override;
  procedure SaveToStream(AStream:TStream); override;
 public
  constructor Create;
  destructor Destroy; override;
  procedure SetSize(AWidth,AHeight:integer);
  procedure Clear;
  procedure Add(Image:TEditImage);
  function NewImage(x,y,Index,Base:integer; Flags:TImageFlags):TEditImage;
  function AddImage(x,y,Index,Base:integer; Flags:TImageFlags):TEditImage;
  procedure Exchange(i,j:integer);
  procedure Move(Old,NewIndex:integer);
  procedure Remove(Index:Integer);
  procedure DrawImages(Front:boolean; AWidth,AHeight,AScrollX,AScrollY:integer);
  procedure _DrawMap(AWidth,AHeight,AScrollX,AScrollY:integer);
  procedure _DrawItems(AWidth,AHeight,AScrollX,AScrollY:integer);
  function GetItemXY(x,y:integer; var Current:TEditImage; ForeGroundOnly:boolean):integer;
  procedure AddFlag(Name:string);
  function FlagIndex(Mask:integer):integer;
  function FlagMask(index:integer):integer;
  function Intersect(Image1,Image2:integer):boolean;
  procedure SetSpritePos(x,y:integer);
  property Cells[x,y:integer]:PCell read GetCell;
  property ImageCount:integer read GetImageCount;
  property Images[Index:integer]:TEditImage read GetImage;
  property ItemCount:integer read GetItemCount;
  property Items[Index:integer]:TEditImage read GetItem;
  property ImageLib:TADKImageLib read fImageLib write fImageLib;
  property Width:integer read fHeader.Width;
  property Height:integer read fHeader.Height;
  property Origin:TPoint read fHeader.Origin write fHeader.Origin;
  property Ambient:integer read fHeader.Ambient write fHeader.Ambient;
  property Options:TMapOptions read fHeader.Options write fHeader.Options;
  property Flags:integer read fHeader.Flags write fHeader.Flags;
  property FlagNames:TStringList read fFlagNames;
  property TextList:TStringList read fTextList;
  property OnResize:TNotifyEvent read EOnResize write EOnResize;
  property ShowSprite:boolean read fShowSprite write fShowSprite;
 end;

implementation

uses ADKScreens;

//--------------------------------------------------------------------//

function SortItems(Item1,Item2:pointer):integer;
var
 Image1:TEditImage absolute Item1;
 Image2:TEditImage absolute Item2;
begin
 result:=(Image1.Data.Position.y+Image1.Data.BaseLine)-(Image2.Data.Position.y+Image2.Data.BaseLine);
 if result=0 then Result:=integer(Item1)-integer(Item2);
end;

//--------------------------------------------------------------------//
constructor TEditItem.Create(AMap:TEditMap);
begin
 fMap:=AMap;
end;

function TEditItem.Copy:TEditItem;
begin
 raise Exception.Create('Ne peut copier '+ClassName);
end;

function TEditItem.Past(AMap:TEditMap; dx,dy:integer):integer;
begin
 Result:=-1;
 fMap:=AMap;
end;

//--------------------------------------------------------------------//
procedure TEditSprite.Show;
begin
 fMap.fItems.Add(Self);
end;

procedure TEditSprite.Hide;
begin
 fMap.fItems.Extract(Self);
end;

//--------------------------------------------------------------------//

procedure TEditLight.SaveToStream(AStream:TStream);
begin
 Data.ObjectID:=fMap.fImages.IndexOf(Image);
 if Data.ObjectID<0 then Data.ObjectID:=fMap.fItems.IndexOf(Image)+fMap.fImages.Count;
 AStream.WriteBuffer(Data,SizeOf(Data));
end;

procedure TEditLight.LoadFromStream(AStream:TStream);
var
 c:integer;
begin
 AStream.ReadBuffer(Data,SizeOf(Data));
 c:=fMap.fImages.Count;
 if Data.ObjectID<c then
  Image:=fMap.fImages[Data.ObjectID]
 else
  Image:=fMap.fItems[Data.ObjectID-c];
 Image.fLight:=Self;
end;

//--------------------------------------------------------------------//
destructor TEditImage.Destroy;
begin
 if fArray<>nil then fMap.fArrays.Remove(fArray);
 inherited;
end;

procedure TEditImage.SetArray(Width,Height,StepX,StepY:integer);
begin
 if (Width<=1)and(Height<=1) then begin
  RemoveArray;
  exit;
 end;
 if fArray=nil then begin
  fArray:=TArray.Create(fMap);
//  fArray.Index:=Data.Index;
  fMap.fArrays.Add(fArray);
  include(Data.Flags,ifArray);
 end;
 fArray.Width:=Width;
 fArray.Height:=Height;
 fArray.StepX:=StepX;
 fArray.stepY:=StepY;
end;

procedure TEditImage.SetLight(Left,Top,Radius,Color:integer);
begin
 if fLight=nil then begin
  fLight:=TEditLight.Create(fMap);
  fMap.fLights.Add(fLight);
  fLight.Image:=Self;
 end;
 fLight.Data.Color:=Color;
 fLight.Data.Position.X:=Left;
 fLight.Data.Position.Y:=Top;
 fLight.Data.Radius:=Radius;
 Include(Data.Flags,ifLightSource);
end;

procedure TEditImage.RemoveLight;
begin
 if fLight=nil then exit;
 fMap.fLights.Remove(fLight);
 Exclude(Data.Flags,ifLightSource);
 fLight.Free;
 fLight:=nil;
end;

procedure TEditImage.RemoveArray;
begin
 if fArray=nil then exit;
 fMap.fArrays.Remove(fArray);
 fArray:=nil;
 exclude(Data.Flags,ifArray);
end;

procedure TEditImage.SetForeground(Value:boolean);
begin
 if Value = (ifForeground in Data.Flags) then exit;
 if Value then begin
  include(Data.Flags,ifForeground);
  fMap.fImages.Extract(Self);
  fMap.fItems.Add(Self);
 end else begin
  exclude(Data.Flags,ifForeground);
  fMap.fItems.Extract(Self);
  fMap.fImages.Add(Self);
 end;
end;

function TEditImage.GetPoint(x,y:integer):boolean;
var
 size:TPoint;
 ox,oy:integer;
 px,py:integer;
 w,h:integer;
begin
 result:=false;
 if (y<Data.Position.Y) then exit;
 if ifArray in Data.Flags then begin
  ox:=Data.Position.x;
  oy:=Data.Position.y;
  size:=fMap.fImageLib.Size[Data.Index];
  for h:=0 to fArray.Height-1 do begin
   px:=ox;
   py:=oy;
   for w:=0 to fArray.Width-1 do begin
    if (x<px) then break;
    if (y<py) then exit;
    if (x<px+size.x)and(y<py+size.y) then begin
     result:=true;
     exit;
    end;
    inc(px,fArray.Stepx);
    inc(py,fArray.Stepy);
   end;
   dec(ox,fArray.Stepx);
   inc(oy,fArray.Stepy);
  end;
 end else begin
  if (x<Data.Position.x) then exit;
  size:=fMap.fImageLib.Size[Data.Index];
  result:=(x<Data.Position.x+size.x)and(y<Data.Position.y+size.y);
 end;
end;

procedure TEditImage.SaveToStream(AStream:TStream);
begin
 if ifArray in Data.Flags then Data.Data:=fMap.fArrays.IndexOf(fArray);
 AStream.WriteBuffer(Data,SizeOf(Data));
end;

procedure TEditImage.LoadFromStream105(AStream:TStream);
var
 Data105:TImageData105;
begin
 AStream.ReadBuffer(Data105,SizeOf(Data105));
 Data.Index:=Data105.Index;
 Data.Position:=Data105.Position;
 Data.BaseLine:=Data105.BaseLine;
 Data.Flags:=Data105.Flags;
 Data.Hidden:=0;
 if ifArray in Data.Flags then begin
  fArray:=fMap.fArrays[Data.Index];
  Data.Data:=Data.Index;
  Data.Index:=fArray.Index105;
 end;
end;

procedure TEditImage.LoadFromStream(AStream:TStream);
begin
 AStream.ReadBuffer(Data,SizeOf(Data));
 if ifArray in Data.Flags then begin
  fArray:=fMap.fArrays[Data.Data];
  //Data.Index:=fArray.Index;
 end;
end;

function TEditImage.Copy:TEditItem;
begin
 Result:=TEditImage.Create(fMap);
 with TEditImage(Result) do begin
  Data:=Self.Data;
  if ifArray in Data.Flags then begin
   fArray:=TArray.Create(fMap);
   //fArray.Index :=Data.Index;
   fArray.Width :=Self.fArray.Width;
   fArray.Height:=Self.fArray.Height;
   fArray.StepX :=Self.fArray.StepX;
   fArray.stepY :=Self.fArray.StepY;
  end;
  if ifLightSource in Data.Flags then begin
   fLight:=TEditLight.Create(fMap);
   fLight.Image:=TEditImage(Result);
   fLight.Data:=Self.fLight.Data;
  end;
 end;
end;

function TEditImage.Past(AMap:TEditMap; dx,dy:integer):integer;
begin
 fMap:=AMap;
 if ifForeGround in data.Flags then
  Result:=AMap.fItems.add(Self)
 else
  Result:=AMap.fImages.Add(Self);
 if ifArray in Data.Flags then AMap.fArrays.Add(fArray);
 if ifLightSource in Data.Flags then AMap.fLights.Add(fLight); 
 inc(Data.Position.x,dx);
 inc(Data.Position.y,dy);
 Mask:=$ff0000;
end;

function TEditImage.GetText:string;
var
 i:integer;
begin
 i:=Map.TextList.IndexOfObject(Self);
 if i<0 then Result:='' else Result:=Map.TextList[i];
end;

procedure TEditImage.SetText(Value:string);
var
 i:integer;
begin
 i:=Map.TextList.IndexOfObject(Self);
 if i<0 then begin
  if Value='' then exit;
  Map.TextList.AddObject(Value,Self);
 end else begin
  if Value='' then Map.TextList.Delete(i) else Map.TextList[i]:=Value;
 end;
end;

//--------------------------------------------------------------------//
procedure TArray.SaveToStream(AStream:TStream);
begin
 AStream.WriteBuffer(Data,SizeOf(Data));
end;

procedure TArray.LoadFromStream105(AStream:TStream);
begin
 AStream.ReadBuffer(Index105,SizeOf(Index105));
 AStream.ReadBuffer(Data,SizeOf(Data));
end;

procedure TArray.LoadFromStream(AStream:TStream);
begin
 AStream.ReadBuffer(Data,SizeOf(Data));
end;
//--------------------------------------------------------------------//

constructor TEditMap.Create;
begin
 FillChar(fHeader,SizeOf(fHeader),0);
 fImages:=TObjectList.Create;
 fArrays:=TObjectList.Create;
 fLights:=TObjectList.Create;
//  fAnimations:=TObjectList.Create;
//  fPortals:=TObjectList.Create;
//  fTeleports:=TObjectList.Create;
 fItems:=TObjectList.Create;
 fFlagNames:=TStringList.Create;
 fTextList:=TStringList.Create;
 List:=TList.Create;
 fHeader.Options:=[moDayLight];
 if FileExists('Temoin.SPR') then Creeper:=TADKSprite.Create('Temoin.SPR');
end;

destructor TEditMap.Destroy;
begin
 fImages.Free;
  fArrays.Free;
 fLights.Free;
//  fAnimations.Free;
//  fPortals.Free;
//  fTeleports.Free;
 fItems.Free;
 List.Free;
 inherited;
end;

function TEditMap.GetCell(x,y:integer):PCell;
begin
 result:=@fMap[x+fHeader.Width*y];
end;

function TEditMap.GetImageCount:integer;
begin
 result:=fImages.Count;
end;

function TEditMap.GetImage(Index:integer):TEditImage;
begin
 result:=fImages[Index];
end;

function TEditMap.GetItemCount:integer;
begin
 result:=fItems.Count;
end;

function TEditMap.GetItem(Index:integer):TEditImage;
begin
 result:=fItems[Index];
end;

procedure TEditMap.SetSize(AWidth,AHeight:integer);
var
 i:integer;
begin
 fHeader.Width:=AWidth;
 fHeader.Height:=AHeight;
 i:=AWidth*AHeight;
 SetLength(fMap,i);
 FillChar(fMap[0],i*SizeOf(TCell),0);
 if Assigned(EOnResize) then EOnResize(Self);
end;

procedure TEditMap.Clear;
begin
 fImages.Clear;
 fArrays.Clear;
 fLights.Clear;
// fAnimations.Clear;
// fPortals.Clear;
// fTeleports.Clear;
 fItems.Clear;
 fFlagNames.Clear;
 fTextList.Clear;
end;

procedure TEditMap.LoadFromStream105(AStream:TStream);
var
 Header105:THeader105;
 i:integer;
 img:TEditImage;
 arr:TArray;
begin
 AStream.ReadBuffer(Header105,SizeOf(Header105));
 if Header105.Sign<>cardinal(MAP_SIGNATURE) then Error('Carte invalide');
 if Header105.Ver<>105 then Error('Version de carte non supporte');
 FillChar(fHeader,SizeOf(fHeader),0);
 fHeader.Signature:=cardinal(MAP_SIGNATURE);
 fHeader.Version:=MAP_VERSION;
 fHeader.Width:=Header105.Width;
 fHeader.Height:=Header105.Height;
 fHeader.Origin:=Header105.Origin;
 fHeader.Options:=[moDayLight];
 fHeader.Arrays:=Header105.Arrays;
 fHeader.Images:=Header105.Images;
 fHeader.Items :=Header105.Items;
// les cases
 i:=fHeader.Width*fHeader.Height;
 SetLength(fMap,i);
 if Assigned(EOnResize) then EOnResize(Self);
 AStream.ReadBuffer(fMap[0],i*SizeOf(TCell));
// les tableaux
 for i:=0 to fHeader.Arrays.Count-1 do begin
  arr:=TArray.Create(Self);
  arr.LoadFromStream105(AStream);
  fArrays.Add(arr)
 end;
// les images
 for i:=0 to fHeader.Images.Count-1 do begin
  img:=TEditImage.Create(Self);
  img.LoadFromStream105(AStream);
  fImages.Add(img)
 end;
// les objets 3D
 for i:=0 to fHeader.Items.Count-1 do begin
  img:=TEditImage.Create(Self);
  img.LoadFromStream105(AStream);
  fItems.add(img);
 end;
 ShowMessage(
  'Attention ! cette carte est dans un ancien format (105).'#13#10+
  'Si vous la sauvegardez elle sera automatiquement au nouveau format ('+IntToStr(MAP_VERSION)+').'
 );
end;

procedure TEditMap.LoadFromStream106(AStream:TStream);
var
 Header106:THeader106;
 i:integer;
 img:TEditImage;
 arr:TArray;
begin
 AStream.ReadBuffer(Header106,SizeOf(Header106));
 if Header106.Signature<>cardinal(MAP_SIGNATURE) then Error('Carte invalide');
 if Header106.Version<>106 then Error('Version de carte non supporte');
 FillChar(fHeader,SizeOf(fHeader),0);
 fHeader.Signature:=cardinal(MAP_SIGNATURE);
 fHeader.Version:=MAP_VERSION;
 fHeader.Width:=Header106.Width;
 fHeader.Height:=Header106.Height;
 fHeader.Origin:=Header106.Origin;
 fHeader.Options:=[moDayLight];
 fHeader.Arrays:=Header106.Arrays;
 fHeader.Images:=Header106.Images;
 fHeader.Items :=Header106.Items;
// les cases
 i:=fHeader.Width*fHeader.Height;
 SetLength(fMap,i);
 if Assigned(EOnResize) then EOnResize(Self);
 AStream.ReadBuffer(fMap[0],i*SizeOf(TCell));
// les tableaux
 for i:=0 to fHeader.Arrays.Count-1 do begin
  arr:=TArray.Create(Self);
  arr.LoadFromStream(AStream);
  fArrays.Add(arr)
 end;
// les images
 for i:=0 to fHeader.Images.Count-1 do begin
  img:=TEditImage.Create(Self);
  img.LoadFromStream(AStream);
  fImages.Add(img)
 end;
// les objets 3D
 for i:=0 to fHeader.Items.Count-1 do begin
  img:=TEditImage.Create(Self);
  img.LoadFromStream(AStream);
  fItems.add(img);
 end;
 ShowMessage(
  'Attention ! cette carte est dans un ancien format (106).'#13#10+
  'Si vous la sauvegardez elle sera automatiquement au nouveau format ('+IntToStr(MAP_VERSION)+').'
 );
end;

procedure TEditMap.LoadFromStream(AStream:TStream);
var
 i,j:integer;
 img:TEditImage;
 arr:TArray;
 light:TEditLight;
begin
 Clear;
 AStream.ReadBuffer(fHeader,SizeOf(fHeader));
 if fHeader.Signature<>cardinal(MAP_SIGNATURE) then Error('Carte invalide');
 if fHeader.Version=105 then begin
  AStream.Position:=0;
  LoadFromStream105(AStream);
  exit;
 end;
 if fHeader.Version=106 then begin
  AStream.Position:=0;
  LoadFromStream106(AStream);
  exit;
 end;
 if fHeader.Version<>MAP_VERSION then Error('Version de carte non supporte');
// les cases
 i:=fHeader.Width*fHeader.Height;
 SetLength(fMap,i);
 if Assigned(EOnResize) then EOnResize(Self);
 AStream.ReadBuffer(fMap[0],i*SizeOf(TCell));
// les tableaux
 for i:=0 to fHeader.Arrays.Count-1 do begin
  arr:=TArray.Create(Self);
  arr.LoadFromStream(AStream);
  fArrays.Add(arr)
 end;
// les images
 for i:=0 to fHeader.Images.Count-1 do begin
  img:=TEditImage.Create(Self);
  img.LoadFromStream(AStream);
  fImages.Add(img)
 end;
// les objets 3D
 for i:=0 to fHeader.Items.Count-1 do begin
  img:=TEditImage.Create(Self);
  img.LoadFromStream(AStream);
  fItems.add(img);
 end;
// Les Textes
 LoadStrings(AStream,fTextList,fHeader.Texts);
 for i:=0 to fTextList.Count-1 do begin
  j:=integer(fTextList.Objects[i]);
  if j>=fImages.Count then
   img:=fItems[j-fImages.Count]
  else
   img:=fImages[j];
  fTextList.Objects[i]:=img;
 end;
// Les Indicateurs
 LoadStrings(AStream,fFlagNames,fHeader.FlagNames);
// Les Lumires
 for i:=0 to fHeader.Lights.Count-1 do begin
  light:=TEditLight.Create(Self);
  light.LoadFromStream(AStream);
  fLights.add(light);
 end;
end;

procedure TEditMap.SaveStrings(AStream:TStream; List:TStringList; var Info:TArrayInfo);
var
 Strings:array of TStringInfo;
 i,l,o:integer;
 s:string;
begin
 Info.Count:=List.Count;
 Info.Offset:=AStream.Position;
 SetLength(Strings,Info.Count);
 o:=Info.Offset+cardinal(Info.Count*SizeOf(TStringInfo));
 for i:=0 to Info.Count-1 do begin
  l:=Length(List[i]);
  Strings[i].ObjectID:=integer(List.Objects[i]);
  Strings[i].Offset:=o;
  Strings[i].Length:=l;
  inc(o,l);
 end;
 AStream.WriteBuffer(Strings[0],Info.Count*SizeOf(TStringInfo));
 for i:=0 to Info.Count-1 do begin
  s:=List[i];
  l:=Length(s);
  AStream.WriteBuffer(s[1],l);
 end;
end;

procedure TEditMap.LoadStrings(Stream:TStream; List:TStringList; const Info:TArrayInfo);
var
 i:integer;
 Sizes:array of TStringInfo;
 s:string;
 l:integer;
begin
 SetLength(Sizes,Info.Count);
 Stream.ReadBuffer(Sizes[0],Info.Count*SizeOf(TStringInfo));
 for i:=0 to Info.Count-1 do begin
  l:=sizes[i].Length;
  SetLength(s,l);
  Stream.ReadBuffer(s[1],l);
  List.AddObject(s,TObject(sizes[i].ObjectID));
 end;
end;

procedure TEditMap.SaveToStream(AStream:TStream);
var
 i,j:integer;
 img:TEditImage;
begin
 fHeader.Signature:=cardinal(MAP_SIGNATURE);
 fHeader.Version :=MAP_VERSION;
 AStream.WriteBuffer(fHeader,SizeOf(fHeader));

 AStream.WriteBuffer(fMap[0],fHeader.Width*fHeader.Height*SizeOf(TCell));

 fHeader.Arrays.Count:=fArrays.Count;
 fHeader.Arrays.Offset:=AStream.Position;
 for i:=0 to fArrays.Count-1 do begin
  TArray(fArrays[i]).SaveToStream(AStream);
 end;

 fHeader.Images.Count:=fImages.Count;
 fHeader.Images.Offset:=AStream.Position;
 for i:=0 to fImages.Count-1 do begin
  TEditImage(fImages[i]).SaveToStream(AStream);
 end;

 fHeader.Items.Count:=fItems.Count;
 fHeader.Items.Offset:=AStream.Position;
 for i:=0 to fItems.Count-1 do begin
  TEditImage(fItems[i]).SaveToStream(AStream);
 end;

 for i:=0 to fTextList.Count-1 do begin
  img:=fTextList.Objects[i] as TEditImage;
  if ifForeGround in img.Data.Flags then begin
   j:=fHeader.Images.Count+cardinal(fItems.IndexOf(img));
  end else begin
   j:=fImages.IndexOf(img);
  end;
  fTextList.Objects[i]:=TObject(j);
 end;
 SaveStrings(AStream,fTextList,fHeader.Texts);
 for i:=0 to fTextList.Count-1 do begin
  j:=integer(fTextList.Objects[i]);
  if j>=fImages.Count then
   img:=fItems[j-fImages.Count]
  else
   img:=fImages[j];
  fTextList.Objects[i]:=img;
 end;
 SaveStrings(AStream,fFlagNames,fHeader.FlagNames);

 fHeader.Lights.Count:=fLights.Count;
 fHeader.Lights.Offset:=AStream.Position;
 for i:=0 to fLights.Count-1 do begin
  TEditLight(fLights[i]).SaveToStream(AStream);
 end;

 AStream.Position:=0;
 AStream.WriteBuffer(fHeader,SizeOf(fHeader));
end;

procedure TEditMap.Add(Image:TEditImage);
begin
 fImages.Add(Image);
end;

function TEditMap.NewImage(x,y,Index,Base:integer; Flags:TImageFlags):TEditImage;
begin
 Result:=TEditImage.Create(Self);
 Result.Data.Index:=Index;
 Result.Data.Position.x:=x;
 Result.Data.Position.Y:=y;
 Result.Data.BaseLine:=Base;
 Result.Data.Flags:=Flags;
end;

function TEditMap.AddImage(x,y,Index,Base:integer; Flags:TImageFlags):TEditImage;
begin
 Result:=NewImage(x,y,Index,Base,Flags);
 Add(Result);
end;

procedure TEditMap.DrawImage(Img:TEditImage; ox,oy:integer);
var
 x,y:integer;
 size:TPoint;
 clip:TRect;
 step:integer;
 pitch:integer;
begin
 size:=fImageLib.Size[Img.Data.Index];
 if ox+size.x<Client.Left then exit;
 if oy+size.y<Client.Left then exit;
 x:=ox;
 y:=oy;
 pitch:=ADKScreen.Pitch;//ClientWidth;
 if ClipRect(Client,ox,oy,size.x,size.y,clip) then begin
  if ifSwap in img.Data.Flags then begin
   step:=-1;
   inc(x,size.x-1);
   SwapRect(clip,size.x);
  end else begin
   step:=+1;
  end;
  if ifFlip in img.Data.Flags then begin
   pitch:=-pitch;
   inc(y,size.y-1);
   FlipRect(clip,size.y);
  end;
  fImageLib.DrawRect(img.Data.Index,ADKScreen.Pixels[x,y],pitch,step,Clip,img.Mask);
 end else begin
  if ifSwap in img.Data.Flags then begin
   step:=-1;
   inc(x,size.x-1);
  end else begin
   step:=+1;
  end;
  if ifFlip in img.Data.Flags then begin
   pitch:=-pitch;
   inc(y,size.y-1);
  end;
  fImageLib.Draw(img.Data.Index,ADKScreen.Pixels[x,y],pitch,step,img.Mask);
 end;
 if (ifLightSource in img.Data.Flags)and(ADKScreen is TADKScreen3D) then begin
  with Img.fLight.Data do
   TADKScreen3D(ADKScreen).Circle(ox+Position.X,oy+Position.Y,Radius,Color);
 end;
end;

procedure TEditMap.DrawSprite(Sprite:TADKSprite;AScrollX,AScrollY:integer);
const
 Direction=DirS;
var
 x,y,w,h:integer;
 R,C:TRect;
begin
 x:=fSpritePos.X+Origin.x+AScrollX;
 y:=fSpritePos.Y+Origin.y+AScrollY;

 ADKScreen.PutPixel(x-2,y,$ffffff);
 ADKScreen.PutPixel(x-1,y,$ffffff);
 ADKScreen.PutPixel(x,y,$ffffff);
 ADKScreen.PutPixel(x+1,y,$ffffff);
 ADKScreen.PutPixel(x+2,y,$ffffff);
 ADKScreen.PutPixel(x,y-1,$ffffff);
 ADKScreen.PutPixel(x,y+1,$ffffff);

// Frame:=(GetTickCount div 100) mod cardinal(Creeper.FrameCount(Direction));
 with Creeper.SpriteFrames(Direction)^ do begin
  dec(x,Origin.X);
  dec(y,Origin.Y);
 end;
 with Creeper.FrameHeader(Direction,fSpriteFrame)^ do begin
  inc(x,Left);
  inc(y,Top);
  w:=Width;
  h:=Height;
 end;
 R:=Rect(0,0,ADKScreen.Width,ADKScreen.Height);
 if ClipRect(R,x,y,w,h,C) then
  Sprite.FillRGB32Rect(Direction,fSpriteFrame,ADKScreen.Pixels[x,y],ADKScreen.Pitch,C)
 else
  Sprite.FillRGB32(Direction,fSpriteFrame,ADKScreen.Pixels[x,y],ADKScreen.Pitch);
end;

procedure TEditMap.DrawImages(Front:boolean; AWidth,AHeight,AScrollX,AScrollY:integer);
var
 list:TList;
 i:integer;
 img:TEditImage;
 imgindex:integer;
 x,y:integer;
 size:TPoint;
 DrawCreeper:boolean;
 CreeperTop :integer;

 procedure DrawArray;
 var
  ar:TArray;
  ox:integer;
  oy:integer;
  ay:integer;
  ax:integer;
 begin
  ar:=img.GetArray;
  //imgindex:=ar.Index;
  size:=fImageLib.Size[imgIndex];
  ox:=x;
  oy:=y;
  for ay:=0 to ar.Height-1 do begin
   x:=ox;
   y:=oy;
   for ax:=0 to ar.Width-1 do begin
    if x>AWidth then break;
    if y>AHeight then break;
    if ((x+size.x)>=0)and((y+size.y)>=0) then DrawImage(img,x,y);
    inc(x,ar.Stepx);
    inc(y,ar.Stepy);
   end;
   dec(ox,ar.Stepx);
   inc(oy,ar.Stepy);
   if oy>AHeight then exit;
  end;
 end;

begin
 if fImageLib<>nil then begin
  fNotFlags:=not fHeader.Flags;
  if Front then begin
   fItems.Sort(SortItems);
   list:=fItems;
   DrawCreeper:=fShowSprite and (Creeper<>nil);
   if DrawCreeper then begin
    fSpriteFrame:=(GetTickCount div 100) mod cardinal(Creeper.FrameCount(DirS));
    CreeperTop:=fSpritePos.Y+Origin.Y+AScrollY+Creeper.FrameHeader(DirS,fSpriteFrame).Top;
   end;
  end else begin
   list:=fImages;
   DrawCreeper:=False;
  end;
  Client:=Rect(0,0,AWidth-1,AHeight-1);
  ClientWidth:=AWidth;
  for i:=0 to list.Count-1 do begin
   img:=list[i];
   y:=img.Data.Position.y+AScrollY;
   // le Creeper
   if DrawCreeper then begin
    if y+img.Data.BaseLine>CreeperTop then begin
     DrawSprite(Creeper,AScrollX,AScrollY);
     DrawCreeper:=False;
    end;
   end;
   // test les flags !
   if ifNotFlag in img.Data.Flags then begin
    if (img.Data.Hidden and fHeader.Flags)<>0 then continue;
   end else begin
    if (img.Data.Hidden and fNotFlags)<>0 then continue;
   end;
   x:=img.Data.Position.x+AScrollX;
   // image trop  gauche ou en bas
   if y>Client.Bottom then continue;
   imgindex:=img.Data.Index;
   if ifArray in img.Data.Flags then begin
    DrawArray;
   end else begin
    if x>Client.Right  then continue;
    // image trop  droite ou en haut
    size:=fImageLib.Size[imgIndex];
    if x+size.x<Client.Left then continue;
    if y+size.y<Client.Top  then continue;
    // clipping
    DrawImage(img,x,y);
   end;
  end;
  if DrawCreeper then DrawSprite(Creeper,AScrollX,AScrollY);
 end;
end;

procedure TEditMap._DrawMap(AWidth,AHeight,AScrollX,AScrollY:integer);
var
 i:integer;
 img:TEditImage;
 imgindex:integer;
 x,y:integer;
 size:TPoint;

 procedure DrawArray;
 var
  ar:TArray;
  ox:integer;
  oy:integer;
  ay:integer;
  ax:integer;
 begin
  ar:=img.GetArray;
  //imgindex:=ar.Index;
  size:=fImageLib.Size[imgIndex];
  ox:=x;
  oy:=y;
  for ay:=0 to ar.Height-1 do begin
   x:=ox;
   y:=oy;
   for ax:=0 to ar.Width-1 do begin
    if x>AWidth then break;
    if y>AHeight then break;
    if ((x+size.x)>=0)and((y+size.y)>=0) then DrawImage(img,x,y);
    inc(x,ar.Stepx);
    inc(y,ar.Stepy);
   end;
   dec(ox,ar.Stepx);
   inc(oy,ar.Stepy);
   if oy>AHeight then exit;
  end;
 end;

begin
 if fImageLib<>nil then begin
  Client:=Rect(0,0,AWidth-1,AHeight-1);
  ClientWidth:=AWidth;
  for i:=0 to ImageCount-1 do begin
   img:=Images[i];
   x:=img.Data.Position.x+AScrollX;
   y:=img.Data.Position.y+AScrollY;
   // image trop  gauche ou en bas
   if y>Client.Bottom then continue;
   if ifArray in img.Data.Flags then begin
    DrawArray;
   end else begin
    if x>Client.Right  then continue;
    imgindex:=img.Data.Index;
    // image trop  droite ou en haut
    size:=fImageLib.Size[imgIndex];
    if x+size.x<Client.Left then continue;
    if y+size.y<Client.Left then continue;
    // clipping
    DrawImage(img,x,y);
   end;
  end;
 end;
end;

procedure TEditMap._DrawItems(AWidth,AHeight,AScrollX,AScrollY:integer);
var
 i:integer;
 img:TEditImage;
 x,y:integer;
begin
 fItems.Sort(SortItems);
 if fImageLib<>nil then begin
  Client:=Rect(0,0,AWidth-1,AHeight-1);
  ClientWidth:=AWidth;
  for i:=0 to fItems.Count-1 do begin
   img:=fItems[i];
   x:=img.Data.Position.x+AScrollX;
   y:=img.Data.Position.y+AScrollY;
   // image trop  gauche ou en bas
   if y>Client.Bottom then continue;
   if x>Client.Right  then continue;
   // clipping
   DrawImage(img,x,y);
  end;
 end;
end;

procedure TEditMap.Exchange(i,j:integer);
begin
 fImages.Exchange(i,j);
end;

procedure TEditMap.Move(Old,NewIndex:integer);
begin
 fImages.Move(Old,NewIndex);
end;

procedure TEditMap.Remove(Index:Integer);
begin
 if Index<fImages.Count then
  fImages.Delete(Index)
 else
  fItems.Delete(Index-fImages.Count);
end;

function TEditMap.GetItemXY(x,y:integer; var Current:TEditImage; ForeGroundOnly:boolean):integer;
var
 i:integer;
begin
 try
  for i:=fItems.Count-1  downto 0 do if Items[i].getPoint(x,y)  then List.Add(Items[i]);
  if not ForeGroundOnly then
   for i:=fImages.Count-1 downto 0 do if Images[i].getPoint(x,y) then List.Add(Images[i]);
  if List.count=0 then begin
   Current:=nil;
   Result:=-1
  end else begin
   if (Current=nil)or(List.Count=1) then
    i:=0 // par dfaut le premier
   else begin
    i:=List.IndexOf(Current)+1;
    if i=List.count then i:=0; // boucle
   end;
   Current:=List[i];
   if ifForeground in Current.Data.Flags then
    Result:=fItems.IndexOf(Current)
   else
    Result:=fImages.IndexOf(Current);
  end;
 finally
  List.Clear;
 end;
end;

procedure TEditMap.AddFlag(Name:string);
var
 f:integer;
 i:integer;
 m:integer;
begin
 f:=0;
 for i:=0 to fFlagNames.Count-1 do begin
  f:=f or integer(fFlagNames.Objects[i]);
 end;
 m:=1;
 for i:=0 to 31 do begin
  if f and m=0 then begin
   fFlagNames.AddObject(Name,TObject(m));
   exit;
  end;
  m:=m shl 1;
 end;
end;

function TEditMap.FlagIndex(Mask:integer):integer;
var
 i:integer;
begin
 for i:=0 to fFlagNames.Count-1 do begin
  if Integer(fFlagNames.Objects[i])=Mask then begin
   Result:=i;
   exit;
  end;
 end;
 Result:=-1;
end;

function TEditMap.FlagMask(index:integer):integer;
begin
 Result:=integer(fFlagNames.Objects[Index]);
end;

function TEditMap.Intersect(Image1,Image2:integer):boolean;
var
 i1,i2:TEditImage;
 s1,s2:TPoint;
begin
 i1:=Images[Image1];
 i2:=Images[Image2];
 s1:=fImageLib.Size[i1.Data.Index];
 s2:=fImageLib.Size[i2.Data.Index];
 Result:=False;
 if (i1.Data.Position.x>i2.Data.Position.x+s2.x)
 or (i2.Data.Position.x>i1.Data.Position.x+s1.x)
 or (i1.Data.Position.y>i2.Data.Position.y+s2.y)
 or (i2.data.Position.Y>i1.data.Position.y+s2.y) then exit;
 Result:=True;
(*
function TEditImage.GetPoint(x,y:integer):boolean;
var
 size:TPoint;
 ox,oy:integer;
 px,py:integer;
 w,h:integer;
begin
 result:=false;
 if (y<Data.Position.Y) then exit;
 if ifArray in Data.Flags then begin
  ox:=Data.Position.x;
  oy:=Data.Position.y;
  size:=fMap.fImageLib.Size[Data.Index];
  for h:=0 to fArray.Height-1 do begin
   px:=ox;
   py:=oy;
   for w:=0 to fArray.Width-1 do begin
    if (x<px) then break;
    if (y<py) then exit;
    if (x<px+size.x)and(y<py+size.y) then begin
     result:=true;
     exit;
    end;
    inc(px,fArray.Stepx);
    inc(py,fArray.Stepy);
   end;
   dec(ox,fArray.Stepx);
   inc(oy,fArray.Stepy);
  end;
 end else begin
  if (x<Data.Position.x) then exit;
  size:=fMap.fImageLib.Size[Data.Index];
  result:=(x<Data.Position.x+size.x)and(y<Data.Position.y+size.y);
 end;
end;
*)
end;

procedure TEditMap.SetSpritePos(x,y:integer);
var
 ox,oy:integer;
 mx,my:integer;
begin
 ox:=x-fHeader.Origin.x;
 oy:=y-fHeader.Origin.y;
 mx:=(2*oy+ox) div 96;
 my:=(2*oy-ox) div 96;
 if mx<0 then mx:=0 else if mx>=Width  then mx:=Width-1;
 if my<0 then my:=0 else if my>=Height then my:=Height-1;
 fSpritePos.x:=48*(mx-my);
 fSpritePos.y:=24*(mx+my)+24;
end;

end.
