unit ADKMainFrame;

{
 projet ADK-ISO (c)2002-2003 Paul TOTH <tothpaul@free.fr>

 http://www.web-synergy.net/naug-land/

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,MMSystem,Sound,
  ADKMaps, ADKObjects, ADKSprites, ADKParticules,ADKWalkers ,ADKIniLoader,
  ADKScreens,ADKImages;


type
// NB: pour des besoins de sparation total entre l'appli VCL et les units ADK
// TAction et TDirection sont redclars ici, mais cette unit fait une jointure transparente avec les autres dclaration
  TAction=(aStand,aWalk,aRun,aHit);
  TDirection=(dirN,dirNE,dirE,dirSE,dirS,dirSW,dirW,dirNW);

  TPlayerStart=procedure(Sender:TObject; PlayerID:integer; const Target:TPoint; How:TAction; var Cancel:boolean) of object;
  TPlayerMove=procedure(Sender:TObject; PlayerID:integer; const Leave,Enter:TPoint) of object;
  TPlayerStop=procedure(Sender:TObject; PlayerID:integer; How:TAction; Direction:TDirection; const Target:TPoint) of object;
  TRotateEvent=procedure(Sender:TObject; PlayerID:integer; Direction:TDirection) of object;
  TObjectClick=procedure(Sender:TObject; ObjectID:integer; Shift:TShiftState; var HitTest:boolean) of object;
  TObjectHint=procedure(Sender:TObject; ObjectID:integer; var Hint:string) of object;
  TTextRequest=procedure(Sender:TObject; Text:string) of object;
  TCellType=ADKWalkers.TOnCellType;
  TObstacle=ADKMaps.TObstacle;
  TCellCursor=procedure(Sender:TObject; Obstacle:TObstacle; var Cursor:TCursor) of object;

  TDialog=record
   Left,Top,Width,Height:integer;
   Show:boolean;
   Msg:string;
  end;

  TMainFrame = class(TFrame)
   procedure FrameResize(Sender: TObject);
  private
    { Private declarations }
   fSprites:TStringList;
   fLines:TStrings;
   fTopLine:integer;
   fOldHeight:integer;
   fMaxLines:integer;
   fDrag:boolean;
   fTimer:cardinal;
   fPaint:boolean;
   fSoundMax:cardinal;
   LastTicks:cardinal;
   LastObstacle:TObstacle;
   EOnPlayerStart:TPlayerStart;
   EOnPlayerMove:TPlayerMove;
   EOnPlayerStop:TPlayerStop;
   EOnObjectClick:TObjectClick;
   EOnObjectHint:TObjectHint;
   EOnRotate:TRotateEvent;
   EOnTextRequest:TTextRequest;
   EOnShowMessage:TTextRequest;
   EOnCellCursor:TCellCursor;
   procedure WMEraseBkGnd(var msg:TMessage); message wm_erasebkgnd;
   procedure WMPaint(var msg:TWMPaint); message wm_paint;
   procedure MMMCINotify(var msg:TMessage); message MM_MCINOTIFY;
   function GetOnCellType:TCellType;
   procedure SetOnCellType(Value:TCellType);
   function GetSoundPos:cardinal;
  protected
   Dialog:TDialog;
   GUI:TADKImageLib;
   procedure CreateWnd; override;
   procedure DestroyWnd; override;
   function ScreenToGrid(x,y:integer):TPoint;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure NewFrame;
   procedure Fog3D;
  public
    { Public declarations }
   constructor Create(AOwner:TComponent); override;
   destructor Destroy; override;
   function LoadSprites(Name:string):integer;
   procedure LoadImages(FileName:string);
   procedure LoadMap(FileName:string);
   procedure LoadDefaults;
   procedure ClearSprites;
   function AddPlayer(x,y,Sprites,PlayerID:integer):TObject;
   function AddFire(x,y,w,h,speed:integer):TObject;
   function AddSphere(x,y,diam,density:integer):TObject;
   procedure Select(Player:TObject);
   function Selected:TObject;
   procedure AddChat(Text:string; Color:integer=clLime);
   procedure PlayerSay(Player:TObject; Text:string);
   procedure MovePlayer(Player:TObject; Target:TPoint; How:TAction);
   procedure SetPlayerPos(Player:TObject; NewPos:TPoint; Direction:TDirection);
   procedure SetAction(Player:TObject; Action:TAction; Loop:boolean=False);
   procedure SetDirection(Player:TObject; Direction:TDirection);
   procedure SetSpeed(Player:TObject; Value:integer);
   procedure SetLight(Value:integer);
   procedure SetFlag(Index:integer; OnOff:boolean);
   function FramesPerSec:cardinal;
   function CanHit(Player,Target:TObject; var Direction:TDirection):boolean;
   procedure MessageBox(Msg:string);
   property OnPlayerStart:TPlayerStart read EOnPlayerStart write EOnPlayerStart;
   property OnPlayerMove:TPlayerMove   read EOnPlayerMove  write EOnPlayerMove;
   property OnPlayerStop:TPlayerStop  read EOnPlayerStop  write EOnPlayerStop;
   property OnObjectClick:TObjectClick read EOnObjectClick write EOnObjectClick;
   property OnObjectHint:TObjectHint read EOnObjectHint write EOnObjectHint;
   property OnCellType:TCellType read GetOnCellType write SetOnCellType;
   property OnRotate:TRotateEvent read EOnRotate write EOnRotate;
   property OnTextRequest:TTextRequest read EOnTextRequest write EOnTextRequest;
   property OnShowMessage:TTextRequest read EOnShowMessage write EOnShowMessage;
   property OnCellCursor:TCellCursor read EOnCellCursor write EOnCellCursor;
   property MaxLines:integer read fMaxLines write fMaxLines;
   property SoundMax:cardinal read fSoundMax;
   property SoundPos:cardinal read GetSoundPos;
  end;

var
 UseDirectX :boolean=true;
 UseDirect3D:boolean=true;
 UseSound   :boolean=true;

implementation

uses ADKData, MiniDX, DXError, ADKRender, ADKDepth;

{$R *.dfm}

var
 Frame:TMainFrame;

procedure OnWalkerStart(Walker:TWalker; const Target:TPoint; How:ADKIniLoader.TAction; var Cancel:boolean);
begin
 if Assigned(Frame) and Assigned(Frame.EOnPlayerStart) then Frame.EOnPlayerStart(Frame,Walker.Tag,Target,TAction(How),Cancel);
end;

procedure OnWalkerMove(Walker:TWalker; Const Leave,Enter:TPoint);
begin
 if Assigned(Frame) and Assigned(Frame.EOnPlayerMove) then Frame.EOnPlayerMove(Frame,Walker.Tag,Leave,Enter);
end;

procedure OnWalkerStop(Walker:TWalker; const Target:TPoint);
begin
 if Assigned(Frame) and Assigned(Frame.EOnPlayerStop) then Frame.EOnPlayerStop(Frame,Walker.Tag,TAction(Walker.Action),TDirection(Walker.Dir),Target);
end;

procedure OnWalkerRotate(Walker:TWalker; Dir:ADKSprites.TDirection);
begin
 if Assigned(Frame) and Assigned(Frame.EOnRotate) then Frame.EOnRotate(Frame,Walker.Tag,TDirection(Dir));
end;

constructor TMainFrame.Create(AOwner:TComponent);
begin
 Frame:=Self; // <!> global var
 inherited;
 InitData;
 GUI:=TADKImageLib.Create;
 GUI.LoadFromFile(StringProperty('Path','Root')+'GUI2.IML');
 fSprites:=TStringList.Create;
 fLines:=TStringList.Create;
 fMaxLines:=150;
end;

destructor TMainFrame.Destroy;
begin
 if UseSound then StopSound;
 if fTimer<>0 then timeKillEvent(fTimer);
 Frame:=nil;
 fSprites.Free;
 fLines.Free;
 FreeData;
 inherited;
end;

function TMainFrame.GetSoundPos:cardinal;
begin
 Result:=Sound.PlayPos;
end;

procedure TMainFrame.MMMCINotify(var msg:TMessage);
begin
 if UseSound then Replay;
end;

procedure TimerCallBack(ID,Msg,Sender,dw1,dw2:cardinal); stdcall;
begin
 TMainFrame(Sender).NewFrame;
end;

procedure TMainFrame.CreateWnd;
var
 s:string;
begin
 inherited;
// DirectDraw
 if UseDirectX then begin
  if UseDirect3D then
   ADKScreen:=TADKScreen3D.Create(Handle,false)
  else
   ADKScreen:=TADKScreenDX.Create(Handle,false);
 end else
  ADKScreen:=TADKScreenGDI.Create(Handle);

 if UseSound then begin
  s:=StringProperty('Path','Root')+StringProperty('Path','Music');
  fSoundMax:=StartSound(Handle,s+StringProperty('Startup','Music'));
 end;

 fOldHeight:=ClientHeight;
 fTopLine:=(2*fOldHeight div 3) div 14;
 FrameResize(nil);

 fTimer:=timeSetEvent(40,5,TimerCallback,cardinal(self),TIME_PERIODIC);
end;

procedure TMainFrame.DestroyWnd;
begin
 timeKillEvent(fTimer); fTimer:=0;
 FreeAndNil(ADKScreen);
 inherited;
end;

function TMainFrame.ScreenToGrid(x,y:integer):TPoint;
var
 u,v:integer;
begin
 dec(x,Map.Origin.x+ScrollX);
 dec(y,Map.Origin.y+ScrollY);
 u:=2*y+x;
 v:=2*y-x;
 Result.x:=u div 96;
 Result.y:=v div 96;
 if (u<0) then dec(Result.x);
 if (v<0) then dec(Result.y);
end;

function TMainFrame.GetOnCellType:TCellType;
begin
 Result:=Map.fOnCellType;
end;

procedure TMainFrame.SetOnCellType(Value:TCellType);
begin
 Map.fOnCellType:=Value;
end;

procedure TMainFrame.FrameResize(Sender: TObject);
begin
 if ADKScreen=nil then exit;
 ADKScreen.SetSize(Width,Height,WindowDepth);
 inc(fTopLine,(ClientHeight-fOldHeight) div 14);
 fOldHeight:=ClientHeight;
 Dialog.Width:=Width div 2;
 Dialog.Height:=Height div 2;
 //if 14*fTopLine>ClientHeight-4 then fTopLine:=(ClientHeight-4) div 14;
end;

function TMainFrame.LoadSprites(Name:string):integer;
begin
 Result:=fSprites.IndexOf(Name);
 if Result<0 then Result:=fSprites.AddObject(Name,GetSprites(Name));
end;

procedure TMainFrame.LoadImages(FileName:string);
begin
 ImageLib.LoadFromFile(FileName);
end;

procedure TMainFrame.LoadMap(FileName:string);
begin
 ClearSprites;
 Map.LoadFromFile(FileName);
end;

procedure TMainFrame.ClearSprites;
var
 i:integer;
begin
 for i:=ObjectList.Count-1 downto 0 do TADKObject(ObjectList[i]).Free;
// WalkerList.Clear;
end;

procedure TMainFrame.LoadDefaults;
var
 r,s:string;
begin
 r:=StringProperty('Path','Root');
 s:=StringProperty('Path','Images');
 LoadImages(r+s+StringProperty('Startup','Images'));
 s:=StringProperty('Path','Maps');
 LoadMap(r+s+StringProperty('Startup','Map'));
end;

function TMainFrame.AddPlayer(x,y,Sprites,PlayerID:integer):TObject;
begin
 Map.FixPos(x,y);
 result:=TWalker.Create(x,y,TSprites(fSprites.Objects[Sprites]),ADKSprites.DirS);
 with TWalker(Result) do begin
 { non ! pas par dfaut
  OnStart:=OnWalkerStart;
  OnMove:=OnWalkerMove;
  OnStop:=OnWalkerStop;
 }
  Tag:=PlayerID;
 end;
end;

function TMainFrame.AddFire(x,y,w,h,speed:integer):TObject;
var
 cx,cy:integer;
begin
 cx:=48*(x-y)-w div 2;
 cy:=24*(x+y)-h+HalfTile;
 Result:=TADKFire.Create(cx,cy,w,h,speed);
end;

function TMainFrame.AddSphere(x,y,diam,density:integer):TObject;
var
 cx,cy:integer;
begin
 cx:=48*(x-y)-diam;
 cy:=24*(x+y)-2*diam+HalfTile;
 Result:=TADKsphere.Create(cx,cy,diam,density);
end;

procedure TMainFrame.PlayerSay(Player:TObject; Text:string);
begin
 TWalker(Player).Text:=Text;
end;

procedure TMainFrame.MovePlayer(Player:TObject; Target:TPoint; How:TAction);
begin
 TWalker(Player).Go(Target,ADKIniLoader.TAction(How));
end;

procedure TMainFrame.SetPlayerPos(Player:TObject; NewPos:TPoint; Direction:TDirection);
begin
 Map.FixPos(NewPos.x,NewPos.y);
 with TWalker(Player) do begin
  Action:=ADKIniLoader.aStand;
  SetPos(NewPos.x,NewPos.y);
  Dir:=ADKSprites.TDirection(Direction);
 end;
end;

procedure TMainFrame.SetAction(Player:TObject; Action:TAction; Loop:boolean=False);
begin
 TWalker(Player).SetAction(ADKIniLoader.TAction(Action),Loop);
end;

procedure TMainFrame.SetDirection(Player:TObject; Direction:TDirection);
begin
 TWalker(Player).Dir:=ADKSprites.TDirection(Direction);
end;

procedure TMainFrame.SetSpeed(Player:TObject; Value:integer);
begin
 if Value<=200 then Value:=Round(2000*100/Value);
 if Value<>0 then TWalker(Player).Speed:=Value;
end;

procedure TMainFrame.SetLight(Value:integer);
begin
 DayLight:=Value;
// fLines.addObject(IntToHex(Value,8)+'+'+IntToHex(Map.Ambient,8),TObject(clWhite));
end;

procedure TMainFrame.SetFlag(Index:integer; OnOff:boolean);
var
 f,m:integer;
begin
 f:=Map.Flags;
 m:=1 shl Index;
 if OnOff then f:=f or m else f:=f and (not m);
 Map.Flags:=f;
end;

procedure TMainFrame.AddChat(Text:string; Color:integer=clLime);
begin
 fLines.AddObject(Text,TObject(Color));
end;

procedure TMainFrame.Select(Player:TObject);
begin
 if Focus<>nil then
 with Focus do begin
  OnStart:=nil;
  OnMove:=nil;
  OnStop:=nil;
  OnRotate:=nil;
 end;
 Focus:=TWalker(Player);
 with Focus do begin
  OnStart:=OnWalkerStart;
  OnMove:=OnWalkerMove;
  OnStop:=OnWalkerStop;
  OnRotate:=OnWalkerRotate;
 end;
end;

function TMainFrame.Selected:TObject;
begin
 result:=Focus;
end;

procedure TMainFrame.MouseMove(Shift: TShiftState; X, Y: Integer);
var
 p:TPoint;
 c:TCursor;
 o:TObstacle;
begin
 if not Dialog.Show then begin
  if fDrag then begin
   if (Y>2)and(Y<ClientHeight-2) then fTopLine:=Y div 14;
  end else
  if abs(y-14*fTopLine)<4 then Cursor:=crVSplit else begin
  // c'est ici qu'il faut dfinir le type de terrain et donc le curseur
   c:=crDefault;
   if Assigned(EOnCellCursor) then begin
    p:=ScreenToGrid(X,Y);
    if (p.x>=0)and(p.y>=0)and(p.x<Map.Width)and(p.y<Map.Height) then begin
     o:=Map.Cells[p.x,p.y].Obstacle;
     if o<>LastObstacle then begin
      LastObstacle:=o;
      EOnCellCursor(Self,o,c);
     end;
    end;
   end;
   if c<>Cursor then Cursor:=c;
   inherited;
  end;
 end;
 MousePos.X:=x;
 MousePos.Y:=y;
end;

procedure TMainFrame.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 HitTest:boolean;
begin
 inherited;
 if Dialog.Show then begin
  if (X>Dialog.Left)and(X<Dialog.Left+Dialog.Width)
  and(Y>Dialog.Top )and(Y<Dialog.Top +Dialog.Height) then Dialog.Show:=False;
  exit;
 end;
 // dplacement de la zone de chat
 if Cursor=crVSplit then begin
  fDrag:=True;
  exit;
 end;
 // click sur un objet cliquable
 if ImageText<>'' then begin
  if ImageRequest then begin
   if Assigned(EOnTextRequest) then EOnTextRequest(Self,ImageText);
  end else begin
   //fLines.AddObject(ImageText,TObject(clWhite));
   if Assigned(OnShowMessage) then EOnShowMessage(Self,ImageText) else ShowMessage(ImageText);
  end;
  ImageText:='';
  if ImageFlip=-1 then exit;
 end;
 if ImageFlip<>-1 then begin
  Map.Flags:=Map.Flags xor ImageFlip;
  exit;
 end;
 // afficher le Hint d'un personnage
 HitTest:=False;
 if MouseHint<>nil then begin
  if Assigned(EOnObjectClick) then EOnObjectClick(MouseHint,MouseHint.Tag,Shift,HitTest);
 end;// else -> permettre le click sur la tte"
 // cliquer pour dplacer le perso actif
 if (Focus<>nil)and(HitTest=False) then Focus.OnClick(x,y,ord(button));
end;

procedure TMainFrame.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 inherited;
 fDrag:=False;
end;

procedure TMainFrame.WMEraseBkGnd(var msg:TMessage);
begin
 msg.Result:=1;
end;

procedure TMainFrame.WMPaint(var msg:TWMPaint);
const
 k=2;
const
 delta:array[ADKSprites.TDirection,0..1] of integer=(
  (0,+2),( 1, 1),( 2,0),( 1,-1),
  (0,-2),(-1,-1),(-2,0),(-1,+1)
 );
var
  t:Cardinal;
  s:string;
  i:integer;
  w:TWalker;
  x,y:integer;
begin
 inc(FrameCount);
 t:=GetTickCount;
 if LastTicks<>0 then ADKRender.Idle(t-LastTicks);
 LastTicks:=t;

 if ADKScreen=nil then exit;

 try
  ADKScreen.Clear(0);
  MouseHint:=nil; // important !
  //--
  try

   if Focus<>nil then begin

    ADKRender.ClientRect:=Self.ClientRect;
    ADKRender.ScrollX:=Width  div 2-Focus.Left;//-48*Delta[Focus.Dir,0];
    ADKRender.ScrollY:=Height div 2-Focus.Top ;//+48*Delta[Focus.Dir,1];

    RenderMap;

    //ADKScreen.Fog;
    if ADKScreen is TADKScreen3D then Fog3D;

    if Alpha and (Focus<>nil) then begin
     if ADKScreen.Lock then
      try
       Focus.DrawAlpha;
       //Focus.DrawBorder;
       //TWalker(WalkerList[0]).DrawBorder;
      finally
       ADKScreen.UnLock;
      end;
    end;

    for i:=0 to WalkerList.Count-1 do begin
     w:=TWalker(WalkerList[i]);
     s:=w.Text;
     if s<>'' then
      ADKScreen.DrawHint(w.Left+ScrollX,w.Top-w.Height+ScrollY-7,clWhite,s);
    end;

    if (MouseHint<>nil)and Assigned(EOnObjectHint) then begin
     s:='';
     EOnObjectHint(MouseHint,MouseHint.Tag,s);
     if s<>'' then begin
      i:=MouseHint.Top-MouseHint.Height+ScrollY-7;
      if MouseHint.Text<>'' then dec(i,14);
      ADKScreen.DrawHint(MouseHint.Left+ScrollX,i,clWhite,s);
     end;
    end;

   end; // Focus

   ADKScreen.Trame(2,2+14*fTopLine,ClientWidth-4,ClientHeight-14*fTopLine-4);
   while fLines.Count>fMaxLines do fLines.Delete(0);
   ADKScreen.DrawLines(Rect(4,14*fTopLine+4,width-4,ClientHeight-4),fLines);

   if Dialog.Show then begin
    Dialog.Left:=(ADKScreen.Width-Dialog.Width) div 2;
    Dialog.Top :=(ADKScreen.Height-Dialog.Height) div 2;
    // fond
    ADKScreen.Trame(Dialog.Left,Dialog.Top,Dialog.Width,Dialog.Height);
    // cadre
    ADKScreen.Trame(Dialog.Left-2,Dialog.Top-2,2,Dialog.Height+2,$f0f000);
    ADKScreen.Trame(Dialog.Left-2,Dialog.Top+Dialog.Height,Dialog.Width+4,2,$f0f000);
    ADKScreen.Trame(Dialog.Left+Dialog.Width,Dialog.Top-2,2,Dialog.Height+2,$f0f000);
    ADKScreen.Trame(Dialog.Left,Dialog.Top-2,Dialog.Width,2,$f0f000);
    // sparation
    ADKScreen.Trame(Dialog.Left,Dialog.Top+Dialog.Height-36,Dialog.Width,2,$f0f000);
    // text
    x:=Dialog.Left+Dialog.Width-32;
    y:=Dialog.Top+Dialog.Height-32;
    GUI.Draw{Alpha}(9,ADKScreen.Pixels[x,y],ADKScreen.Pitch,1);
    ADKScreen.TextRect(Rect(Dialog.Left+5,Dialog.Top+5,Dialog.Left+Dialog.Width-10,Dialog.Top+Dialog.Height-46),clWhite,Dialog.Msg);
   end;

  finally
   ADKScreen.Unlock;
  end;

  ADKScreen.Flip;
 finally
  ValidateRect(Handle,nil);
 end;
 fPaint:=False;
end;

procedure TMainFrame.NewFrame;
begin
 if not fPaint then begin
  fPaint:=True;
  InvalidateRect(Handle,nil,False);
  //UpdateWindow(Handle);
 end;
end;

procedure TMainFrame.Fog3D;
{
  0 2 4 6
  1 3 5 7
  0 2 4 6
}
const
 kx=12;
 ky=12;
var
 w,x,y,i:integer; // Width div 10
 s:single;
 poly:array of TD3DTLVertex;
begin
// couleur ambiante
 if not (moDayLight in Map.Options) then begin
  SetLength(Poly,4);
  w:=Map.Ambient or $A0000000;
  with poly[0] do begin sx:=0;     sy:=0;      rhw:=1; color:=w; end;
  with poly[1] do begin sx:=width; sy:=0;      rhw:=1; color:=w; end;
  with poly[2] do begin sx:=0;     sy:=height; rhw:=1; color:=w; end;
  with poly[3] do begin sx:=width; sy:=height; rhw:=1; color:=w; end;
  TADKScreen3D(ADKScreen).DrawPoly(Poly);
 end;
// nombre de point sur une ligne
 w:=Width div kx+2;
// il nous faut deux fois plus de points
 SetLength(Poly,2*w);
// calcul des X qui ne bougeront jamais
 for x:=0 to w-1 do begin
  s:=(kx*x);
  with poly[2*x  ] do begin sx:=s; rhw:=1; end;
  with poly[2*x+1] do begin sx:=s; rhw:=1; end;
 end;
// calcul des lignes
 for y:=0 to (Height div ky)+1 do begin
  for x:=0 to w-1 do begin
   with poly[2*x+y and 1] do begin
    sy:=ky*y;
    if moDayLight in Map.Options then color:=DayLight else color:=128;
    for i:=0 to ObjectList.Count-1 do begin
     inc(color,TADKObject(ObjectList[i]).Light(sx,sy));
    end;
    for i:=0 to Map.LightCount-1 do begin
     inc(color,Map.LightSource(i,sx-ScrollX,sy-ScrollY));
    end;
    if color>255 then color:=$ff000000 else color:=color shl 24;
   end;
  end;
  if y>0 then TADKScreen3D(ADKScreen).DrawPoly(Poly);
 end;

end;

function TMainFrame.FramesPerSec:cardinal;
begin
 Result:=ADKRender.FramesPerSec;
end;

function TMainFrame.CanHit(Player,Target:TObject; var Direction:TDirection):boolean;
begin
 Result:=TWalker(Player).CanHit(TWalker(Target),ADKSprites.TDirection(Direction));
end;

procedure TMainFrame.MessageBox(Msg:string);
var
 Size:TSize;
begin
 // on prcalcul ici tout ce qu'il faut pour afficher ensuite la boite lors du rendu
 //Size:=ADKScreen.GetTextExtent(Msg);
 Dialog.Width:=Width div 2;//Size.cx+10;
 Dialog.Height:=Height div 2;//Size.cy+10+32;
 Dialog.Show:=True;
 Dialog.Msg:=Msg+#13#10'(message sur plusieurs lignes...'#13#10'enfin si on veut...)';
end;

end.
