{-----------------------------------------------------------------------------
 Unit Name: PathMap
 Author:    Manuel
 Purpose:   Proposer divers Interface pour le pathfnder
 History:   15 Janvier 03: Ajout d'un composant pour la gestion d'une liste
               de Tmap.
            14 Janvier 03: Cration Unit et mise en place d'une interface bas
               sur un bitmap 2 colors pour identifi les tildes.
               Couleur Blanc= Tilde Libre.
               Couleur Noir = Tilde occupe.
-----------------------------------------------------------------------------}
unit PathMap;

interface
uses windows, Classes, Contnrs, Graphics, ADKPathfinder, LoadMaps;

const
  clWalk = clwhite;
  clDontWalk = clblack;

type

  TmapFinder= Class(Tmap)
  private
    fTrueName: string;
    fId: Cardinal;
  protected
  public
    function LoadFromFile(AFileName:string):boolean; override;
    property TrueName: string read fTrueName;
    property Id: Cardinal read fID write fID;
  end;

  TonNotifyMapfinder = procedure(Sender: TObject; Value: TmapFinder) of object; //<< Notification de Tmap

  TMapsPathFind = class;

  TMapFinderList = class(TObjectlist)
  private
    FOnAdd, FOnDelete: TonNotifyMapfinder;
    FParent: TMapsPathFind;
  protected
    function GetItems(Index: Integer): TmapFinder;
    procedure SetItems(Index: Integer; MapFinder: TmapFinder);

    function Add(MapFinder: TmapFinder): Integer;
    function Remove(MapFinder: TmapFinder): Integer;
    procedure Insert(Index: Integer; MapFinder: TmapFinder);
    procedure Delete(Index: Integer);

  public
    function AddMap(const IDMap: cardinal; const MapFileName: string): integer;
    function IndexOf(MapFinder: TmapFinder): Integer;
    function GetMapByID(Id: Cardinal): TmapFinder;
    function GetMapNameByID(Id: Cardinal): string;

    procedure DeleteByID(const Id: Cardinal);
    property Parent: TMapsPathFind read FParent write FParent;
    property Items[Index: Integer]: TmapFinder read GetItems write SetItems; default;
   //Events
    property OnAdd: TonNotifyMapfinder read FOnAdd write FOnAdd; //<< Appel juste aprs l'ajout du map
    property OnDelete: TonNotifyMapfinder read FOnDelete write FOnDelete; //<< Appel juste AVANT l'effacement du map
  end;



  TMapsPathFind = class(Tcomponent)
  private
    fMaplist: TMapFinderList;
    fOnAdd, fOnDelete: TonNotifyMapfinder;
  protected
    procedure AddedMapFinder(Sender: TObject; Value: TmapFinder);
    procedure DeletedMapFinder(Sender: TObject; Value: TmapFinder);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Maplist: TMapFinderList read fMaplist;
  published
    property OnAdd: TonNotifyMapfinder read FOnAdd write FOnAdd;
    property OnDelete: TonNotifyMapfinder read FOnDelete write FOnDelete;
  end;

procedure Register;


implementation
uses SysUtils, StrStuff;


procedure Register;
begin
  RegisterComponents('MiniChat', [TMapsPathFind]);
end;

{TMapFinder}

function TMapFinder.LoadFromFile(AFileName:string):boolean;
begin
  result:= inherited LoadFromFile(AFileName);
  fTrueName := extractFilename(AFileName);
  fTrueName := extractword(1, fTrueName, ['.']);
end;

{ TMapFinderList }

function TMapFinderList.Add(MapFinder: TmapFinder): Integer;
begin
  Result := inherited Add(MapFinder);
  if assigned(FOnAdd) then FOnAdd(self, MapFinder);
end;

function TMapFinderList.AddMap(const IDMap: cardinal; const MapFileName: string): integer;
var MapFinder: TmapFinder;
begin
  MapFinder := TmapFinder.Create;
  MapFinder.LoadFromFile(MapFileName);
  MapFinder.Id := IDMap;
  result := Add(MapFinder);
end;

procedure TMapFinderList.Delete(Index: Integer);
begin
  if assigned(FOnDelete) then FOnDelete(self, Items[Index]);
  inherited;
end;

procedure TMapFinderList.DeleteByID(const Id: Cardinal);
var i: integer;
begin
  for i := 0 to Count - 1 do
    if ID = Items[i].ID then
    begin
      Delete(i);
      exit;
    end;
end;

function TMapFinderList.GetMapByID(Id: Cardinal): TmapFinder;
var i: integer;
begin
  result := nil;
  for i := 0 to Count - 1 do
    if ID = Items[i].ID then
    begin
      result := TmapFinder(inherited Items[i]);
      exit;
    end;
end;

function TMapFinderList.GetMapNameByID(Id: Cardinal): string;
var i: integer;
begin
  result := '';
  for i := 0 to Count - 1 do
    if ID = Items[i].ID then
    begin
      result := TmapFinder(inherited Items[i]).Truename;
      exit;
    end;
end;

function TMapFinderList.GetItems(Index: Integer): TmapFinder;
begin
  Result := TmapFinder(inherited Items[Index]);
end;

function TMapFinderList.IndexOf(MapFinder: TmapFinder): Integer;
begin
  Result := inherited IndexOf(MapFinder);
end;

procedure TMapFinderList.Insert(Index: Integer; MapFinder: TmapFinder);
begin
  inherited Insert(Index, MapFinder);
  if assigned(FOnAdd) then FOnAdd(self, MapFinder);
end;

function TMapFinderList.Remove(MapFinder: TmapFinder): Integer;
begin
  if assigned(FOnDelete) then FOnDelete(self, MapFinder);
  Result := inherited Remove(MapFinder);
end;

procedure TMapFinderList.SetItems(Index: Integer; MapFinder: TmapFinder);
begin
  inherited Items[Index] := MapFinder;
  if assigned(FOnAdd) then FOnAdd(self, MapFinder);
end;


{TMapsPathFind}

constructor TMapsPathFind.Create(AOwner: TComponent);
begin
  inherited;
  fMaplist := TMapFinderList.create;
  fMaplist.OnAdd := AddedMapFinder;
  fMaplist.OnDelete := DeletedMapFinder;
  fMaplist.FParent := self;
end;

destructor TMapsPathFind.Destroy;
begin
  fMaplist.destroy;
  inherited;
end;

procedure TMapsPathFind.AddedMapFinder(Sender: TObject; Value: TmapFinder);
begin
  if assigned(fOnAdd) then fOnAdd(self, Value);
end;

procedure TMapsPathFind.DeletedMapFinder(Sender: TObject; Value: TmapFinder);
begin
  if assigned(fOnDelete) then fOnDelete(self, Value);
end;

end.
