unit ADKImages;

{
 Bibliothque d'images pour le projet ADK-ISO (c)2003 Paul TOTH <tothpaul@free.fr>

 http://www.web-synergy.net/naug-land/

}

{
  Format des fichiers IML

  [Header              ]
  [Donnes compresses ](array of data)
  [Tableau des images  ](array of TImage)
  [Palettes de couleurs](array of TPalette)

}

interface

uses
 Windows,Graphics,Classes,SysUtils,ADKCompress,ADKDepth,ADKScreens;

const
 LIB_SIGNATURE:array[0..3] of char='IMGL';
 LIB_VERSION  :Cardinal=102;

// Historique des versions
// 101 - premire version
// 102 - packed records, dplacement de la compression dans ADKCompress

type
 THeader=packed record
  Signature  :cardinal; // LIB_SIGNATURE
  Version    :cardinal; // LIB_VERSION
  Images     :cardinal; // nombre d'images (8bit)
  ImagesOfs  :cardinal; // position du tableau de TImage
  Palettes   :cardinal; // nombre de palettes (256 couleurs)
  PalettesOfs:cardinal; // position du tableau de TPalette
 end;

 TADKImage=packed record
  Palette    :Cardinal; // numro de la palette
  Transparent:SmallInt; // couleur de transparence (ou -1)
  Width      :integer;  // Largeur de l'image (sans alignement)
  Height     :integer;  // Hauteur de l'image
  Pixels     :Cardinal; // position des pixels (ou 0 si image supprime)
 end;
 TADKImages=array[word] of TADKImage;

// TPalette=ADKCompress.TPalette;
// TPalettes=array[word] of TPalette32;
// TPaletteW=array[#0..#255] of word;

 TBlendW=function(a,b:word):word;

 TADKImageLib=class(TADKCompressedDIB)
 private
  fFileName:string;       // nom du fichier
  fReadOnly:boolean;      // mode d'accs au fichier
  fFile    :integer;      // Handle du fichier
  fSize    :Cardinal;     // Taille du fichier
  fMap     :integer;      // le FileMapping !
  fBase    :pchar;        // pointer sur le fichier sur "disque mapp en mmoire"
  fHeader  :^THeader;
  fImages  :^TADKImages;
  fPal32   :^TPalettes32;
  fPal15   :array of TPalette16;
  fPal16   :array of TPalette16;
  fMaskPal :TPalette32;
  procedure MapFile;
  procedure UnMapFile;
  function GetCount:integer;
  function GetPaletteCount:integer;
  function GetSize(Index:Cardinal):TPoint;
 protected
  dta:pchar;
  len:cardinal;
  rgb:array of array[#0..#255] of word;
  blendW:TBlendW;
  function getLen:integer;
  function getPal(index:integer; mask:integer):pointer;
  function EndOfData(index:Cardinal):Cardinal;
 public
  destructor Destroy; override;
  procedure LoadFromFile(AFileName:string; AReadOnly:boolean=True);
  procedure Close;
//  procedure SetDepth(Depth:integer);
  function FindPalette(const Palette:TPalette32):integer;
  function AddPalette(const Palette:TPalette32):integer;
  function AddImage(DIB:pointer; AWidth,AHeight,APitch,APalette,ATransparent:integer):integer;
  function AddCompressedImage(const Image:TADKImage; Pixels:pointer; Size:integer):integer;
  procedure GetImage(Index:integer;var Image:TADKImage;var Palette:PPalette32; var Pixels:pointer; var Size:integer);
  procedure DelImage(Index:Cardinal);
  procedure ReplaceImage(Index:Cardinal; DIB:pointer; AWidth,AHeight,APitch,APalette,ATransparent:integer);
  function Deleted(Index:integer):boolean;
  function GetBitmap(Image:cardinal):TBitmap;
  procedure Draw(Image:cardinal; target:pointer; Pitch,Mirror:integer; Mask:integer=0);
  procedure DrawRect(Image:cardinal; target:pointer; Pitch,Mirror:integer; src:TRect; Mask:integer=0);
  procedure DrawAlpha(Image:cardinal; target:pointer; Pitch,Mirror:integer);
  procedure DrawAlphaRect(Image:cardinal; target:pointer; Pitch,Mirror:integer; src:TRect);
  function Empty:boolean;
  property FileName:string read fFileName;
  property Count:integer read GetCount;
  property PaletteCount:integer read GetPaletteCount;
  property Size[index:Cardinal]:TPoint read GetSize;
 end;

procedure CreateEmptyLib(AFileName:string);

implementation

function blend(a,b:cardinal):cardinal;
begin
 result:=((a and $FEFEFE)+(b and $FEFEFE)) shr 1;
end;

function cvt15(i:integer):word; // 3 * 5 bits
begin                           // 11111000.11111000.11111000
 result:=(i and $F80000) shr 9
        +(i and $00F800) shr 6
        +(i and $0000F8) shr 3;
end;

function blend15(a,b:word):word;
begin
 result:=((a and $7BDE)+(b and $7BDE)) shr 1;
end;

function cvt16(i:integer):word; // 2 * 5 bits + 6 bits
begin                           // 11111000.11111100.11111000
 result:=(i and $F80000) shr 8
        +(i and $00FC00) shr 5
        +(i and $0000F8) shr 3;
end;

function blend16(a,b:word):word;
begin
 result:=(a and $F7DE) shr 1+(b and $F7DE) shr 1;
end;

function SamePalette(const a,b:TPalette32):boolean;
var
 x:integer;
begin
 for x:=0 to 255 do begin
  if (a[x] and $FFFFFF)<>(b[x] and $FFFFFF) then begin
   Result:=False;
   exit;
  end;
 end;
 Result:=True;
end;

procedure CreateEmptyLib(AFileName:string);
var
 f:file;
 h:THeader;
begin
 h.Signature:=cardinal(LIB_SIGNATURE);
 h.Version:=cardinal(LIB_VERSION);
 h.Images:=0;
 h.ImagesOfs:=SizeOf(h);
 h.Palettes:=0;
 h.PalettesOfs:=SizeOf(h);
 Assign(f,AFileName);
 Rewrite(f,1);
 BlockWrite(f,h,sizeof(h));
 Close(f);
end;

destructor TADKImageLib.Destroy;
begin
 close;
 inherited;
end;

procedure TADKImageLib.LoadFromFile(AFileName:string; AReadOnly:boolean=True);
const
 OpenMode:array[false..true] of Cardinal=(OPEN_ALWAYS,OPEN_EXISTING);
 FileMode:array[false..true] of Cardinal=(GENERIC_READ or GENERIC_WRITE,GENERIC_READ);
begin
 Close;
 fFileName:=AFileName;
 fReadOnly:=AReadOnly;
 fFile:=CreateFile(PChar(fFileName),FileMode[fReadOnly],FILE_SHARE_READ,nil,OpenMode[fReadOnly],FILE_ATTRIBUTE_NORMAL,0);
 try
  if fFile=0 then raise Exception.Create('Impossible d''ouvrir '+fFileName);
  MapFile;
 except
  Close;
  raise;
 end;
end;

procedure TADKImageLib.Close;
begin
 UnMapFile;
 CloseHandle(fFile);
 fFile:=0;
end;

function TADKImageLib.Empty:boolean;
begin
 Result:=(fFile=0)or(fHeader.Images=0);
end;

(*
procedure TADKImageLib.SetDepth(Depth:integer);
type
 TCvt=function(i:integer):word;
var
 i,j:integer;
 cvt:TCvt;
begin
 case Depth of
  15: begin cvt:=@cvt15; blendW:=@blend15; end;
  16: begin cvt:=@cvt16; blendW:=@blend16; end;
  else exit;
 end;
 SetLength(rgb,fHeader.Palettes);
 for i:=0 to fHeader.Palettes-1 do begin
  for j:=0 to 255 do begin
   rgb[i,chr(j)]:=cvt(fPalettes[i,j]);
  end;
 end;
end;
*)
procedure TADKImageLib.MapFile;
const
 PageMode:array[false..true] of integer=(PAGE_READWRITE,PAGE_READONLY);
 MapMode :array[false..true] of integer=(FILE_MAP_ALL_ACCESS,FILE_MAP_READ);
begin
 fSize:=GetFileSize(fFile,nil);
 fMap :=CreateFileMapping(fFile,nil,PageMode[fReadOnly],0,0,nil);
 if fMap=0 then raise Exception.Create('Impossible de lire le fichier  '+fFileName);
 fBase:=MapViewOfFile(fMap,MapMode[fReadOnly],0,0,0);
 if fBase=nil then raise Exception.Create('Problme d''accs au fichier '+fFileName);
 fHeader:=@fBase[0];
 if fHeader.Signature<>Cardinal(LIB_SIGNATURE) then raise Exception.Create('Le fichier contenant les images est invalide');
 if fHeader.Version<LIB_VERSION then raise Exception.Create('Le fichier contenant les images est trop ancien');
 if fHeader.Version>LIB_VERSION then raise Exception.Create('Le fichier contenant les images est trop rcent');
 fImages:=@fBase[fHeader.ImagesOfs];
 fPal32:=@fBase[fHeader.PalettesOfs];
 (*
{$IFDEF DEPTH32}
{$ELSE}
 GetMem(fPalettes,fHeader.Palettes*SizeOf(TPalette));
 Palette32:=@fBase[fHeader.PalettesOfs];
 for i:=0 to fHeader.Palettes-1 do begin
  for j:=0 to 255 do begin
   fPalettes[i,j]:=cvt(Palette32[i,j]);
  end;
 end;
{$ENDIF}
*)
end;

procedure TADKImageLib.UnMapFile;
begin
 UnMapViewOfFile(fBase);
 CloseHandle(fMap);
end;

function TADKImageLib.GetCount:integer;
begin
 Result:=fHeader.Images;
end;

function TADKImageLib.GetPaletteCount:integer;
begin
 Result:=fHeader.Palettes;
end;

function TADKImageLib.GetSize(Index:Cardinal):TPoint;
begin
 if Index>=fHeader.Images then begin
  Result.X:=0;
  Result.Y:=0;
 end else
 with fImages[Index] do begin
  Result.X:=Width;
  Result.Y:=Height;
 end;
end;

function TADKImageLib.FindPalette(const Palette:TPalette32):integer;
var
 i:integer;
begin
 for i:=0 to fHeader.Palettes-1 do begin
  if SamePalette(Palette,fPal32[i]) then begin
   result:=i;
   exit;
  end;
 end;
 Result:=-1;
end;

function TADKImageLib.AddPalette(const Palette:TPalette32):integer;
var
 i:cardinal;
begin
// vrifier que la palette n'existe pas dj
 Result:=FindPalette(Palette);
 if Result<0 then begin
  UnMapFile;
  SetFilePointer(fFile,0,nil,FILE_END);
  WriteFile(fFile,Palette,SizeOf(Palette),i,nil);
  MapFile;
  Result:=fHeader.Palettes;
  inc(fHeader.Palettes);
 end;
end;

function TADKImageLib.AddImage(DIB:pointer; AWidth,AHeight,APitch,APalette,ATransparent:integer):integer;
var
 Temp:TMemoryStream;
 Size:Cardinal;
 i,x,p:Cardinal;
begin
 Temp:=TMemoryStream.Create;
 try
  CompressDIB(DIB,AWidth,AHeight,-APitch,ATransparent,Temp);
  Size:=Temp.Size;
  // rserver de la place pour l'image
  UnMapFile;
  SetFilePointer(fFile,Size+SizeOf(TADKImage),nil,FILE_END);
  SetEndOfFile(fFile);
  MapFile;
  // dplacer les palettes
  i:=fHeader.PalettesOfs;
  p:=i+fHeader.Palettes*SizeOf(TPalette32);
  inc(i,Size+SizeOf(TADKImage));
  fHeader.PalettesOfs:=i;
  fPal32:=@fBase[i];
  if fHeader.Palettes>0 then
   for x:=fHeader.Palettes-1 downto 0 do begin
    dec(p,SizeOf(TPalette32));
    move(fBase[p],fPal32[x],SizeOf(TPalette32));
   end;
  // dplacer les Images
  i:=fHeader.ImagesOfs;
  p:=i+fHeader.Images*SizeOf(TADKImage);
  inc(i,Size);
  fHeader.ImagesOfs:=i;
  fImages:=@fBase[i];
  if fHeader.Images>0 then // Warning ! Images is a Cardinal, Images-1 is NEVER "-1" !
   for x:=fHeader.Images-1 downto 0 do begin
    dec(p,SizeOf(TADKImage));
    move(fBase[p],fImages[x],SizeOf(TADKImage));
   end;
  dec(i,Size);
  // Ajouter l'image
  Move(Temp.Memory^,fBase[i],Size);
  Temp.Clear;
  Result:=fHeader.Images;
  inc(fHeader.Images);
  with fImages[Result] do begin
   Palette:=APalette;
   Transparent:=ATransparent;
   Width:=AWidth;
   Height:=AHeight;
   Pixels:=i;
  end;
 finally
  Temp.Free;
 end;
end;

function TADKImageLib.AddCompressedImage(const Image:TADKImage; Pixels:pointer; Size:integer):integer;
var
 i,x,p:Cardinal;
begin
 // rserver de la place pour l'image
 UnMapFile;
 SetFilePointer(fFile,Size+SizeOf(TADKImage),nil,FILE_END);
 SetEndOfFile(fFile);
 MapFile;
 // dplacer les palettes
 i:=fHeader.PalettesOfs;
 p:=i+fHeader.Palettes*SizeOf(TPalette32);
 inc(i,Size+SizeOf(TADKImage));
 fHeader.PalettesOfs:=i;
 fPal32:=@fBase[i];
 if fHeader.Palettes>0 then
  for x:=fHeader.Palettes-1 downto 0 do begin
   dec(p,SizeOf(TPalette32));
   move(fBase[p],fPal32[x],SizeOf(TPalette32));
  end;
 // dplacer les Images
 i:=fHeader.ImagesOfs;
 p:=i+fHeader.Images*SizeOf(TADKImage);
 inc(i,Size);
 fHeader.ImagesOfs:=i;
 fImages:=@fBase[i];
 if fHeader.Images>0 then // Warning ! Images is a Cardinal, Images-1 is NEVER "-1" !
  for x:=fHeader.Images-1 downto 0 do begin
   dec(p,SizeOf(TADKImage));
   move(fBase[p],fImages[x],SizeOf(TADKImage));
  end;
 dec(i,Size);
 // Ajouter l'image
 Move(Pixels^,fBase[i],Size);
 Result:=fHeader.Images;
 inc(fHeader.Images);
 fImages[Result]:=Image;
 fImages[Result].Pixels:=i;
end;

procedure TADKImageLib.GetImage(Index:integer;var Image:TADKImage;var Palette:PPalette32; var Pixels:pointer; var Size:integer);
begin
 Image:=fImages[Index];
 Palette:=@fPal32[Image.Palette];
 Pixels:=@fBase[Image.Pixels];
 Size:=EndOfData(Index)-Image.Pixels;
end;

function TADKImageLib.EndOfData(index:Cardinal):Cardinal;
begin
 if Index=fHeader.Images-1 then begin
  result:=fHeader.PalettesOfs;
 end else begin
  Result:=fImages[index+1].Pixels;
  if result=0 then result:=EndOfData(index+1);
 end;
end;


procedure TADKImageLib.DelImage(Index:Cardinal);
var
 NewName:string;
 NewHeader:THeader;
 NewLib:TFileStream;
 NewImages:array of TADKImage;
 i:integer;
begin
//- raise exception.create('procdure bugge, il faut ajuster Images[x].Pixels pour x>Index !');
 if (Index>=fHeader.Images)or(fImages[Index].Pixels=0) then exit;
 NewName:=fFileName+'.NEW';
 NewLib:=TFileStream.Create(NewName,fmCreate);
 try
 try
 // entte
  NewHeader:=fHeader^;
  NewLib.WriteBuffer(NewHeader,SizeOf(THeader));
 // les donnes
  SetLength(NewImages,fHeader.Images);
  for i:=0 to fHeader.Images-1 do begin
   NewImages[i]:=fImages[i];
  end;
  NewImages[Index].Pixels:=0;
  for i:=0 to fHeader.Images-1 do begin
   if NewImages[i].Pixels<>0 then begin
    NewImages[i].Pixels:=NewLib.Position;
    NewLib.WriteBuffer(fBase[fImages[i].Pixels],EndOfData(i)-fImages[i].Pixels);
   end;
  end;
 // le tableau d'images
  NewHeader.ImagesOfs:=NewLib.Position;
  NewLib.WriteBuffer(NewImages[0],Length(NewImages)*SizeOf(TADKImage));
 // les palettes
  NewHeader.PalettesOfs:=NewLib.Position;
  NewLib.WriteBuffer(fPal32^,fHeader.Palettes*SizeOf(TPalette32));
 // mise  jour de l'entte
  NewLib.Position:=0;
  NewLib.WriteBuffer(NewHeader,SizeOf(THeader));
 finally
  NewLib.Free;
 end;
  Close;
  DeleteFile(fFileName+'.OLD');
  RenameFile(fFileName,fFileName+'.OLD');
  RenameFile(NewName,fFileName);
  LoadFromFile(fFileName,fReadOnly);
 except
  NewLib.Free;
  DeleteFile(NewName);
 end;
end;

procedure TADKImageLib.ReplaceImage(Index:cardinal; DIB:pointer; AWidth,AHeight,APitch,APalette,ATransparent:integer);
var
 Temp:TMemoryStream;
 Size:cardinal;
 NewName:string;
 NewHeader:THeader;
 NewLib:TFileStream;
 NewImages:array of TADKImage;
 i:integer;
begin
 if (Index>=fHeader.Images)or(fImages[Index].Pixels=0) then exit;
 NewName:=fFileName+'.NEW';
 NewLib:=TFileStream.Create(NewName,fmCreate);
 try
 Temp:=TMemoryStream.Create;
 try
  CompressDIB(DIB,AWidth,AHeight,-APitch,ATransparent,Temp);
  Size:=Temp.Size;
 try
 // entte
  NewHeader:=fHeader^;
  NewLib.WriteBuffer(NewHeader,SizeOf(THeader));
 // les donnes
  SetLength(NewImages,fHeader.Images);
  for i:=0 to fHeader.Images-1 do begin
   NewImages[i]:=fImages[i];
  end;
  for i:=0 to fHeader.Images-1 do begin
   if NewImages[i].Pixels<>0 then begin
    NewImages[i].Pixels:=NewLib.Position;
    if cardinal(i)=Index then begin
     NewImages[i].Palette:=APalette;
     NewImages[i].Transparent:=ATransparent;
     NewImages[i].Width:=AWidth;
     NewImages[i].Height:=AHeight;
     NewLib.WriteBuffer(Temp.Memory^,Size);
    end else
     NewLib.WriteBuffer(fBase[fImages[i].Pixels],EndOfData(i)-fImages[i].Pixels);
   end;
  end;
 // le tableau d'images
  NewHeader.ImagesOfs:=NewLib.Position;
  NewLib.WriteBuffer(NewImages[0],Length(NewImages)*SizeOf(TADKImage));
 // les palettes
  NewHeader.PalettesOfs:=NewLib.Position;
  NewLib.WriteBuffer(fPal32^,fHeader.Palettes*SizeOf(TPalette32));
 // mise  jour de l'entte
  NewLib.Position:=0;
  NewLib.WriteBuffer(NewHeader,SizeOf(THeader));
 finally
  Temp.Free;
 end;
 finally
  NewLib.Free;
 end;
  Close;
  DeleteFile(fFileName+'.OLD');
  RenameFile(fFileName,fFileName+'.OLD');
  RenameFile(NewName,fFileName);
  LoadFromFile(fFileName,fReadOnly);
 except
  NewLib.Free;
  DeleteFile(NewName);
 end;
end;
(*
var
 Temp:TMemoryStream;
 Size:integer;
 OldSize:integer;
 Delta:integer;
 i,x,p:Cardinal;
begin
 Temp:=TMemoryStream.Create;
 try

  CompressDIB(DIB,AWidth,AHeight,-APitch,ATransparent,Temp);
  Size:=Temp.Size;

  // ancienne image
  OldSize:=fImages[Index].Pixels;
  if OldSize<>0 then OldSize:=EndOfData(Index)-OldSize;
  Delta:=Size-OldSize;

  // cas trivial, les images ont la mme taille
  if Delta=0 then begin
   Move(Temp.Memory^,fBase[fImages[Index].Pixels],Size);
  end else
  // la nouvelle image est plus grosse que la prcdente
  if Delta>0 then begin
   UnMapFile;
   SetFilePointer(fFile,Delta,nil,FILE_END);
   SetEndOfFile(fFile);
   MapFile;
   // dplacer les palettes
   i:=fHeader.PalettesOfs;
   inc(i,Delta);
   fHeader.PalettesOfs:=i;
   fPal32:=@fBase[i];
   p:=fHeader.Palettes*SizeOf(TPalette32);
   inc(i,p);
   for x:=1 to p do begin
    dec(i);
    fBase[i]:=fBase[i-Delta];
   end;
   // dplacer les Images
   i:=fHeader.ImagesOfs;
   inc(i,Delta);
   fHeader.ImagesOfs:=i;
   fImages:=@fBase[i];
   p:=fHeader.Images*SizeOf(TADKImage);
   inc(i,p);
   for x:=1 to p do begin
    dec(i);
    fBase[i]:=fBase[i-Delta];
   end;
   // dplacer les donnes
   for x:=fHeader.ImagesOfs-1 downto fImages[Index].Pixels+OldSize do begin
    fBase[x]:=fBase[x-Delta];
   end;
   for x:=Index+1 to fHeader.Images-1 do begin
    with fImages[x] do if Pixels<>0 then inc(Pixels,Delta);
   end;
   Move(Temp.Memory^,fBase[fImages[Index].Pixels],Size);
  end else
  // la nouvelle image est plus petite que la prcdente
  if Delta<0 then begin
   // nouvelle image
   Move(Temp.Memory^,fBase[fImages[Index].Pixels],Size);
   // dplacer les donnes
   Delta:=-Delta; // ne nous trompons pas SVP
   for x:=fImages[Index].Pixels+Size to fHeader.ImagesOfs-1 do begin
    fBase[x]:=fBase[x+Delta];
   end;
   for x:=Index+1 to fHeader.Images-1 do begin
    with fImages[x] do if Pixels<>0 then dec(Pixels,Delta);
   end;
   // dplacer les Images
   i:=fHeader.ImagesOfs;
   dec(i,Delta);
   fHeader.ImagesOfs:=i;
   fImages:=@fBase[i];
   for x:=1 to fHeader.Images*SizeOf(TADKImage) do begin
    fBase[i]:=fBase[i+Delta];
    inc(i);
   end;
   // dplacer les palettes
   i:=fHeader.PalettesOfs;
   dec(i,Delta);
   fHeader.PalettesOfs:=i;
   for x:=1 to fHeader.Palettes*SizeOf(TPalette32) do begin
    fBase[i]:=fBase[i+Delta];
    inc(i);
   end;
   // fixer la fin de fichier
   UnMapFile;
   SetFilePointer(fFile,i,nil,FILE_BEGIN);
   SetEndOfFile(fFile);
   MapFile;
  end;

  fImages[Index].Palette:=APalette;
  fImages[Index].Transparent:=ATransparent;
  fImages[Index].Width:=AWidth;
  fImages[Index].Height:=AHeight;

 finally
  Temp.Free;
 end;
*)

function TADKImageLib.Deleted(Index:integer):boolean;
begin
 result:=fImages[Index].Pixels=0;
end;

function TADKImageLib.GetLen:integer;
begin
 result:=0;
 while dta^>#127 do begin
  inc(result,ord(dta^) and 127);
  result:=result shl 7;
  inc(dta);
  dec(len);
 end;
 inc(Result,ord(dta^));
 inc(dta);
 dec(len);
end;

function TADKImageLib.GetPal(index:integer; mask:integer):pointer;
var
 i,j:integer;
 pw:^word;
 pc:^cardinal;
begin
 case ADKScreen.Depth of
  15: begin
       fPal16:=nil;
       if fPal15=nil then begin
        SetLength(fPal15,fHeader.Palettes);
        for i:=0 to fHeader.Palettes-1 do begin
         for j:=0 to 255 do fPal15[i,j]:=cvt15(fPal32[i,j]);
        end;
       end;
       Result:=@fPal15[Index];
      end;
  16: begin
       fPal15:=nil;
       if fPal16=nil then begin
        SetLength(fPal16,fHeader.Palettes);
        for i:=0 to fHeader.Palettes-1 do begin
         for j:=0 to 255 do fPal16[i,j]:=cvt16(fPal32[i,j]);
        end;
       end;
       Result:=@fPal16[Index];
      end;
  32: begin
       fPal15:=nil;
       fPal16:=nil;
       Result:=@fPal32[Index];
      end;
  else begin
   Result:=nil;
   exit;
  end;
 end;
 if mask<>0 then begin
  move(Result^,fMaskPal,256*ADKScreen.BPP);
  Result:=@fMaskPal;
  if ADKScreen.Depth=32 then begin
   pc:=Result;
   for i:=0 to 255 do begin pc^:=pc^ and Mask; inc(pc); end;
  end else begin
   pw:=Result;
   for i:=0 to 255 do begin pw^:=pw^ and Mask; inc(pw); end;
  end;
 end;
end;

function TADKImageLib.GetBitmap(Image:cardinal):TBitmap;
type
 TBMP=packed record
  bf:TBitmapFileHeader;
  bi:TBitmapInfoHeader;
  cl:array[0..255] of integer;
 end;
var
 mem:TMemoryStream;
 dib:pchar;
 w,h:integer;
 ww :integer;
 bmp:^TBMP;
begin
 Result:=nil;
 if fHeader=nil then exit;
 if Image>=fHeader.Images then exit;
 mem:=TMemoryStream.Create;
 try

  New(bmp);
  try
   bmp.bf.bfType:=Ord('B') + Ord('M')*$100;
   w:=fImages[Image].Width;
   h:=fImages[Image].Height;
   ww:=(w+3) and (not 3);
   bmp.bf.bfSize:=SizeOf(bmp)+ww*h;
   bmp.bf.bfReserved1:=0;
   bmp.bf.bfReserved2:=0;
   bmp.bf.bfOffBits:=SizeOf(bmp);
   bmp.bi.biSize:=SizeOf(bmp.bi);
   bmp.bi.biWidth:=w;
   bmp.bi.biHeight:=h;
   bmp.bi.biPlanes:=1;
   bmp.bi.biBitCount:=8;
   bmp.bi.biCompression:=BI_RGB;
   bmp.bi.biSizeImage:=ww*h;
   bmp.bi.biXPelsPerMeter:=3790;
   bmp.bi.biYPelsPerMeter:=3800;
   bmp.bi.biClrUsed:=256;
   bmp.bi.biClrImportant:=0;
   Move(fPal32[fImages[Image].Palette],bmp.cl,SizeOf(bmp.cl));
   mem.WriteBuffer(bmp^,SizeOf(bmp^));
  finally
   Dispose(bmp);
  end;

  GetMem(dib,ww*h);
  FillChar(dib^,ww*h,fImages[Image].Transparent);
  try
   Draw8(
    @fBase[fImages[Image].Pixels],
    @dib[ww*(h-1)],
    h,
    -ww,
    1
   );
   mem.WriteBuffer(dib[0],ww*h);
  finally
   FreeMem(dib);
  end;
  //mem.Position:=0;
  //mem.SaveToFile('c:\windows\bureau\test.bmp');
  Result:=TBitmap.Create;
  mem.Position:=0;
  Result.LoadFromStream(mem);
 finally
  mem.Free;
 end;
end;

procedure TADKImageLib.Draw(Image:cardinal; target:pointer; Pitch,Mirror:integer; Mask:integer=0);
var
 ofs:cardinal;
begin
 if fHeader=nil then exit;
 if Image>=fHeader.Images then exit;
 ofs:=fImages[Image].Pixels;
 if ofs=0 then exit;
 XDraw(
  @fBase[ofs],
  target,
  fImages[Image].Height,
  Pitch,
  Mirror,
  GetPal(fImages[Image].Palette,Mask)^
 );
end;

procedure TADKImageLib.DrawRect(Image:cardinal; target:pointer; Pitch,Mirror:integer; src:TRect; Mask:integer=0);
var
 w,h:integer;
 ofs:cardinal;
begin
 if Image>=fHeader.Images then exit;
 w:=fImages[Image].Width;
 if src.left>w then exit;
 h:=fImages[Image].Height;
 if src.top>h then exit;
 // <!> src.Right & src.Bottom are used as Width & Height !!!
 if src.right>=w then src.right:=w-1;
 if src.bottom>=h then src.bottom:=h-1;

 ofs:=fImages[Image].Pixels;
 if ofs=0 then exit;
 XDrawRect(
  @fBase[ofs],
  target,
  Pitch,
  Mirror,
  src,
  GetPal(fImages[Image].Palette,Mask)^
 );
end;

procedure TADKImageLib.DrawAlpha(Image:cardinal; target:pointer; Pitch,Mirror:integer);
begin
 if Image>=fHeader.Images then exit;
 XDrawBlend(@fBase[fImages[Image].Pixels],target,fImages[Image].Height,Pitch,Mirror,GetPal(fImages[Image].Palette,0)^);
end;

procedure TADKImageLib.DrawAlphaRect(Image:cardinal; target:pointer; Pitch,Mirror:integer; src:TRect);
var
 w,h:integer;
begin
 if Image>=fHeader.Images then exit;
 w:=fImages[Image].Width;
 if src.left>w then exit;
 h:=fImages[Image].Height;
 if src.top>h then exit;
 if src.right>=w then src.right:=w-1;
 if src.bottom>=h then src.bottom:=h-1;

 XDrawBlendRect(
  @fBase[fImages[Image].Pixels],
  target,
  Pitch,
  Mirror,
  src,
  GetPal(fImages[Image].Palette,0)^
 );
end;

end.
