unit ADKMaps;

{
 projet ADK-ISO (c)2002-2003 Paul TOTH <tothpaul@free.fr>

 http://www.web-synergy.net/naug-land/

}

interface

uses
 Windows,SysUtils;

const
 MAP_SIGNATURE:array[0..3] of char='MAPF';
 MAP_VERSION =107;

// Version history :
// 101 - first release, only Background Images
// 102 - added Cells
// 103 - added BaseLine
// 104 - Origin
// 105 - restructuration
// 106 - restructuration (compatibilit 105 assure)
// 107 - ajout des sources de lumire

type
 TArrayInfo=record
  Count :cardinal;
  Offset:cardinal;
 end;

 TStringInfo=packed record
  ObjectID:integer;
  Offset  :integer;
  Length  :integer;
 end;
 TStringInfos=array[word] of TStringInfo;
 PStringInfos=^TStringInfos;

 THeader105=record
 // id
  Sign      :cardinal;
  Ver       :cardinal;
 // map info
  Width     :integer;
  Height    :integer;
  Origin    :TPoint;
 // array info
  Arrays    :TArrayInfo;
  Images    :TArrayInfo;
  Items     :TArrayInfo;
 end;

 TMapOption =(moDayLight);
 TMapOptions=set of TMapOption;

 THeader106=packed record
 // id
  Signature :cardinal;
  Version   :cardinal;
 // map info
  Width     :integer;
  Height    :integer;
  Origin    :TPoint;
  Flags     :integer; // Valeur initiale des indicateurs
  Ambient   :integer; // couleur ambiente
  Options   :TMapOptions;
 // array info
  Arrays    :TArrayInfo;
  Images    :TArrayInfo;
  Items     :TArrayInfo;
  Texts     :TArrayInfo; // StringList (ifTextPanel)
  FlagNames :TArrayInfo; // StringList
 end;

 THeader107=packed record
 // id
  Signature :cardinal;
  Version   :cardinal;
 // map info
  Width     :integer;
  Height    :integer;
  Origin    :TPoint;
  Flags     :integer; // Valeur initiale des indicateurs
  Ambient   :integer; // couleur ambiente
  Options   :TMapOptions;
 // array info
  Arrays    :TArrayInfo;
  Images    :TArrayInfo;
  Items     :TArrayInfo;
  Texts     :TArrayInfo; // StringList (ifTextPanel)
  FlagNames :TArrayInfo; // StringList
  Lights    :TArrayInfo;
 end;

 TImageFlag =(
  ifSwap,        // retournement horizontal
  ifFlip,        // retournement vertical
  ifBlend,       // affichage en alphablending  50%
  ifTextPanel,   // dsigne un panneau de texte - v106 (lookup de l'index dans Texts->TStringInfo)
  ifRequestText, // demande le texte au serveur - v106
  ifNotFlag,     // Visible si l'indicateur est faux (Flags AND Hidden = 0)
  ifAnimation,   // "Index" dsigne une animation (squence personnalise)
  ifArray,       // l'image se rpte (ne peut pas tre anime)
  ifForeground,  // l'image est dessine par dessus les personnages (suivant position en Y)
  ifAutoFlip,    // inverse automatiquement le flag quand on clique dessus
  ifLightSource  // l'image est une source de lumire
 );
 TImageFlags=set of TImageFlag;

 TImageData105=packed record
  Index   :integer;
  Position:TPoint;
  BaseLine:integer;
  Flags   :TImageFlags;
 end;

 TImageData106=packed record
  Index   :integer;
  Position:TPoint;
  BaseLine:integer;
  Flags   :TImageFlags;
  Hidden  :integer; // Flags (si <0 alors NOT Flags)
  Data    :integer; // -> Animation Index, Array Index...
 end;
 TImages=array[word] of TImageData106;
 PImages=^TImages;

 TArrayData105=packed record
  Index :integer; // Image ou animation
  Width :integer; // nombre de rptitions en largeur
  Height:integer; // nombre de rptitions en hauteur
  Step  :TPoint;  // nombre de cases (96x48) de dplacement
 end;

 TArrayData106=packed record
  Width :integer; // nombre de rptitions en largeur
  Height:integer; // nombre de rptitions en hauteur
  Step  :TPoint;  // nombre de cases (96x48) de dplacement
 end;
 TArrays=array[word] of TArrayData106;
 PArrays=^TArrays;

 TLightSource=packed record
  ObjectID:integer;
  Color   :integer; // AARRGGBB
  Position:TPoint;  // Par rapport  l'image
  Radius  :integer;
 end;
 PLightSource=^TLightSource;
 TLightSources=array[word] of TLightSource;
 PLightSources=^TLightSources;

 TObstacle = (
  TobsNone, TobsSpecial,
  //Plutot des type de terrains que des obstacles
  TobsStandard, TobsForest, TobsIce, TobsRugged, TobsSand, TobsWater_WaistDeep,
  //Eaux profondes. Donc Obstacle
  TobsWater_ShoulderDeep, TobsWater_Deeper,
  //Differents trous qui sont des obstacles. TobsBlackHole est un peu spcial et utiliser pour les tilde non initialiser
  TobsLitteHole, TobsMediumHole, TobsBlackHole, TobsRavin,
  // muret, mur de maison et mur de forteresse.
  TobsLittleWall, TobsMediumWall, TobsBigWall,
  // Obstcale Pure
  TobsSet, TobsHighSet, TobsPorte, TobsGrille, TobsHerse
 );

 TCell=record
  Obstacle:TObstacle;
 end;
 PCell=^TCell;
 TCells=array[word] of TCell;

 TADKMap=class
 private
  fFileName:string;       // nom du fichier
  fReadOnly:boolean;      // mode d'accs au fichier
  fFile    :integer;      // Handle du fichier
  fSize    :Cardinal;     // Taille du fichier
  fMap     :integer;      // le FileMapping !
  fBase    :pchar;        // pointer sur le fichier sur "disque mapp en mmoire"
  fHeader  :^THeader107;
  fArrays  :PArrays;
  fImages  :PImages;
  fItems   :PImages;
  fTexts   :PStringInfos;
  fLights  :PLightSources;
  fCells   :^TCells;
  fFlags   :integer; // initialis avec fHeader.Flags
  procedure MapFile;
  procedure UnMapFile;
  function GetImageCount:integer;
  function GetItemCount:integer;
  function GetCell(x,y:integer):PCell;
  function GetLightSource(Index:integer):PLightSource;
 public
  destructor Destroy; override;
  procedure LoadFromFile(AFileName:string; AReadOnly:boolean=True);
  procedure Close;
  function Width :integer;
  function Height:integer;
  function Origin:TPoint;
  function Ambient:integer;
  function Options:TMapOptions;
  function Text(ObjectID:integer):string;
  function Light(ObjectID:integer):PLightSource;
  function LightCount:integer;
  function LightSource(Index:integer; lx,ly:single):integer;
  property Lights[Index:integer]:PLightSource read GetLightSource;
  property FileName:string read fFileName;
  property ImageCount:integer read GetImageCount;
  property Images:PImages read fImages;
  property ItemCount:integer read GetItemCount;
  property Items:PImages read fItems;
  property Arrays:PArrays read fArrays;
  property Cells[x,y:integer]:PCell read GetCell;
  property Flags:integer read fFlags write fFlags;
 end;

implementation

destructor TADKMap.Destroy;
begin
 close;
 inherited;
end;

procedure TADKMap.LoadFromFile(AFileName:string; AReadOnly:boolean=True);
const
 OpenMode:array[false..true] of Cardinal=(OPEN_ALWAYS,OPEN_EXISTING);
 FileMode:array[false..true] of Cardinal=(GENERIC_READ or GENERIC_WRITE,GENERIC_READ);
begin
 Close;
 fFileName:=AFileName;
 fReadOnly:=AReadOnly;
 fFile:=CreateFile(PChar(fFileName),FileMode[fReadOnly],FILE_SHARE_READ,nil,OpenMode[fReadOnly],FILE_ATTRIBUTE_NORMAL,0);
 try
  if fFile=0 then raise Exception.Create('Impossible d''ouvrir '+fFileName);
  MapFile;
 except
  Close;
  raise;
 end;
end;

procedure TADKMap.Close;
begin
 UnMapFile;
 CloseHandle(fFile);
end;

procedure TADKMap.MapFile;
const
 PageMode:array[false..true] of integer=(PAGE_READWRITE,PAGE_READONLY);
 MapMode :array[false..true] of integer=(FILE_MAP_ALL_ACCESS,FILE_MAP_READ);
begin
 fSize:=GetFileSize(fFile,nil);
 fMap :=CreateFileMapping(fFile,nil,PageMode[fReadOnly],0,0,nil);
 if fMap=0 then raise Exception.Create('Impossible de lire le fichier  '+fFileName);
 fBase:=MapViewOfFile(fMap,MapMode[fReadOnly],0,0,0);
 if fBase=nil then raise Exception.Create('Problme d''accs au fichier '+fFileName);
 fHeader:=@fBase[0];
 if fHeader.Signature<>Cardinal(MAP_SIGNATURE) then raise Exception.Create('Le fichier carte est invalide');
 if fHeader.Version<MAP_VERSION then raise Exception.Create('Le fichier carte est trop ancien');
 if fHeader.Version>MAP_VERSION then raise Exception.Create('Le fichier carte est trop rcent');
 fCells :=@fBase[SizeOf(fHeader^)];
 fArrays:=@fBase[fHeader.Arrays.Offset];
 fImages:=@fBase[fHeader.Images.Offset];
 fItems :=@fBase[fHeader.Items.Offset];
 fTexts :=@fBase[fHeader.Texts.Offset];
 fLights:=@fBase[fHeader.Lights.Offset];
 fFlags :=fHeader.Flags;
end;

procedure TADKMap.UnMapFile;
begin
 UnMapViewOfFile(fBase);
 CloseHandle(fMap);
end;

function TADKMap.Width:integer;
begin
 Result:=fHeader.Width;
end;

function TADKMap.Height:integer;
begin
 Result:=fHeader.Height;
end;

function TADKMap.Origin:TPoint;
begin
 Result:=fHeader.Origin;
end;

function TADKMap.Ambient;
begin
 Result:=fHeader.Ambient;
end;

function TADKMap.Options:TMapOptions;
begin
 Result:=fHeader.Options;
end;

function TADKMap.Text(ObjectID:integer):string;
var
 i,l:integer;
begin
 Result:=IntToStr(fHeader.Texts.Count)+':';
 for i:=0 to fHeader.Texts.Count-1 do begin
  if fTexts[i].ObjectID=ObjectID then begin
   l:=fTexts[i].Length;
   SetLength(Result,l);
   Move(fBase[fTexts[i].Offset],Result[1],l);
   exit;
  end;
  Result:=Result+IntToStr(fTexts[i].ObjectID)+',';
 end;
 Result:=Result+'???#'+IntToStr(ObjectID);
end;

function TADKMap.Light(ObjectID:integer):PLightSource;
var
 i:integer;
begin
 for i:=0 to fHeader.Lights.Count-1 do begin
  if fLights[i].ObjectID=ObjectID then begin
   Result:=@fLights[i];
   exit;
  end;
 end;
 Result:=nil;
end;

function TADKMap.GetImageCount:integer;
begin
 Result:=fHeader.Images.Count;
end;

function TADKMap.GetItemCount:integer;
begin
 Result:=fHeader.Items.Count;
end;

function TADKMap.GetCell(x,y:integer):PCell;
begin
 Result:=@fCells[x+fHeader.Width*y];
end;

function TADKMap.LightCount:integer;
begin
 Result:=fHeader.Lights.Count;
end;

function TADKMap.LightSource(Index:integer;lx,ly:single):integer;
var
 l:single;
begin
 with fLights[Index] do begin
  if ObjectID<fHeader.Images.Count then begin
   with fImages[ObjectID] do begin
    lx:=lx-Position.x+Origin.X;
    ly:=ly-Position.y+Origin.Y;
   end;
  end else begin
   with fItems[ObjectID-fHeader.Images.Count] do begin
    lx:=lx-Position.x+Origin.X;
    ly:=ly-Position.y+Origin.Y;
   end;
  end;
  lx:=lx-Position.x;
  ly:=ly-Position.y;
  l:=sqrt(lx*lx+ly*ly);
  Result:=255-Round(255*l/(255-Color shr 24+Radius));
  if result<0 then Result:=0;
  if Result>255 then Result:=255;
 end;
end;

function TADKMap.GetLightSource(Index:integer):PLightSource;
begin
 Result:=@fLights[Index];
end;

end.
