unit ADKScreens;

{
 projet ADK-ISO (c)2002-2003 Paul TOTH <tothpaul@free.fr>

 http://www.web-synergy.net/naug-land/
}

{$define secure}

interface

uses
 Windows,Classes,SysUtils,Graphics,
 MiniDX,DXError,ADKDepth;

type
 TADKScreen=class
 private
  fWidth,fHeight,fDepth:integer;
  fBPP,RedMask,GreenMask,BlueMask:integer;
  fHalfMask:cardinal;
  fMemory:pointer;
  fPitch:integer;
  procedure PixelFormat(aBPP,aRed,aGreen,aBlue,aHalf:integer);
  function GetPixel(x,y:integer):pointer;
 public
  procedure SetSize(Width,Height,Depth:integer); virtual;
  procedure Trame(Left,Top,width,height:integer; acolor:integer=0); virtual;
  function GetTextExtent(Text:string):TSize;
  procedure DrawHint(x,y,color:integer; Text:string);
  procedure DrawText(x,y,color:integer; Text:string);
  procedure TextRect(R:TRect; Color:integer; Text:string);
  procedure DrawLines(Rect:TRect; Lines:TStrings);
  procedure PutPixel(x,y,c:integer);
  procedure PutAlpha(x,y,c:integer);

  function Clear(Color:integer):boolean; virtual; abstract;
  procedure FillRect(Left,Top,Width,Height,AColor:integer); virtual;
  function Lock:boolean; virtual;
  procedure UnLock; virtual;
  function GetDC:HDC; virtual; abstract;
  procedure ReleaseDC; virtual; abstract;
  procedure Flip; virtual; abstract;

  property Pixels[x,y:integer]:pointer read GetPixel;
  property Pitch:integer read fPitch;
  property Width:integer read fWidth;
  property Height:integer read fHeight;
  property Depth:integer read fDepth;
  property BPP:integer read fBPP;
  property HalfMask:cardinal read fHalfMask;
 end;

 TADKScreenDX=class(TADKScreen)
 private
  fHWnd:HWnd;
  fDC:HDC;
  fFull:boolean;
  DDraw:IDirectDraw2;
  DPrim:IDirectDrawSurface;
  DBack:IDirectDrawSurface;
//  procedure CheckDD(Err:integer; Msg:string);
  procedure CreatePrimary;
  function RestoreBackBuffer:boolean;
 protected
  procedure NeedDirectDraw; virtual;
  procedure CreateBackBuffer; virtual;
 public
  constructor Create(Handle:HWnd; FullScreen:boolean=false);
  destructor Destroy; override;
  procedure SetSize(Width,Height,Depth:integer); override;
  function Lock:boolean; override;
  procedure UnLock; override;
  function Clear(Color:integer):boolean; override;
  procedure Trame(Left,Top,width,height:integer; acolor:integer=0); override;
  function GetDC:HDC; override;
  procedure ReleaseDC; override;
  procedure Flip; override;
 end;

 TADKScreen3D=class(TADKScreenDX)
 private
  D3D  :IDirect3D2;
  DDev :IDirect3DDevice2;
 protected
  procedure NeedDirectDraw; override;
  procedure CreateBackBuffer; override;
 public
  procedure Trame(Left,Top,width,height:integer; acolor:integer=0); override;
  procedure FillRect(Left,Top,Width,Height,AColor:integer); override;
  procedure DrawPoly(const Points:array of TD3DTLVertex);
  procedure Circle(x,y,ARadius,AColor:integer);
 end;

 TADKScreenGDI=class(TADKScreen)
 private
  fCanvas:TCanvas;
  fBitmap:TBitmap;
  fRect:TRect;
 public
  constructor Create(Handle:HWnd);
  destructor Destroy; override;
  procedure SetSize(Width,Height,Depth:integer); override;
  function Clear(Color:integer):boolean; override;
  function GetDC:HDC; override;
  procedure ReleaseDC; override;
  procedure Flip; override;
 end;

var
 ADKScreen:TADKScreen;

function WindowDepth:integer;

implementation

//uses ADKRender, ADKData, ADKObjects, ADKMaps;

function WindowDepth:integer;
var
 dc:HDC;
begin
 dc:=GetDC(0);
 Result:=GetDeviceCaps(DC,BITSPIXEL);
 ReleaseDC(0,DC);
end;

//----------------------------------------------------------------------------//

procedure TADKScreen.Trame(Left,Top,width,height:integer; acolor:integer=0);
var
 py:pchar;
 px:pointer;
 x,y:integer;
begin
{$ifdef secure}
 ReleaseDC;
 Lock;
{$endif}
 if Left<0 then Left:=0;
 if Top<0 then Top:=0;
 if Left+Width>Self.Width then Width:=Self.Width-Left;
 if Top+Height>Self.Height then Height:=Self.Height-Top;
 py:=Pixels[Left,Top];//@dib[Pitch*Top+ADKScreen.BPP*Left];
 for y:=0 to height-1 do begin
  px:=pointer(py);
  if odd(y+Top+Left) then inc(cardinal(px),BPP);
  if BPP=BPP32 then begin
   for x:=0 to (width div 2)-1 do begin
    TPixel32(px^):=AColor;
    inc(cardinal(px),2*BPP);
   end;
  end else begin
   for x:=0 to (width div 2)-1 do begin
    TPixel16(px^):=AColor;
    inc(cardinal(px),2*BPP);
   end;
  end;
  inc(py,Pitch);
 end;
end;

procedure TADKScreen.FillRect(Left,Top,width,height,AColor:integer);
var
 py:pchar;
 px:pointer;
 x,y:integer;
begin
{$ifdef secure}
 ReleaseDC;
 Lock;
{$endif}
 if Left<0 then Left:=0;
 if Top<0 then Top:=0;
 if Left+Width>Self.Width then Width:=Self.Width-Left;
 if Top+Height>Self.Height then Height:=Self.Height-Top;
 py:=Pixels[Left,Top];
 for y:=0 to height-1 do begin
  px:=pointer(py);
  if BPP=BPP32 then begin
   for x:=0 to Width-1 do begin
    TPixel32(px^):=AColor;
    inc(cardinal(px),BPP);
   end;
  end else begin
   for x:=0 to Width-1 do begin
    TPixel16(px^):=AColor;
    inc(cardinal(px),BPP);
   end;
  end;
  inc(py,Pitch);
 end;
end;

procedure TADKScreen.SetSize(Width,Height,Depth:integer);
begin
 fWidth:=Width;    //HalfWidth:=fWidth shr 1;
 fHeight:=Height;  //HalfHeight:=fHeight shr 1;
 fDepth:=Depth;
 case fDepth of
  15: PixelFormat(2,   $7C00,  $03E0,  $001F,   $7BDE);
  16: PixelFormat(2,   $F800,  $07E0,  $001F,   $F7DE);
  32: PixelFormat(4, $FF0000,$00FF00,$0000FF, $FEFEFE);
  else raise Exception.Create('Format graphique non support ('+IntToStr(fDepth)+')');
 end;
end;

procedure TADKScreen.PixelFormat(aBPP,aRed,aGreen,aBlue,aHalf:integer);
begin
 fBPP:=aBPP;
 RedMask:=aRed;
 GreenMask:=aGreen;
 BlueMask:=aBlue;
 fHalfMask:=aHalf;
end;

function TADKScreen.Lock:boolean;
begin
 Result:=True;
end;

procedure TADKScreen.UnLock;
begin
end;

function TADKScreen.GetPixel(x,y:integer):pointer;
begin
{$ifdef secure}
 ReleaseDC;
 Lock;
{$endif}
 Result:=pointer(integer(fMemory)+fBPP*x+fPitch*y);
end;

procedure TADKScreen.PutPixel(x,y,c:integer);
var
 pix:pointer;
begin
 if (x<0)or(x>=fWidth)or(y<0)or(y>=fHeight) then exit;
 pix:=Pixels[x,y];
 if fBPP=BPP32 then TPixel32(pix^):=c else TPixel16(pix^):=c;
end;

procedure TADKScreen.PutAlpha(x,y,c:integer);
var
 pix:pointer;
begin
 if (x<0)or(x>=fWidth)or(y<0)or(y>=fHeight) then exit;
 pix:=Pixels[x,y];
 if fBPP=BPP32 then
  TPixel32(pix^):=((c and HalfMask)+(TPixel32(pix^) and HalfMask)) shr 1
 else
  TPixel16(pix^):=((c and HalfMask)+(TPixel16(pix^) and HalfMask)) shr 1;
end;

function TADKScreen.GetTextExtent(Text:string):TSize;
var
 dc:HDC;
begin
{$ifdef secure}
 UnLock;
{$endif}
 dc:=GetDC;
 try
  GetTextExtentPoint32(dc,@Text[1],Length(Text),Result);
 finally
  ReleaseDC;
 end;
end;

procedure TADKScreen.DrawHint(x,y,color:integer; Text:string);
var
 dc:HDC;
begin
{$ifdef secure}
 UnLock;
{$endif}
 dc:=GetDC;
 try
  SetBkMode(DC,TRANSPARENT);
  SetTextColor(dc,Color);
  SetTextAlign(dc,TA_CENTER);
  TextOut(dc,x,y,@Text[1],Length(Text));
 finally
  ReleaseDC;
 end;
end;

procedure TADKScreen.DrawText(x,y,color:integer; Text:string);
var
 dc:HDC;
begin
{$ifdef secure}
 UnLock;
{$endif}
 dc:=GetDC;
 try
  SetBkMode(DC,TRANSPARENT);
  SetTextColor(dc,Color);
  SetTextAlign(dc,TA_LEFT);
  TextOut(dc,x,y,@Text[1],Length(Text));
 finally
  ReleaseDC;
 end;
end;

procedure TADKScreen.TextRect(R:TRect; Color:integer; Text:string);
var
 dc:HDC;
begin
{$ifdef secure}
 UnLock;
{$endif}
 dc:=GetDC;
 try
  SetBkMode(DC,TRANSPARENT);
  SetTextColor(dc,Color);
  DrawTextEx(dc,@Text[1],Length(Text),R,DT_WORDBREAK or DT_NOCLIP or DT_NOPREFIX,nil);
 finally
  ReleaseDC;
 end;
end;

procedure TADKScreen.DrawLines(Rect:TRect; Lines:TStrings);
const
 k=2;
var
 dc:HDC;
  R:TRect;
  y:integer;
  i:integer;
  s:string;
begin
{$ifdef secure}
 UnLock;
{$endif}
 dc:=GetDC;
 try
  SetBkMode(DC,TRANSPARENT);
  SetTextAlign(dc,TA_LEFT or TA_TOP or TA_NOUPDATECP);
  if Rect.Top>0 then ExcludeClipRect(dc,0,0,Width,Rect.Top);
  y:=Rect.Bottom;
  for i:=Lines.Count-1 downto 0 do begin
   s:=Lines[i];
   if y<Rect.Top then break;
   dec(y,14);
   R.Left:=Rect.Left;
   R.Top:=y;
   R.Right:=Rect.Right;
   R.Bottom:=y+14;
   DrawTextEx(dc,@s[1],Length(s),R,DT_CALCRECT or DT_WORDBREAK or DT_NOCLIP or DT_NOPREFIX,nil);
   dec(y,R.Bottom-R.Top-14);
   R.Top:=y;

   SetTextColor(DC,0);
   dec(R.Top,k);
   DrawTextEx(dc,@s[1],Length(s),R,DT_WORDBREAK or DT_NOCLIP or DT_NOPREFIX,nil);
   inc(R.Top,k); inc(R.Left,k); inc(R.Right,k);
   DrawTextEx(dc,@s[1],Length(s),R,DT_WORDBREAK or DT_NOCLIP or DT_NOPREFIX,nil);
   inc(R.Top,k); inc(R.Bottom,k); dec(R.Left,k); dec(R.Right,k);
   DrawTextEx(dc,@s[1],Length(s),R,DT_WORDBREAK or DT_NOCLIP or DT_NOPREFIX,nil);
   dec(R.Top,k); dec(R.Bottom,k); dec(R.Left,k); dec(R.Right,k);
   DrawTextEx(dc,@s[1],Length(s),R,DT_WORDBREAK or DT_NOCLIP or DT_NOPREFIX,nil);
   inc(R.Left,k); inc(R.Right,k);

   SetTextColor(DC,integer(Lines.Objects[i]));
   DrawTextEx(dc,@s[1],Length(s),R,DT_WORDBREAK or DT_NOCLIP or DT_NOPREFIX,nil);
  end;
 finally
  ReleaseDC;
 end;
end;

//----------------------------------------------------------------------------//

constructor TADKScreenDX.Create(Handle:HWnd; FullScreen:boolean=false);
begin
 fHWnd:=Handle;
 fFull:=FullScreen;
end;

destructor TADKScreenDX.Destroy;
begin
 DBack:=nil;
 DPrim:=nil;
 DDraw:=nil;
 inherited;
end;
(*
procedure TADKScreenDX.CheckDD(Err:integer; Msg:string);
begin
 if Err=0 then exit;
 raise Exception.Create('DirectX error #'+IntToHex(Err,8)+' '+Msg);
end;
*)
procedure TADKScreenDX.NeedDirectDraw;
var
 DD:IUnknown;
begin
 if DDraw=nil then begin
  DirectDrawCreate(nil,DD,nil);
  if DD=nil then raise Exception.Create('DirectDrawCreate');
  CheckDD(DD.QueryInterface(IID_IDirectDraw2,DDraw),'QueryInterface IDirectDraw2');
  if fFull then begin
//   raise exception.Create('FullScreen mode not allowed')
   CheckDD(DDraw.SetCooperativeLevel(fHWnd,DDSCL_ALLOWREBOOT or DDSCL_EXCLUSIVE or DDSCL_FULLSCREEN),'SetCooperativeLevel');
 // change display - doit se faire APRES le SetCooperativeLevel (bug Owen)
   CheckDD(DDraw.SetDisplayMode(Width,Height,Depth,0,0),'SetDisplayMode');
  end else
   CheckDD(DDraw.SetCooperativeLevel(fHWnd,DDSCL_NORMAL),'SetCooperativeLevel');
  CreatePrimary;
 end;
end;

procedure TADKScreenDX.CreatePrimary;
var
 SD:TDDSurfaceDesc;
 CP:IDirectDrawClipper;
begin
 NeedDirectDraw;
 DPrim:=nil;
 FillChar(SD,SizeOf(SD),0);
 with SD do begin
  dwSize :=SizeOf(SD);
  dwFlags:=DDSD_CAPS;
  DDSCaps.dwCaps:=DDSCAPS_PRIMARYSURFACE;
 end;
 CheckDD(DDraw.CreateSurface(SD,DPrim,nil),'CreateSurface S1');
 // ajouter un clipper en rapport  la fentre
 CheckDD(DDraw.CreateClipper(0,CP,nil),'CreateClipper');
 CheckDD(CP.SetHWnd(0,fHWnd),'SetHWnd');
 CheckDD(DPrim.SetClipper(CP),'SetClipper');
 CP:=nil;
end;

procedure TADKScreenDX.CreateBackBuffer;
var
 SD:TDDSurfaceDesc;
// VP:IDirect3DViewPort2;
// VP2:TD3DViewPort2;
begin
 NeedDirectDraw;
 DBack:=nil;
 FillChar(SD,SizeOf(SD),0);
 with SD do begin
  dwSize :=SizeOf(SD);
  dwFlags:=DDSD_CAPS or DDSD_WIDTH or DDSD_HEIGHT or DDSD_PIXELFORMAT;
  DDSCaps.dwCaps:=DDSCAPS_OFFSCREENPLAIN or DDSCAPS_3DDEVICE;
  dwWidth :=fWidth;
  dwHeight:=fHeight;
  ddpfPixelFormat.dwSize:=SizeOf(ddpfPixelFormat);
  ddpfPixelFormat.dwFlags:=DDPF_RGB;
  ddpfPixelFormat.dwRGBBitCount:=fDepth;
  ddpfPixelFormat.dwRBitMask:=RedMask;
  ddpfPixelFormat.dwGBitMask:=GreenMask;
  ddpfPixelFormat.dwBBitMask:=BlueMask;
  ddpfPixelFormat.dwRGBAlphaBitMask:=0;
 end ;
 CheckDD(DDraw.CreateSurface(SD,DBack,nil),'CreateSurface S2');
end;

function TADKScreenDX.RestoreBackBuffer:boolean;
var
 err:HResult;
begin
 err:=DBack.Restore;
 if err=DDERR_WRONGMODE then begin
  CreateBackBuffer;
  Result:=False;
 end else begin
  CheckDD(err,'Restore S2');
  Result:=True;
 end;
end;

procedure TADKScreenDX.SetSize(Width,Height,Depth:integer);
begin
 inherited;
 NeedDirectDraw;
 CreateBackBuffer;
end;

function TADKScreenDX.Clear(Color:integer):boolean;
var
 FX:TDDBltFX;
 er:integer;
begin
 UnLock;
 ReleaseDC;
 FillChar(FX,SizeOf(FX),0);
 FX.dwSize:=SizeOf(FX);
 FX.dwFillColor:=Color;
 er:=DBack.Blt(TRect(nil^), nil, TRect(nil^), DDBLT_COLORFILL or DDBLT_WAIT, FX);
 if er<>DDERR_SURFACELOST then
  CheckDD(er,'DirectX.Clear')
 else begin
  if not RestoreBackBuffer then begin
   Result:=False;
   exit;
  end;
  CheckDD(DBack.Blt(TRect(nil^), nil, TRect(nil^), DDBLT_COLORFILL or DDBLT_WAIT, FX),'DirectX.Clear 2');
 end;
 Result:=True;
end;

procedure TADKScreenDX.Trame(Left,Top,Width,Height:integer; acolor:integer=0);
begin
 Lock;
 inherited;
end;

function TADKScreenDX.Lock:boolean;
var
 SD:TDDSurfaceDesc;
 er:integer;
begin
 if fMemory=nil then begin
  ReleaseDC;
  FillChar(SD,SizeOf(SD),0);
  SD.dwSize:=SizeOf(SD);
  er:=DBack.Lock(nil,SD,DDLOCK_SURFACEMEMORYPTR or DDLOCK_WAIT,0);
  if er<>DDERR_SURFACELOST then
   CheckDD(er,'Lock S2 1')
  else begin
   if not RestoreBackBuffer then begin
    Result:=False;
    exit;
   end;
   CheckDD(DBack.Lock(nil,SD,DDLOCK_SURFACEMEMORYPTR or DDLOCK_WAIT,0),'Relock S2 1');
  end;
  fMemory:=SD.lpSurface;
  fPitch:=SD.lPitch;
 end;
 Result:=True;
end;

procedure TADKScreenDX.UnLock;
begin
 if fMemory=nil then exit;
 fMemory:=nil;
// fPitch:=0;
 CheckDD(DBack.Unlock(nil),'Unlock S2 1');
end;

function TADKScreenDX.GetDC:HDC;
begin
 UnLock;
 if fDC=0 then CheckDD(DBack.GetDC(fDC),'GetDC S2');
 Result:=fDC;
end;

procedure TADKScreenDX.ReleaseDC;
begin
 if fDC=0 then exit;
 CheckDD(DBack.ReleaseDC(fDC),'RelaseDC S2');
 fDC:=0;
end;

procedure TADKScreenDX.Flip;
var
 FX:TDDBltFX;
  R:TRect;
 er:integer;
begin
 UnLock;
 ReleaseDC;
 FillChar(FX,SizeOf(FX),0);
 FX.dwSize:=SizeOf(FX);
 R.Top:=0;
 R.Left:=0;
 R.Right:=fWidth;
 R.Bottom:=fHeight;
 Windows.ClientToScreen(fHwnd,R.TopLeft);
 Windows.ClientToScreen(fHwnd,R.BottomRight);
 er:=DPrim.Blt(R, DBack, TRect(nil^), DDBLT_WAIT, FX);
 if er<>DDERR_SURFACELOST then
  CheckDD(er,'Blt S2/S1')
 else begin
  er:=DPrim.Restore;
  if er=DDERR_WRONGMODE then CreatePrimary else CheckDD(er,'Restore S1');
  CheckDD(DPrim.Blt(R, DBack, TRect(nil^), DDBLT_WAIT, FX),'Blt S1/S2 2');
 end;
end;

//----------------------------------------------------------------------------//

procedure TADKScreen3D.NeedDirectDraw;
begin
 if D3D=nil then begin
  inherited;
  CheckDD(DDraw.QueryInterface(IID_IDirect3D2,D3D),'QueryInterface IDirect3D2');
 end;
end;

procedure TADKScreen3D.CreateBackBuffer;
var
 VP:IDirect3DViewPort2;
 VP2:TD3DViewPort2;
begin
 inherited;
 CheckDD(D3D.CreateDevice(IID_IDirect3DHALDevice,DBack,DDev),'CreateDevice IDirect3DHALDevice');
 CheckDD(D3D.CreateViewport(VP,nil),'CreateViewPort D3D');
 CheckDD(DDev.AddViewport(VP),'AddViewPort Dev3D');
 CheckDD(DDev.SetCurrentViewport(VP),'SetCurrentViewPort D3D');
 VP2.dwSize:=SizeOf(VP2);
 VP2.dwX:=0;
 VP2.dwY:=0;
 VP2.dwWidth :=fWidth;
 VP2.dwHeight:=fHeight;
 VP2.dvClipX:=-1;
 VP2.dvClipY:=+1;
 VP2.dvClipWidth:=2;
 VP2.dvClipHeight:=2;
 VP2.dvMinZ:=0;
 VP2.dvMaxZ:=1;
 CheckDD(VP.SetViewport2(VP2),'SetViewport2 D3D');
 CheckDD(DDev.SetRenderState(D3DRENDERSTATE_CULLMODE, D3DCULL_NONE ),'SetRenderState D3DCULL_NONE');
end;

procedure TADKScreen3D.Trame(Left,Top,Width,Height:integer; acolor:integer=0);
var
 Rect:array[0..3] of TD3DTLVertex;
begin
 UnLock;
 ReleaseDC;
 FillChar(Rect,SizeOf(Rect),0);
 acolor:=$80000000+acolor;
 with Rect[0] do begin sx:=Left;       sy:=Top;        rhw:=1; color:=acolor; end;
 with Rect[1] do begin sx:=Left+Width; sy:=Top;        rhw:=1; color:=acolor; end;
 with Rect[2] do begin sx:=Left;       sy:=Top+Height; rhw:=1; color:=acolor; end;
 with Rect[3] do begin sx:=Left+Width; sy:=Top+Height; rhw:=1; color:=acolor; end;
 CheckDD(DDev.SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_SRCALPHA),'SetRenderState SRCBLEND');
 CheckDD(DDev.SetRenderState(D3DRENDERSTATE_DESTBLEND,D3DBLEND_INVSRCALPHA),'SetRenderState DESTBLEND');
 CheckDD(DDev.SetRenderState(D3DRENDERSTATE_BLENDENABLE,1),'SetRenderState ALPHABLENDENABLE');
 CheckDD(DDev.BeginScene,'BeginScene');
 CheckDD(DDev.DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DVT_TLVERTEX,Rect, 4,0),'DrawPrimitive');
 CheckDD(DDev.EndScene,'EndScene');
end;

procedure TADKScreen3D.FillRect(Left,Top,Width,Height,AColor:integer);
var
 Rect:array[0..3] of TD3DTLVertex;
begin
 UnLock;
 ReleaseDC;
 FillChar(Rect,SizeOf(Rect),0);
 with Rect[0] do begin sx:=Left;       sy:=Top;        rhw:=1; color:=AColor; end;
 with Rect[1] do begin sx:=Left+Width; sy:=Top;        rhw:=1; color:=AColor; end;
 with Rect[2] do begin sx:=Left;       sy:=Top+Height; rhw:=1; color:=AColor; end;
 with Rect[3] do begin sx:=Left+Width; sy:=Top+Height; rhw:=1; color:=AColor; end;
 CheckDD(DDev.SetRenderState(D3DRENDERSTATE_BLENDENABLE,0),'SetRenderState ALPHABLENDENABLE');
 CheckDD(DDev.BeginScene,'BeginScene');
 CheckDD(DDev.DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DVT_TLVERTEX,Rect, 4,0),'DrawPrimitive');
 CheckDD(DDev.EndScene,'EndScene');
end;

{
function RGB2BGR(i:integer):integer;
begin
 Result:=i and $00FF00 + (i shl 16) + (i shr 16);
end;
}
procedure TADKScreen3D.DrawPoly(const Points:array of TD3DTLVertex);
begin
 UnLock;
 ReleaseDC;
 CheckDD(DDev.SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_INVSRCALPHA),'SetRenderState SRCBLEND');
 CheckDD(DDev.SetRenderState(D3DRENDERSTATE_DESTBLEND,D3DBLEND_SRCALPHA),'SetRenderState DESTBLEND');
 CheckDD(DDev.SetRenderState(D3DRENDERSTATE_BLENDENABLE,1),'SetRenderState ALPHABLENDENABLE');
 CheckDD(DDev.BeginScene,'BeginScene');
 CheckDD(DDev.DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DVT_TLVERTEX,Points[0], Length(Points),0),'DrawPrimitive');
 CheckDD(DDev.EndScene,'EndScene');
end;

procedure TADKScreen3D.Circle(x,y,ARadius,AColor:integer);
var
 fan:array[0..9] of TD3DTLVertex;
 r2:integer;
begin
 UnLock;
 ReleaseDC;
 FillChar(fan,SizeOf(fan),0);
 r2:=(2*ARadius) div 3;
 with fan[0] do begin sx:=x; sy:=y; rhw:=1; color:=AColor end;
 AColor:=AColor or $FF000000;
 with fan[1] do begin sx:=x; sy:=y-ARadius; rhw:=1; color:=AColor end;
 with fan[2] do begin sx:=x+r2; sy:=y-r2; rhw:=1; color:=AColor end;
 with fan[3] do begin sx:=x+ARadius; sy:=y; rhw:=1; color:=AColor end;
 with fan[4] do begin sx:=x+r2; sy:=y+r2; rhw:=1; color:=AColor end;
 with fan[5] do begin sx:=x; sy:=y+ARadius; rhw:=1; color:=AColor end;
 with fan[6] do begin sx:=x-r2; sy:=y+r2; rhw:=1; color:=AColor end;
 with fan[7] do begin sx:=x-ARadius; sy:=y; rhw:=1; color:=AColor end;
 with fan[8] do begin sx:=x-r2; sy:=y-r2; rhw:=1; color:=AColor end;
 fan[9]:=fan[1];
 CheckDD(DDev.SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_INVSRCALPHA),'SetRenderState SRCBLEND');
 CheckDD(DDev.SetRenderState(D3DRENDERSTATE_DESTBLEND,D3DBLEND_SRCALPHA),'SetRenderState DESTBLEND');
 CheckDD(DDev.SetRenderState(D3DRENDERSTATE_BLENDENABLE,1),'SetRenderState ALPHABLENDENABLE');
 CheckDD(DDev.BeginScene,'BeginScene');
 CheckDD(DDev.DrawPrimitive(D3DPT_TRIANGLEFAN, D3DVT_TLVERTEX,fan[0], Length(fan),0),'DrawPrimitive');
 CheckDD(DDev.EndScene,'EndScene');
end;

//----------------------------------------------------------------------------//

constructor TADKScreenGDI.Create(Handle:HWnd);
begin
 fCanvas:=TCanvas.Create;
 fCanvas.Handle:=Windows.GetDC(Handle);
 fBitmap:=TBitmap.Create;
 fBitmap.Canvas.Font.Style:=[fsBold];
 fBitmap.Canvas.Font.Size:=11;
 fBitmap.HandleType:=bmDDB;
end;

destructor TADKScreenGDI.Destroy;
begin
 fBitmap.Free;
 inherited;
end;

procedure TADKScreenGDI.SetSize(Width,Height,Depth:integer);
begin
 inherited;
 case fDepth of
  15: fBitmap.PixelFormat:=pf15bit;
  16: fBitmap.PixelFormat:=pf16bit;
  32: fBitmap.PixelFormat:=pf32bit;
 end;
 fBitmap.Width:=fWidth;
 fBitmap.Height:=2*fHeight;
 fRect.Right:=fWidth;
 fRect.Bottom:=fHeight;
 fMemory:=fBitmap.ScanLine[0];
 fPitch:=-((fBPP*fWidth+3) and (not 3));
end;

function TADKScreenGDI.Clear(Color:integer):boolean;
begin
 with fBitmap,Canvas do begin
  Brush.Color:=color;
  FillRect(fRect);
 end;
 Result:=True;
end;

function TADKScreenGDI.GetDC:HDC;
begin
 Result:=fBitmap.Canvas.Handle;
end;

procedure TADKScreenGDI.ReleaseDC;
begin
end;

procedure TADKScreenGDI.Flip;
begin
 fCanvas.Draw(0,0,fBitmap);
end;

end.
