unit ADKSprites;

{
 Biblithque de Sprites pour le projet ADK-ISO (c)2002 Paul TOTH <tothpaul@free.fr>

 http://www.web-synergy.net/naug-land/

}

interface

uses
 Windows,Classes,SysUtils,Graphics,ADKCompress,ADKDepth,ADKScreens;

Const
 LIB_SIGN:array[0..3] of char='SPRL';
 LIB_VERSION=105;

// historique des versions
// 101 - premire version
// 102 - ajout de TFrameHeader pour grer les dplacements
// 103 - sparation des actions en plusieurs fichiers SPR
// 104 - nouvelle compression (supportant le clipping) dans ADKCompress
// 105 - palette RGB et non BGR

type
 TDirection=(dirN,dirNE,dirE,dirSE,dirS,dirSW,dirW,dirNW);

 TFrameHeader=packed record
  Left  :word;
  Top   :word;
  Width :word;
  Height:word;
  Delay :integer;
  MoveX :integer;
  MoveY :integer;
  MoveZ :integer;
 end;
 PFrameHeader=^TFrameHeader;

 TSpriteFrames=packed record
  Colors:Cardinal; { Palette color index }
  First :word;     { Offsets index of the first frame }
  Count :word;     { number of frames for that sprite }
  Trans :byte;     { transparent color }
  Origin:TPoint;
 end;
 PSpriteFrames=^TSpriteFrames;

 TSpriteHeader=packed record
  Sign    :integer;
  Version :word;
  Width   :word;
  Height  :word;
  Sprites :array[TDirection] of TSpriteFrames;
  Offsets :Cardinal; // tableau d'offsets
  Palettes:Cardinal; // tableau de palette
 end;

 TSpriteOffsets=array[word] of integer;
// TSpritePalettes=array[word] of TPalette32;
 PPalette32=^TPalette32;

 TADKSprite=class(TADKCompressedDIB)
 private
  fFileName:string;
  fReadOnly:boolean;      // mode d'accs au fichier
  fFile    :integer;      // Handle du fichier
  fSize    :Cardinal;     // Taille du fichier
  fMap     :integer;      // le FileMapping !
  fBase    :pchar;        // pointer sur le fichier sur "disque mapp en mmoire"
  fHeader  :^TSpriteHeader;
  fOffsets :^TSpriteOffsets;
//  fPalettes:^TSpritePalettes;
  fPal32   :^TPalettes32;
  fPal15   :array of TPalette16;
  fPal16   :array of TPalette16;
  fPalCount:integer;
  function getPal(index:integer):pointer;
  procedure Open;
  procedure MapFile;
  procedure UnMapfile;
  function Fill32(Direction:TDirection; Frame:Cardinal; dib,pal:pointer; pitch:integer):integer;
  function Fill32Rect(Direction:TDirection; Frame:Cardinal; dib,pal:pointer; pitch:integer; Rect:TRect):integer;
  function Alpha32(Direction:TDirection; Frame:Cardinal; dib,pal:pointer; pitch:integer):integer;
  function Alpha32Rect(Direction:TDirection; Frame:Cardinal; dib,pal:pointer; pitch:integer; Rect:TRect):integer;
 public
  constructor Create(AFileName:string; ReadOnly:boolean=True);
  destructor destroy; override;
  function FrameCount(Direction:TDirection):integer;
  function Draw(Direction:TDirection; Frame,x,y:integer; DC:HDC):integer;
  function DrawDIB(Direction:TDirection; Frame:Cardinal; dib:pointer):integer;
  function DrawRGB32(Direction:TDirection; Frame:Cardinal; dib:pointer; pitch:integer):integer;
  function FillRGB32(Direction:TDirection; Frame:Cardinal; dib:pointer; pitch:integer):integer;
  function FillRGB32Rect(Direction:TDirection; Frame:Cardinal; dib:pointer; pitch:integer;const Rect:TRect):integer;
  function AlphaRGB32(Direction:TDirection; Frame:Cardinal; dib:pointer; pitch:integer):integer;
  function AlphaRGB32Rect(Direction:TDirection; Frame:Cardinal; dib:pointer; pitch:integer;const Rect:TRect):integer;
  procedure BorderRect(Direction:TDirection; Frame:Cardinal; dib:pointer; pitch:integer;const Rect:TRect);
  procedure Border(Direction:TDirection; Frame:Cardinal; dib:pointer; pitch:integer);
  function TransparentColor(Direction:TDirection):integer;
  function Palette(Direction:TDirection):PPalette32;
  function SpriteFrames(Direction:TDirection):PSpriteFrames;
  function FrameHeader(Direction:TDirection; Frame:integer):PFrameHeader;
  procedure Reduice;
  function Width:integer;
  function Height:integer;
  property FileName:string read fFileName;
 end;

implementation

procedure error(s:string);
begin
 raise exception.Create(s);
end;

procedure FillLong(var data; count,value:integer);
var
 p:^integer;
 i:integer;
begin
 p:=@data;
 for i:=0 to count-1 do begin
  p^:=value;
  inc(p);
 end;
end;

constructor TADKSprite.Create(AFileName:string; ReadOnly:boolean=True);
begin
 fFileName:=AFileName;
 fReadOnly:=ReadOnly;
 Open;
end;

destructor TADKSprite.Destroy;
begin
 UnMapFile;
 CloseHandle(fFile);
 inherited
end;

procedure TADKSprite.Open;
const
 OpenMode:array[false..true] of Cardinal=(OPEN_ALWAYS,OPEN_EXISTING);
 FileMode:array[false..true] of Cardinal=(GENERIC_READ or GENERIC_WRITE,GENERIC_READ);
begin
 fFile:=CreateFile(PChar(fFileName),FileMode[fReadOnly],FILE_SHARE_READ,nil,OpenMode[fReadOnly],FILE_ATTRIBUTE_NORMAL,0);
 if fFile=0 then error('Impossible d''ouvrir '+fFileName);
 fSize:=GetFileSize(fFile,nil);
 if fSize=0 then error('le fichier '+fFileName+' est vide !');
 MapFile;
end;

procedure TADKSprite.MapFile;
const
 PageMode:array[false..true] of integer=(PAGE_READWRITE,PAGE_READONLY);
 MapMode :array[false..true] of integer=(FILE_MAP_ALL_ACCESS,FILE_MAP_READ);
begin
 fMap :=CreateFileMapping(fFile,nil,PageMode[fReadOnly],0,0,nil);
 if fMap=0 then error('Impossible d''accder au fichier '+fFileName);
 fBase:=MapViewOfFile(fMap,MapMode[fReadOnly],0,0,0);
 if fBase=nil then error('Erreur d''accs au fichier '+fFileName);
 fHeader:=@fBase[0];
 if fHeader.Sign<>integer(LIB_SIGN) then error('Le fichier de sprites est invalide');
 if fHeader.Version<LIB_VERSION then error('Le fichier '+fFileName+' est trop ancien');
 if fHeader.Version>LIB_VERSION then error('Le fichier '+fFileName+' est trop rcent');
 fOffsets:=@fBase[fHeader.Offsets];
 fPal32:=@fBase[fHeader.Palettes];
 fPalCount:=(fSize-fHeader.Palettes) div SizeOf(TPalette32);
end;

procedure TADKSprite.UnMapFile;
begin
 UnMapViewOfFile(fBase);
 CloseHandle(fMap);
end;

function TADKSprite.FrameCount(Direction:TDirection):integer;
begin
 Result:=fHeader.Sprites[Direction].Count;
end;

function TADKSprite.GetPal(Index:integer):pointer;
var
 i,j:integer;
begin
 case ADKScreen.Depth of
  15: begin
       fPal16:=nil;
       if fPal15=nil then begin
        SetLength(fPal15,fPalCount);
        for i:=0 to fPalCount-1 do begin
         for j:=0 to 255 do fPal15[i,j]:=cvt15(fPal32[i,j]);
        end;
       end;
       Result:=@fPal15[Index];
      end;
  16: begin
       fPal15:=nil;
       if fPal16=nil then begin
        SetLength(fPal16,fPalCount);
        for i:=0 to fPalCount-1 do begin
         for j:=0 to 255 do fPal16[i,j]:=cvt16(fPal32[i,j]);
        end;
       end;
       Result:=@fPal16[Index];
      end;
  32: begin
       fPal15:=nil;
       fPal16:=nil;
       Result:=@fPal32[Index];
      end;
  else Result:=nil;
 end;
end;

function TADKSprite.Fill32(Direction:TDirection; Frame:Cardinal; dib,pal:pointer;pitch:integer):integer;
var
 FRM:PFrameHeader;
 SRC:PFrameHeader;
begin
 FRM:=@fBase[fOffsets[fHeader.Sprites[Direction].First+Frame]];
 Result:=FRM^.Delay;
 SRC:=FRM; inc(SRC);
 XDraw(SRC,@pchar(dib)[0],FRM.Height,pitch,1,pal^);
end;

function TADKSprite.Fill32Rect(Direction:TDirection; Frame:Cardinal; dib,pal:pointer;pitch:integer;Rect:TRect):integer;
var
 FRM:PFrameHeader;
 SRC:PFrameHeader;
begin
 FRM:=@fBase[fOffsets[fHeader.Sprites[Direction].First+Frame]];
 Result:=FRM^.Delay;
 SRC:=FRM; inc(SRC);
 XDrawRect(SRC,@pchar(dib)[0],pitch,1,Rect,pal^);
end;

function TADKSprite.Alpha32(Direction:TDirection; Frame:Cardinal; dib,pal:pointer;pitch:integer):integer;
var
 FRM:PFrameHeader;
 SRC:PFrameHeader;
begin
 FRM:=@fBase[fOffsets[fHeader.Sprites[Direction].First+Frame]];
 Result:=FRM^.Delay;
 SRC:=FRM; inc(SRC);
 XDrawBlend(SRC,@pchar(dib)[0],FRM.Height,pitch,1,pal^);
end;

function TADKSprite.Alpha32Rect(Direction:TDirection; Frame:Cardinal; dib,pal:pointer; pitch:integer; Rect:TRect):integer;
var
 FRM:PFrameHeader;
 SRC:PFrameHeader;
begin
 FRM:=@fBase[fOffsets[fHeader.Sprites[Direction].First+Frame]];
 Result:=FRM^.Delay;
 SRC:=FRM; inc(SRC);
 XDrawBlendRect(SRC,@pchar(dib)[0],pitch,1,Rect,pal^);
end;

function TADKSprite.Draw(Direction:TDirection; Frame,x,y:integer; DC:HDC):integer;
type
 TFullBitmapInfoHeader=packed record
  bmiHeader:TBitmapInfoHeader;
  bmiColors:array[0..255] of integer;
 end;
var
 FBI:TFullBitmapInfoHeader;
  BI:TBitmapInfo absolute FBI;
 DIB:PChar;
 SPR:PChar;
 SPW:^word absolute SPR;
 SPB:^byte absolute SPR;
 SPL:^integer absolute SPR;
 PAL:PPalette32;

begin
 GetMem(DIB,fHeader.Width*fHeader.Height);
 try
  Result:=DrawDIB(Direction,Frame,DIB);
  PAL:=@fBase[fHeader.Sprites[Direction].Colors];
  FillChar(FBI,SizeOf(FBI),0);
  with FBI do begin
   bmiHeader.biSize:=SizeOf(FBI.bmiHeader);
   bmiHeader.biPlanes:=1;
   bmiHeader.biWidth:=fHeader.Width;
   bmiHeader.biHeight:=fHeader.Height;
   bmiHeader.biBitCount:=8;
   bmiHeader.biCompression:=BI_RGB;
   bmiHeader.biSizeImage:=fHeader.Width*fHeader.Height;
   bmiHeader.biClrImportant:=256;
   move(PAL^,bmiColors,4*256);
  end;
  SetDIBitsToDevice(
   DC,
   x,y,fHeader.Width,fHeader.Height,
   0,0,0,fHeader.Height,
   DIB,BI,DIB_RGB_COLORS
  );
 finally
  FreeMem(DIB);
 end;
end;

function TADKSprite.DrawDIB(Direction:TDirection; Frame:Cardinal; dib:pointer):integer;
var
 FRM:^TFrameHeader;
begin
 FRM:=@fBase[fOffsets[fHeader.Sprites[Direction].First+Frame]];
 Result:=FRM.Delay; inc(FRM);
 FillChar(dib^,fHeader.Width*fHeader.Height,fHeader.Sprites[Direction].Trans);
 Draw8(FRM,dib,fHeader.Height,fHeader.Width,1);
end;

function TADKSprite.DrawRGB32(Direction:TDirection; Frame:Cardinal; dib:pointer;pitch:integer):integer;
var
 ofs:Cardinal;
 PAL:PPalette32;
 SPB:^byte;
begin
 ofs:=fOffsets[fHeader.Sprites[Direction].First+Frame]+SizeOf(TFrameHeader);
 PAL:=@fBase[fHeader.Sprites[Direction].Colors];
 SPB:=@fBase[ofs];
 FillLong(dib^,fHeader.Width*fHeader.Height,PAL[SPB^]);
 Result:=Fill32(Direction,Frame,dib,PAL,pitch);
end;

function TADKSprite.FillRGB32(Direction:TDirection; Frame:Cardinal; dib:pointer; pitch:integer):integer;
begin
 Result:=Fill32(Direction,Frame,dib,GetPal(fHeader.Sprites[Direction].Colors),pitch);
end;

function TADKSprite.FillRGB32Rect(Direction:TDirection; Frame:Cardinal; dib:pointer; pitch:integer; const Rect:TRect):integer;
begin
 Result:=Fill32Rect(Direction,Frame,dib,GetPal(fHeader.Sprites[Direction].Colors),pitch,Rect);
end;

function TADKSprite.AlphaRGB32(Direction:TDirection; Frame:Cardinal; dib:pointer; pitch:integer):integer;
begin
 Result:=Alpha32(Direction,Frame,dib,GetPal(fHeader.Sprites[Direction].Colors),pitch);
end;

function TADKSprite.AlphaRGB32Rect(Direction:TDirection; Frame:Cardinal; dib:pointer; pitch:integer;const Rect:TRect):integer;
begin
 Result:=Alpha32Rect(Direction,Frame,dib,GetPal(fHeader.Sprites[Direction].Colors),pitch,Rect);
end;

procedure TADKSprite.BorderRect(Direction:TDirection; Frame:Cardinal; dib:pointer; pitch:integer;const Rect:TRect);
var
 FRM:PFrameHeader;
 SRC:PFrameHeader;
begin
 FRM:=@fBase[fOffsets[fHeader.Sprites[Direction].First+Frame]];
 SRC:=FRM; inc(SRC);
 XBorderRect(SRC,@pchar(dib)[0],pitch,1,Rect);
end;

procedure TADKSprite.Border(Direction:TDirection; Frame:Cardinal; dib:pointer; pitch:integer);
var
 FRM:PFrameHeader;
 SRC:PFrameHeader;
begin
 FRM:=@fBase[fOffsets[fHeader.Sprites[Direction].First+Frame]];
 SRC:=FRM; inc(SRC);
 XBorder(SRC,@pchar(dib)[0],FRM.Height,pitch,1);
end;

function TADKSprite.TransparentColor(Direction:TDirection):integer;
var
 ofs:Cardinal;
 SPR:^Byte;
 PAL:PPalette32;
begin
 ofs:=fOffsets[fHeader.Sprites[Direction].First];
 SPR:=@fBase[ofs+SizeOf(TFrameHeader)];
 PAL:=@fBase[fHeader.Sprites[Direction].Colors];
 Result:=PAL[SPR^];
end;

function TADKSprite.Palette(Direction:TDirection):PPalette32;
begin
 Result:=@fPal32[fHeader.Sprites[Direction].Colors];
end;

function TADKSprite.Width:integer;
begin
 Result:=fHeader.Width;
end;

function TADKSprite.Height:integer;
begin
 Result:=fHeader.Height;
end;

function TADKSprite.SpriteFrames(Direction:TDirection):PSpriteFrames;
begin
 Result:=@fHeader.Sprites[Direction];
end;

function TADKSprite.FrameHeader(Direction:TDirection; Frame:integer):PFrameHeader;
begin
 Result:=@fBase[fOffsets[fHeader.Sprites[Direction].First+Frame]];
end;

procedure TADKSprite.Reduice;
var
 r:TRect;
 d:TDirection;
 f:integer;
 frm:PFrameHeader;
begin
 r.Top:=fHeader.Height;
 r.Left:=fHeader.Width;
 r.Right:=0;
 r.Bottom:=0;
 for d:=Low(TDirection) to high(TDirection) do begin
  for f:=0 to fHeader.Sprites[d].Count-1 do begin
   frm:=FrameHeader(d,f);
   if frm.Top<r.Top then r.Top:=frm.Top;
   if frm.Left<r.Left then r.Left:=frm.Left;
   if frm.Left+frm.Width>r.Right then r.Right:=frm.Left+frm.Width;
   if frm.Top+frm.Height>r.Bottom then r.Bottom:=frm.Top+frm.Height;
  end;
 end;
 fHeader.Width:=r.Right-r.Left+1;
 fHeader.Height:=r.Bottom-r.Top+1;
 for d:=Low(TDirection) to high(TDirection) do begin
  with fHeader.Sprites[d].Origin do begin
   dec(x,r.Left);
   dec(y,r.Top);
  end;
  for f:=0 to fHeader.Sprites[d].Count-1 do begin
   frm:=FrameHeader(d,f);
   dec(frm.Top,r.Top);
   dec(frm.Left,r.Left);
  end;
 end;
end;

end.
