unit ADKWalkers;

interface

uses
 Windows,Types,
 ADKObjects,
 ADKMaps,ADKSprites,ADKPathFinder,ADKIniLoader,ADKScreens;

const
 HalfTile=24;
 Walkable=[TobsStandard..TobsSand];

type
 TOnCellType=procedure(Sender:TObject; x,y:integer; var Obstable:TObstacle) of object;

 TADKWalkableMap=class(TADKMap,IMap)
  RefCount:integer;
  fOnCellType:TOnCellType;
  procedure FixPos(var x,y:integer);
 // IUnknown
  function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
  function _AddRef: Integer; stdcall;
  function _Release: Integer; stdcall;
 // IMap (pour ADKPathFinder)
 {function Width :integer;}
 {function Height:integer;}
  function CanWalk(const Pos:TPoint):boolean;
 end;

 TWalker=class;

 TWalkerStart=procedure(Walker:TWalker; const Target:TPoint; How:TAction; var Cancel:boolean);
 TWalkerMove=procedure(Walker:TWalker; const Leave,Enter:TPoint);
 TWalkerStop=procedure(Walker:TWalker; const Target:TPoint);
 TRotateEvent=procedure(Walker:TWalker; Dir:TDirection);

 TWalker=class(TADKObject)       // informations sur un sprite donn
  Sprites:TSprites;
  Action :TAction;
  Image  :TADKSprite;
  Dir    :TDirection;
  Head   :TDirection;
  Cell   :TPoint;      // position sur la carte (en case)
  Pos    :TPoint;      // position sur la carte (en pixels)
  Next   :TPoint;      // prochaine position sur la carte (en case)
  Finder :TADKPathFinder;
  Time   :cardinal;
  Frame  :integer;
  fTag   :integer;
  FrameHeader:PFrameHeader;
  SpriteFrames:PSpriteFrames;
  Target,NewTarget:TPoint;
  NewAction:TAction;
  Step:integer;
  Speed:integer;
  Walk:TPoint;
  HalfStep:boolean;
  constructor Create(x,y:integer; Skin:TSprites; ADir:TDirection);
  destructor Destroy; override;
  procedure Draw; override;
  procedure DrawAlpha;
  procedure DrawBorder;
  procedure SetSkin(Value:TSprites);
  function BaseLine:integer; override;
  procedure Update(ms:cardinal); override;
  procedure SetFrame(Value:integer);
  procedure SetAction(Value:TAction; Loop:boolean=False);
  procedure SetDir(Value:TDirection);
  procedure OnClick(x,y,button:integer);
  procedure Go(const ATarget:TPoint; How:TAction);
  procedure IA;
  procedure Change;
  procedure LookMouse;
 private
  EOnStart:TWalkerStart;
  EOnMove :TWalkerMove;
  EOnStop :TWalkerStop;
  EOnRotate:TRotateEvent;
  fRotate :boolean;
  fText   :string;
  fLoop   :boolean; // loop animation
  procedure SetText(Value:string);
  function GetText:string;
 public
  function Top:integer;
  function Left:integer;
  function Height:integer;
  function FrameTop:integer;
  function FrameLeft:integer;
  function Origin:TPoint;
  procedure StepIt;
  procedure SetPos(x,y:integer);
  function ScreenToCell(x,y:integer):TPoint;
  function Light(lx,ly:single):integer; override;
  function CanHit(Target:TWalker; var Direction:TDirection):boolean;
  property Tag:integer read fTag write fTag;
  property OnStart:TWalkerStart read EOnStart write EOnStart;
  property OnMove:TWalkerMove  read EOnMove write EOnMove;
  property OnStop:TWalkerStop read EOnStop write EOnStop;
  property Text:string read GetText write SetText;
  property OnRotate:TRotateEvent read EOnRotate write EOnRotate;
 end;

var
 MousePos :TPoint;
 MouseHint:TWalker;

function GridToScreen(Const Pos:TPoint):TPoint;

implementation

uses
 ADKData,ADKDepth,ADKRender;

const
// TDirection=(dirN,dirNE,dirE,dirSE,dirS,dirSW,dirW,dirNW);
 Delta:array[0..7,0..1] of integer=(
  (-1,-1),(+0,-1),(+1,-1),(+1,+0),
  (+1,+1),(+0,+1),(-1,+1),(-1,+0)
 );

function ScreenToGrid(const Pos:TPoint):TPoint;
var
 x,y:integer;
begin
 x:=Pos.y+Pos.x div 2;
 y:=Pos.y*2-Pos.x;
 Result.x:=x div 48;
 Result.y:=y div 96;
end;

function ScreenToHalfGrid(const Pos:TPoint):TPoint;
var
 x,y:integer;
begin
 x:=Pos.y+Pos.x div 2;
 y:=Pos.y*2-Pos.x;
 Result.x:=x div 24;
 Result.y:=y div 48;
end;

function GridToScreen(Const Pos:TPoint):TPoint;
begin
 Result.x:=48*(Pos.x-Pos.y);
 Result.y:=24*(Pos.x+Pos.y);
end;

function AddPoint(const P:TPoint; x,y:integer):TPoint;
begin
 Result.X:=P.X+X;
 Result.Y:=P.Y+Y;
end;

procedure TADKWalkableMap.FixPos(var x,y:integer);
begin
 while not (Cells[x,y].Obstacle in Walkable) do begin
  inc(x);
  if x=Width then begin
   x:=0;
   inc(y);
   if y=Height then y:=0;
  end;
 end;
end;

function TADKWalkableMap.QueryInterface(const IID: TGUID; out Obj): HResult;
begin
 if GetInterface(IID, Obj) then Result:=0 else Result:=E_NOINTERFACE;
end;

function TADKWalkableMap._AddRef: Integer;
begin
 inc(RefCount);
 Result:=RefCount;
end;

function TADKWalkableMap._Release: Integer;
begin
 dec(RefCount);
 Result:=RefCount;
 if Result=0 then Destroy;
end;

function TADKWalkableMap.CanWalk(const Pos:TPoint):boolean;
var
 o:TObstacle;
begin
 o:=Cells[Pos.x,Pos.Y].Obstacle;
 if Assigned(fOnCellType) then fOnCellType(Self,Pos.x,Pos.y,o);
 result:=o in Walkable;
end;

constructor TWalker.Create(x,y:integer; Skin:TSprites; ADir:TDirection);
begin
 inherited Create;
 WalkerList.Add(Self);
 SetPos(x,y);
 Dir:=ADir;
 SetSkin(Skin);
 Finder:=TADKPathFinder.Create(Map);
 Speed:=1500; // 2000 
end;

destructor TWalker.Destroy;
begin
 WalkerList.Remove(Self);
 if Focus=Self then Focus:=nil;
 inherited;
end;

procedure TWalker.SetPos(x,y:integer);
begin
 Cell.x:=x;
 Cell.y:=y;
 Pos:=GridToScreen(Cell);
 Walk.x:=0;
 Walk.y:=0;
end;

procedure TWalker.SetSkin(Value:TSprites);
begin
 Sprites:=Value;
 Image:=Sprites.Animations[Action];
 Frame:=1;
 SetFrame(0);
end;

function TWalker.Left:integer;
begin
 Result:=Pos.X+Walk.X;
end;

function TWalker.Top:integer;
begin
 Result:=Pos.Y+Walk.Y+HalfTile;
end;

function TWalker.Height:integer;
begin
 Result:=Image.Height;
end;

function TWalker.FrameLeft:integer;
begin
 Result:=FrameHeader.Left;
end;

function TWalker.FrameTop:integer;
begin
 Result:=FrameHeader.Top+HalfTile;
end;

function TWalker.Origin:TPoint;
begin
 Result:=SpriteFrames.Origin;
end;

procedure TWalker.Draw;
begin
 if Image=nil then exit;
 with FrameHeader^ do begin
  DrawX:=Pos.X+Walk.X+ScrollX+Left;
  DrawY:=Pos.Y+Walk.Y+ScrollY+Top+HalfTile;
  DrawSize.X:=Width;
  DrawSize.Y:=Height;
 end;
 with SpriteFrames^ do begin
  dec(DrawX,Origin.X);
  dec(DrawY,Origin.Y);
 end;
 if (MouseHint=nil)and(MousePos.x>DrawX)and(MousePos.y>DrawY)
 and(MousePos.x<DrawX+DrawSize.X)and(MousePos.y<DrawY+DrawSize.Y) then begin
  MouseHint:=Self;
 end;
 if ClipRect then
  Image.FillRGB32Rect(dir,Frame,ADKScreen.Pixels[DrawX,DrawY],ADKScreen.Pitch,DrawRect)
 else
  Image.FillRGB32(dir,Frame,ADKScreen.Pixels[DrawX,DrawY],ADKScreen.Pitch);
end;

procedure TWalker.DrawAlpha;
begin
 if Image=nil then exit;
 with FrameHeader^ do begin
  DrawX:=Pos.X+Walk.X+ScrollX+Left;
  DrawY:=Pos.Y+Walk.Y+ScrollY+Top+HalfTile;
  DrawSize.X:=Width;
  DrawSize.Y:=Height;
 end;
 with SpriteFrames^ do begin
  dec(DrawX,Origin.X);
  dec(DrawY,Origin.Y);
 end;
 if ClipRect then
  Image.AlphaRGB32Rect(dir,Frame,ADKScreen.Pixels[DrawX,DrawY],ADKScreen.Pitch,DrawRect)
 else
  Image.AlphaRGB32(dir,Frame,ADKScreen.Pixels[DrawX,DrawY],ADKScreen.Pitch);
end;

procedure TWalker.DrawBorder;
begin
 if Image=nil then exit;
 with FrameHeader^ do begin
  DrawX:=Pos.X+Walk.X+ScrollX+Left;
  DrawY:=Pos.Y+Walk.Y+ScrollY+Top+HalfTile;
  DrawSize.X:=Width;
  DrawSize.Y:=Height;
 end;
 with SpriteFrames^ do begin
  dec(DrawX,Origin.X);
  dec(DrawY,Origin.Y);
 end;
 if ClipRect then
  Image.BorderRect(dir,Frame,ADKScreen.Pixels[DrawX,DrawY],ADKScreen.Pitch,DrawRect)
 else
  Image.Border(dir,Frame,ADKScreen.Pixels[DrawX,DrawY],ADKScreen.Pitch);
end;

function TWalker.BaseLine:integer;
begin
 if Image=nil then Result:=0 else Result:=Pos.Y+Walk.Y+HalfTile+FrameHeader.Top;
end;

procedure TWalker.LookMouse;
const
 CantLook:array[TDirection] of TDirection=(
 // N   NE    E    SE    S    SW    W    NW
  DirS,DirSW,DirW,DirNW,DirN,DirNE,DirE,DirSE
 );
var
 dx,dy:integer;
 d:TDirection;
begin
 dx:=MousePos.X-ADKScreen.Width div 2;
 dy:=ADKScreen.Height div 2-MousePos.Y;

 if dx>0 then begin
  if dy>0 then begin
   if dy>2*dx then d:=DirN else
   if dx>2*dy then d:=DirE else d:=DirNE;
  end else begin
   if -dy>2*dx then d:=DirS else
   if dx>-2*dy then d:=DirE else d:=DirSE;
  end;
 end else begin
  if dy>0 then begin
   if dy>-2*dx then d:=DirN else
   if -dx>2*dy then d:=DirW else d:=DirNW;
  end else begin
   if -dy>-2*dx then d:=DirS else
   if -dx>-2*dy then d:=DirW else d:=DirSW;
  end;
 end;

 if Action=aStand then begin
  Dir:=d;
  Change;
 end;

 if (d<>CantLook[Dir])and(d<>Head) then Head:=d;
end;

procedure TWalker.Update(ms:cardinal);
const
 Deltas:array[TDirection,0..1] of integer=(
 // dirN,dirNE,dirE,dirSE,dirS,dirSW,dirW,dirNW
  (0,-48),(48,-24),(96,0),(48,24),(0,48),(-48,24),(-96,0),(-48,-24)
 );
 DeltaCells:array[TDirection,0..1] of integer=(
 // dirN,dirNE,dirE,dirSE,dirS,dirSW,dirW,dirNW
  (-1,-1),(0,-1),(+1,-1),(+1,0),(+1,+1),(0,+1),(-1,+1),(-1,0)
 );
var
 p,q:TPoint;
 Cancel:boolean;
 Tempo:integer;
 f:integer;
begin
// vitesse de dplacement (temps ncessaire en ms au passage d'une case  une autre)
 Tempo:=Speed;
 if odd(ord(Dir)+1) then Tempo:=Round(1.4*Tempo);
 if Action=aRun then Tempo:=Tempo div 2;
 if Tempo<=0 then Tempo:=1;
// cumul du temps
 inc(Time,ms);
// sur place, appliquer le Delay de l'animation
 if (FrameHeader.MoveX=0) and (FrameHeader.MoveY=0) then begin
  while (Time>FrameHeader.Delay)and(FrameHeader.Delay>0) do begin
   dec(Time,FrameHeader.Delay);
   SetFrame(Frame+1);
  end;
  if (Focus=Self) then begin
   if (GetKeyState(VK_CONTROL)and $80>0) then begin
    fRotate:=True;
    LookMouse;
   end else begin
    if fRotate then begin
     fRotate:=False;
     if Assigned(EOnRotate) then EOnRotate(Self,Dir);
    end;
   end;
  end;
  exit;
 end;
// milieu de case
 if HalfStep and (Time>Tempo div 2) and Assigned(EOnMove) then begin
  HalfStep:=False;
  EOnMove(Self,Cell,Next);
 end;
// a quelle distance en somme nous ?
 p.x:=abs(Deltas[Dir,0]*Time) div Tempo;
 p.y:=abs(Deltas[Dir,1]*Time) div Tempo;
// moins ce que nous avons dj parcouru
 dec(p.x,abs(Walk.x));
 dec(p.y,abs(Walk.y));
// avons nous fait un pas ?
 while (p.x>=abs(FrameHeader.MoveX))and(p.y>=abs(FrameHeader.MoveY)) do begin
  dec(p.x,abs(FrameHeader.MoveX));
  dec(p.y,abs(FrameHeader.MoveY));
  inc(Walk.X,FrameHeader.MoveX);
  inc(Walk.Y,FrameHeader.MoveY);
  // nouvelle frame = nouveau pas
  SetFrame(Frame+1);
 end;
// il nous faut 2 secondes pour parcourir la distance totale...
 while (Time>=Tempo) do begin
  dec(Time,Tempo);
 // changement de case
  inc(Cell.x,DeltaCells[Dir,0]);
  inc(Cell.y,DeltaCells[Dir,1]);
  Pos:=GridToScreen(Cell);
  // changement de direction ?
  if NewAction<>aStand then begin
   SetAction(aStand);
   Go(NewTarget,NewAction);
   NewAction:=aStand;
   exit;
  end;
  // ajustement des dplacements
  Dec(Walk.X,Deltas[Dir,0]);
  Dec(Walk.Y,Deltas[Dir,1]);
  // prochain dplacement
  StepIt;
  // nouveau dplacement
  if Action=aStand then exit;

  if  Assigned(EOnStart) then begin
   Cancel:=False;
   EOnStart(Self,Next,Action,Cancel); // nouvelle destination
   if Cancel then begin
    SetAction(aStand);
    exit;
   end;
  end;
 end;
(*
var
 p:TPoint;
 Cancel:boolean;
begin
 inc(Time,ms);
 while (Time>Delay)and(Delay>0) do begin
  dec(Time,Delay);
  if (Focus=Self) then begin
   if (GetKeyState(VK_CONTROL)and $80>0) then begin
    fRotate:=True;
    LookMouse;
   end else begin
    if fRotate then begin
     fRotate:=False;
     if Assigned(EOnRotate) then EOnRotate(Self,Dir);
    end;
   end;
  end;
 // en mouvement ?
  if Action<>aStand then begin

   inc(Walk.x,FrameHeader.MoveX);
   inc(Walk.y,FrameHeader.MoveY);

  // milieu de case
   if HalfStep and Assigned(EOnMove) then begin
    p:=ScreenToHalfGrid(Walk);
    if (p.x<>0)or(p.y<>0) then begin
     HalfStep:=False;
     EOnMove(Self,Cell,Next);
    end;
   end;

  // changement de case
   p:=ScreenToGrid(Walk);
   if (p.x<>0)or(p.y<>0) then begin
    inc(Cell.x,p.x);
    inc(Cell.Y,p.y);

    HalfStep:=True;
//    if Assigned(EOnStop) then EOnStop(Self,Cell); // nous sommes arriv

    Pos:=GridToScreen(Cell);
    p:=GridToScreen(p);
    dec(Walk.x,p.x);
    dec(Walk.y,p.y);

   // changement d'action en cours de route...
    if NewAction<>aStand then begin
     SetAction(aStand);
     Go(NewTarget,NewAction);
     NewAction:=aStand;
     exit;
    end;

    StepIt;

    if (Action<>aStand)and Assigned(EOnStart) then begin
     Cancel:=False;
     EOnStart(Self,Next,Action,Cancel); // nouvelle destination
     if Cancel then SetAction(aStand);
    end;

   end;

  end;
  SetFrame(Frame+1);
 end;
 *)
end;

procedure TWalker.StepIt;
begin
 inc(Step);
 // fin d'une ligne "droite"
 if Step>=Finder.PathLen then begin
  if Finder.EndOfPath then begin
   SetAction(aStand);
   if Assigned(EOnStop) then EOnStop(Self,Cell); // nous sommes arriv
   exit;
  end;
  Step:=0;
  Target:=Finder.NextPoint;
 end;
 SetDir(TDirection(Finder.PathDir(Step) and 7));
end;

procedure TWalker.Change;
begin
 if Frame>=Image.FrameCount(Dir) then begin
  Frame:=0;
  if (Action=aHit)and(fLoop=False) then SetAction(aStand);
 end;
 SpriteFrames:=Image.SpriteFrames(Dir);
 FrameHeader:=Image.FrameHeader(Dir,Frame);
end;

procedure TWalker.SetFrame(Value:integer);
begin
 if Value<>Frame then begin
  Frame:=Value;
  Change;
 end;
end;

procedure TWalker.SetAction(Value:TAction; Loop:boolean=False);
begin
 fLoop:=Loop;
 if Value<>Action then begin
  Action:=Value;
  Walk.x:=0;
  Walk.y:=0;
  Image:=Sprites.Animations[Action];
  Frame:=0;
  Change;
 end;
end;

procedure TWalker.SetDir(Value:TDirection);
begin
 if Value<>Dir then begin
  Dir:=Value;
  Head:=Value;
  Change;
  Walk.x:=0;
  Walk.y:=0;
 end;
 Next:=AddPoint(Cell,Delta[integer(dir),0],Delta[integer(dir),1]);
end;

function TWalker.ScreenToCell(x,y:integer):TPoint;
begin
 Result:=AddPoint(Pos,x-ADKScreen.Width div 2+Walk.x,y-ADKScreen.Height div 2+HalfTile+Walk.y);
 Result:=ScreenToGrid(Result);
end;

procedure TWalker.OnClick(x,y,button:integer);
var
 p:TPoint;
begin
// p:=AddPoint(Pos,x-HalfWidth,y-HalfHeight+HalfTile);
// p:=ScreenToGrid(p);
 p:=ScreenToCell(x,y);
 if (p.x<0)or(p.y<0)or(p.x>=Map.Width)or(p.y>=Map.Height) then exit; // en dehors de la carte
 if not Map.CanWalk(p) then exit;
 if button=0 then go(p,aWalk) else go(p,aRun);
end;

procedure TWalker.Go(const ATarget:TPoint; How:TAction);
var
 Cancel:boolean;
begin
 if Action in [aWalk,aRun] then begin // si on est en cours de dplacement...
  NewTarget:=ATarget; // ...programmer une changement de trajectoire
  NewAction:=How;
  exit;
 end;
 if Finder.FindPath(Cell,ATarget) then begin
  Target:=Finder.NextPoint;
  Step:=0;
  if Finder.PathLen>0 then SetDir(TDirection(Finder.PathDir(Step) and 7));
  if Assigned(EOnStart) then begin
   Cancel:=False;
   EOnStart(Self,Next,How,Cancel);
   if Cancel then exit;
  end;
  Time:=0;
  HalfStep:=True;
  SetAction(How);
 end;
end;

procedure TWalker.IA;
begin
 if (Action=aStand) then begin
  if Random(100)<10 then // 10% de chance
   Go(Point(Random(Map.Width),Random(Map.Height)),TAction(Random(2)+1));
 end;
end;

function TWalker.Light(lx,ly:single):integer;
const
 delta:array[TDirection,0..1] of integer=(
  (0,+4),( 3, 2),( 4,0),( 2,-3),
  (0,-4),(-3,-2),(-4,0),(-2,+3)
 );
 baseLight=16;
var
 l:double;
begin
 with FrameHeader^ do begin
  lx:=lx{-48*delta[Head,0]}-(Pos.X+Walk.X+ScrollX+Left);
  ly:=ly{+48*delta[Head,1]}-(Pos.Y+Walk.Y+ScrollY+Top+HalfTile);
 end;
 l:=sqrt(lx*lx+ly*ly);
{
 if l=0 then result:=255 else
 if l>255 then result:=0 else begin
  result:=round(255-l)+BaseLight;
  if result>255 then result:=255;
 end;
}
 result:=255-round((255*l)/256);
 if result<0 then result:=0;
 if result>255 then result:=255;
end;

procedure TWalker.SetText(Value:string);
begin
 fText:='         '+Value;
end;

function TWalker.GetText:string;
begin
 if fText='' then
  Result:=''
 else begin
  Result:=copy(fText,1,10);
  Delete(fText,1,1);
 end;
end;

function TWalker.CanHit(Target:TWalker; var Direction:TDirection):boolean;
const
 Directions:array[-1..+1,-1..+1] of TDirection=(
  (DirS ,DirSE,DirE ), { dx = -1 }
  (DirSW,dirn ,DirNE), { dx =  0 }
  (DirW ,DirNW,DirN )  { dx = +1 }
 );
var
 dx,dy:integer;
begin
 Result:=False;
 if Action=aHit then exit;
 dx:=Cell.x-Target.Cell.x;
 if abs(dx)>1 then exit;
 dy:=Cell.y-Target.Cell.y;
 if abs(dy)>1 then exit;
 if (dx=0)and(dy=0) then exit;
 Direction:=Directions[dx,dy];
 Result:=True;
end;

end.
