unit ADKServer;

{
 Serveur pour le projet ADK-ISO/3D (c)2002 Paul TOTH <tothpaul@free.fr>

 http://www.web-synergy.net/naug-land/

}
interface

uses
  Windows, Classes, SysUtils, ADKSession;

type
  TSessionReadEvent = procedure(Sender: TObject; Session: TADKSession; Count: integer) of object;
  TClientReadEvent = procedure(Sender: TObject; Count: integer) of object;

  TErrorEvent = procedure(Sender: TObject; Msg: string) of object;

  TADKError = class(TComponent)
  private
    EOnError: TErrorEvent;
  protected
    procedure Error(s: string);
    procedure FatalError(s: string); virtual;
  published
    property OnError: TErrorEvent read EOnError write EOnError;
  end;

  TADKServer = class(TCustomADKServer)
  private
    fDispatchers: TList;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Start; // dmarrer le serveur
    procedure Close; // refuser les nouveaux clients
    procedure Stop; // arrter le serveur
  end;

  TADKDispatcher = class(TADKError)
  private
    fServer: TADKServer;
    EOnEnter: TSessionEvent;
    EOnLeave: TSessionEvent;
    EOnRead: TSessionReadEvent;
    procedure SetActive(Value: boolean);
    procedure SetServer(Value: TADKServer);
  protected
    fSessions: TList;
    EOnStart: TNotifyEvent;
    EOnStop: TNotifyEvent;
    function GetActive: boolean; virtual; abstract;
    procedure FatalError(s: string); override;
    procedure StartSession(Session: TADKSession);
  //procedure CloseSession(Session:TADKSession);
    procedure CloseSessions; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Start; virtual; abstract;
    procedure Close; virtual; abstract;
    procedure Stop; virtual;
    property Active: boolean read GetActive write SetActive;
    property ListSessions: TList read fSessions;
    procedure CloseSession(Session: TADKSession);
  published
    property Server: TADKServer read fServer write SetServer;
    property OnStart: TNotifyEvent read EOnStart write EOnStart;
    property OnStop: TNotifyEvent read EOnStop write EOnStop;
    property OnEnter: TSessionEvent read EOnEnter write EOnEnter;
    property OnLeave: TSessionEvent read EOnLeave write EOnLeave;
    property OnRead: TSessionReadEvent read EOnRead write EOnRead;
  end;

  TCustomADKSession = class(TADKSession)
  protected
    fDispatcher: TADKDispatcher;
    fName: string;
  public
    constructor Create(AOwner: TADKDispatcher);
    procedure Error(msg: string); override;
    procedure Kill; virtual;
    procedure Close; virtual;
    property Name: string read fName;
  end;

  TADKProvider = class; // forward

  TCustomADKClient = class(TADKError)
  private
    fProvider: TADKProvider;
    procedure SetProvider(Value: TADKProvider);
  public
    destructor Destroy; override;
    procedure FatalError(msg: string); override;
    procedure Decode(out Data); virtual; abstract;
  published
    property Provider: TADKProvider read fProvider write SetProvider;
  end;

  TADKProvider = class(TADKError)
  protected
    fClient: TCustomADKClient;
    EOnRead: TClientReadEvent;
    EOnConnect: TNotifyEvent;
    EOnClose: TNotifyEvent;
    procedure Kill; virtual; abstract;
  public
    destructor Destroy; override;
    procedure FatalError(msg: string); override;
    function Active: boolean; virtual; abstract;
    procedure Open; virtual; abstract;
    procedure Close; virtual; abstract;
    procedure Encode(out Data); virtual; abstract;
  published
    property OnRead: TClientReadEvent read EOnRead write EOnRead;
    property OnConnect: TNotifyEvent read EOnConnect write EOnConnect;
    property OnClose: TNotifyEvent read EOnClose write EOnClose;
  end;

procedure Register;

implementation

uses
  ADKClient, inifiles;

procedure Register;
begin
  RegisterComponents('MySoft.ADK', [TADKServer, TADKClient]);
end;

//----------------------------------------------------------------------------//

procedure TADKError.Error(s: string);
begin
  if Assigned(EOnError) then begin
    EOnError(Self, s);
  end else begin
    raise Exception.Create(s);
  end;
end;

procedure TADKError.FatalError(s: string);
begin
  if Assigned(EOnError) then EOnError(Self, s);
  raise Exception.Create(s);
end;

//----------------------------------------------------------------------------//

constructor TADKServer.Create(AOwner: TComponent);
begin
  inherited;
  fDispatchers := TList.Create;
end;

destructor TADKServer.Destroy;
var
  i: integer;
begin
// refuser les connexions
  Close;
// tuer toutes les sessions actives
  for i := fDispatchers.Count - 1 downto 0 do TADKDispatcher(fDispatchers[i]).CloseSessions;
// fin
  fDispatchers.Free;
  inherited;
end;

procedure TADKServer.Start;
var
  i: integer;
begin
// dmarrer tous les dispatcher
  for i := fDispatchers.Count - 1 downto 0 do TADKDispatcher(fDispatchers[i]).Start;
end;

procedure TADKServer.Close;
var
  i: integer;
begin
// refuser les connexions des diffrents Dispatcher
  for i := fDispatchers.Count - 1 downto 0 do TADKDispatcher(fDispatchers[i]).Close;
end;

procedure TADKServer.Stop;
var
  i: integer;
begin
// fermer tous les dispatchers
  for i := fDispatchers.Count - 1 downto 0 do TADKDispatcher(fDispatchers[i]).Stop;
end;

//----------------------------------------------------------------------------//

constructor TADKDispatcher.Create(AOwner: TComponent);
begin
  inherited;
  fSessions := TList.Create;
end;

destructor TADKDispatcher.Destroy;
begin
// refuser les connexions
  Close;
// tuer les sessions actives
  CloseSessions;
// supprimer le lien
  if fServer <> nil then fServer.fDispatchers.Remove(Self);
// fin
  fSessions.Free;
  inherited;
end;

procedure TADKDispatcher.FatalError(s: string);
{var i, j: integer;
  inifile: Tinifile;
  Strs: TStringList;
  Allconnecter: String;
 }
begin
  inherited;
  // mettre fin au dispatch en cas d'erreur

  (*Stop;

  inifile:= Tinifile.create('Adkserver.Log');
  try
    Strs:= TStringList.create;
    inifile.ReadSection ('FatalError', Strs);
    if assigned(fSessions) then
      inifile.writeString('FatalError', IntToStr(Strs.Count), DateTimeToStr(Now)+ ';'+
        'Message erreur: '+ s + ';'+
        'Nbr Connects: '+ inttostr(fSessions.Count))
    else
      inifile.writeString('FatalError', IntToStr(Strs.Count), DateTimeToStr(Now)+ ';'+
        'Message erreur: '+ s + ';'+
        'Nbr Connects: Aucun. fSessions not assigned');
  finally
    Inifile.free;
  end;
  *)
end;

procedure TADKDispatcher.CloseSessions;
var
  i: integer;
begin
  // tuer les sessions actives
  for i := fSessions.Count - 1 downto 0 do //TCustomADKSession(fSessions[i]).Close;
    CloseSession(fSessions[i]);
  fSessions.Clear;
end;

procedure TADKDispatcher.SetServer(Value: TADKServer);
begin
  if fServer = Value then exit;
  if fServer <> nil then begin
    Stop;
    fServer.fDispatchers.Remove(Self);
  end;
  fServer := Value;
  if fServer <> nil then fServer.fDispatchers.Add(Self);
end;

procedure TADKDispatcher.StartSession(Session: TADKSession);
begin
  fSessions.Add(Session);
  if Assigned(EOnEnter) then EOnEnter(Self, Session, 0);
end;

procedure TADKDispatcher.CloseSession(Session: TADKSession);
begin
  fSessions.Remove(Session);
  if Assigned(EOnLeave) then EOnLeave(Self, Session, 0);
end;

procedure TADKDispatcher.SetActive(Value: boolean);
begin
  if Value then Start else Stop;
end;
{
function TADKDispatcher.GetActive;
begin
end;

procedure TADKDispatcher.Start;
begin
end;

procedure TADKDispatcher.Close;
begin
end;
}

procedure TADKDispatcher.Stop;
begin
  if not Active then exit;
  Close;
  CloseSessions;
  if Assigned(EOnStop) then EOnStop(Self);
end;

//----------------------------------------------------------------------------//

constructor TCustomADKSession.Create(AOwner: TADKDispatcher);
begin
  inherited Create;
  fDispatcher := AOwner;
  fDispatcher.StartSession(Self);
end;

procedure TCustomADKSession.Kill;
begin
  fDispatcher.CloseSession(Self);
  Destroy;
end;

procedure TCustomADKSession.Close;
begin
  Destroy;
end;

procedure TCustomADKSession.Error(msg: string);
begin
  Kill;
  fDispatcher.FatalError(msg);
end;

//----------------------------------------------------------------------------//

destructor TCustomADKClient.Destroy;
begin
  Provider := nil;
  inherited;
end;

procedure TCustomADKClient.FatalError(msg: string);
begin
  if fProvider <> nil then fProvider.Kill;
  inherited;
end;

procedure TCustomADKClient.SetProvider(Value: TADKProvider);
begin
  if fProvider = Value then exit;
  if fProvider <> nil then begin
    fProvider.Close;
    fProvider.fClient := nil;
  end;
  fProvider := Value;
  if fProvider <> nil then fProvider.fClient := Self;
end;

//----------------------------------------------------------------------------//

destructor TADKProvider.Destroy;
begin
  Close;
  if fClient <> nil then fClient.fProvider := nil;
  inherited;
end;

procedure TADKProvider.FatalError(msg: string);
begin
  Kill;
  inherited;
end;
{
procedure Kill;
begin
 EOnClose(Self);
end;

function Active:boolean;
begin
end;

procedure Encode(out Data);
begin
end;

procedure Open;
begin
 EOnConnect(Self);
end;

procedure Close;
begin
end;
}

end.
