{-----------------------------------------------------------------------------
 Unit Name: ADKSession
 Author:    Manuel
 Purpose:   Envoy des donnes au client
 History:   A chaque Log d'une session le serveur cre un TADKSession.
            C'est cet objet qui permet d'envoy des donnes  la Session
            (et donc au client)
-----------------------------------------------------------------------------}


unit ADKSession;

{
 Session cliente pour le projet ADK-ISO/3D (c)2002 Paul TOTH <tothpaul@free.fr>
 http://www.web-synergy.net/naug-land/


}
interface

uses
  Classes, SysUtils, ADKNETData, definitions, Dialogs, personnage;

type
  TADKSession = class
  protected
  //fName:string;
    fPersosComplets: TPersosComplets;
    fPlayingGame: Boolean; // Le client est-il dja dans le jeu ou  "attend t-il  la porte" ?
    procedure Error(msg: string); virtual; abstract;
    procedure Encode(out data); virtual; abstract;
  public
    constructor create;
    destructor destroy; override;
  // Envoie de texte avec attribut (Pour le chat en thorie)
    procedure Send_TextAttr(const Ident: Cardinal; const Cmd: TServCmd; const TextAttr: TTextAttr; const Text: string);
  // Envoie d'un simple string
    procedure Send_TextInfo(const ID: Cardinal; const Cmd: TServCmd; const text: string);
    procedure Send_Negociate(Name: string; Version: integer; Banish: boolean);
    procedure Send_ServerInfo(StartTime, CurrentTime: TDateTime; Influence, Zonetampon, Speed: integer);
    //procedure Send_PlayerInfo(out PlayerInfo: TPlayerInfo);
    procedure Send_PlayerInfo(const ID: Cardinal; Value: TStream);
  //procedure Send_MiniPerso(const Cmd: TServCmd;const MiniPersoInfo: TRecMiniPerso);
    procedure Send_RawData(const ID: Cardinal; const Cmd: TServCmd; Value: TStream);
    procedure Send_StartGame;
  //Envoie d'un cardinal
    procedure Send_Cardinal(const IDent: Cardinal; const Cmd: TServCmd; const value: Cardinal);
  //envoie d'un message (texte avec attributs) vers un seul autre perso
  //On pourrait utiliser Send_TextAttr mais il lui manque la partie IDDestinataire
  //qui identifie EXACTEMENT a qui est detin le message
    procedure Send_TextAttr121(const Ident: Cardinal; const Cmd: TServCmd;
      const IDDestinataire: Cardinal; const TextAttr: TTextAttr; const Text: string);
  // Comme pour Send_TextAttr121 mais la c'est l'envoie de texte brut (sans attributs)
  // Correspond  Send_TextInfo mais en identifiant EXACTEMENT le destinataire
    procedure Send_TextInfo121(const ID: Cardinal; const Cmd: TServCmd;
      const IDDestinataire: Cardinal; const text: string);
  //Sert  toute les commandes qui doivent (de)placer  un personnage
    procedure Send_Position(const ID: Cardinal; const Cmd: TServCmd;
      const PositionInfo: TPositionInfo);
  // Envoie une commande qui n'a pas besoin de donnes (Ex: scStopMove)
    procedure Send_NoDataCmd(const ID: Cardinal; const Cmd: TServCmd);

    property ListPersos: TPersosComplets read fPersosComplets;
    property PlayingGame: boolean read fPlayingGame write fPlayingGame;
  end;

  TSessionEvent = procedure(Sender: TObject; Session: TADKSession;
    const ID: Cardinal) of object;
  TNoDataEvent = procedure(Sender: TObject; Session: TADKSession;
    const ID: Cardinal; const cmd: TServCmd) of object;
  TMessageEvent = procedure(Sender: TObject; Session: TADKSession;
    const ID: Cardinal; const Cmd: TServCmd; Msg: string; Attr: TTextAttr) of object;
  TNegociateEvent = procedure(Sender: TObject; Session: TADKSession;
    const ID: Cardinal; Negociate: TNegociate) of object;
  TStringEvent = procedure(Sender: TObject; Session: TADKSession;
    const ID: Cardinal; const Cmd: TServCmd; Msg: string) of object;
  //TRecivePJEvent = procedure(Sender: TObject; Session: TADKSession;
  //  const ID: Cardinal; const cmd: TServCmd; PlayerInfo: TPlayerInfo) of object;
  TLogonEvent = procedure(Sender: TObject; Session: TADKSession;
    const ID: Cardinal; Name, Password: string) of object;
  TCardinalEvent = procedure(Sender: TObject; Session: TADKSession;
    const ID: Cardinal; const Cmd: TServCmd; const Value: Cardinal) of object;
  TString21Event = procedure(Sender: TObject; Session: TADKSession;
    const ID: Cardinal; const Cmd: TServCmd; const IDDestinataire: Cardinal; Msg: string) of object;
  TMessageEvent21 = procedure(Sender: TObject; Session: TADKSession;
    const ID: Cardinal; const Cmd: TServCmd; const IDDestinataire: Cardinal; Msg: string; Attr: TTextAttr) of object;
  TPositionEvent = procedure(Sender: TObject; Session: TADKSession;
    const ID: Cardinal; const cmd: TServCmd; PositionInfo: TPositionInfo) of object;
  TRawDataEvent = procedure(Sender: TObject; Session: TADKSession;
    const ID: Cardinal; const cmd: TServCmd; RawData: TStream) of object;

  TCustomADKServer = class(TComponent)
  private
    EOnLogon: TLogonEvent;
    EOnLogout: TSessionEvent;
    EOnMessage: TMessageEvent;
    EOnNegociate: TNegociateEvent;
    EOnString: TStringEvent;
    EOnRecivePJ: TRawDataEvent;
    EOnCardinal: TCardinalEvent;
    EonString21: TString21Event;
    EOnMessage21: TMessageEvent21;
    EOnPosition: TPositionEvent;
    EOnNoData: TNoDataEvent;

  public
    procedure Decode(Session: TADKSession; out Data);
  published
    property OnLogon: TLogonEvent read EOnLogon write EOnLogon;
    property OnLogout: TSessionEvent read EOnLogout write EOnLogout;
    property OnMessage: TMessageEvent read EOnMessage write EOnMessage;
    property OnNegociate: TNegociateEvent read EOnNegociate write EOnNegociate;
    property OnString: TStringEvent read EOnString write EOnString;
    property OnRecivePJ: TRawDataEvent read EOnRecivePJ write EOnRecivePJ;
    property OnCardinal: TCardinalEvent read EOnCardinal write EOnCardinal;
    property onString21: TString21Event read EonString21 write EonString21;
    property OnMessage21: TMessageEvent21 read EOnMessage21 write EOnMessage21;
    property OnPosition: TPositionEvent read EOnPosition write EOnPosition;
    property OnNoData: TNoDataEvent read EOnNoData write EOnNoData;
  end;

implementation
uses ResStr, StrStuff;

constructor TADKSession.create;
begin
  inherited;
  fPersosComplets:= TPersosComplets.Create(nil);
  fPlayingGame:= false;
end;

destructor TADKSession.destroy;
begin
  fPersosComplets.Destroy;
  inherited;
end;

procedure TADKSession.Send_TextAttr(const Ident: Cardinal;
  const Cmd: TServCmd; const TextAttr: TTextAttr; const Text: string);
var
  Block: packed record
    FixedPart: record
      size: integer;
      Head: TBlockHeader;
      Attr: TTextAttr;
      Len: integer;
    end;
    OtherPart: array[0..1024] of char;
  end;
begin
  with Block.FixedPart do begin
    Head.ID:= IDent;
    Head.Cmd:= Cmd;
    Attr:= TextAttr;
    Len:= Length(Text);
    if len > 1024 then Len:= 1024;
    Move(Text[1], Block.OtherPart, len);
    Size:= SizeOf(Block.FixedPart) + 4 + Len;
  end;
  Encode(Block);
end;

procedure TADKSession.Send_TextInfo(const ID: Cardinal; const Cmd: TServCmd; const text: string);
var
  Block: packed record
    FixedPart: record
      size: integer;
      Head: TBlockHeader;
      Len: integer;
    end;
    OtherPart: array[0..MaxTxtInfo] of char;
  end;
begin
  with Block do begin
    with FixedPart do begin
      Size:= SizeOf(FixedPart) + 4 + Length(Text);
      Head.ID:= ID;
      Head.Cmd:= Cmd;
      Len:= Length(Text);
      if len > (MaxTxtInfo + 1) then Len:= MaxTxtInfo + 1;
    end;
    Move(Text[1], OtherPart, FixedPart.Len);
  end;
  Encode(Block);
end;

procedure TADKSession.Send_Negociate(Name: string; Version: integer; Banish: boolean);
var
  Block: record
    size: integer;
    head: TBlockHeader;
    data: TNegociate;
  end;
begin
  block.size:= SizeOf(block);
  block.head.ID:= 0;
  block.head.Cmd:= scNegociate;
  block.data.Server:= Name;
  block.data.Version:= 1;
  block.data.Banish:= Banish;
  block.data.Update:= version; // Version est egal  update en fait
  Encode(block);
end;

procedure TADKSession.Send_ServerInfo(StartTime, CurrentTime: TDateTime; Influence, ZoneTampon, Speed: integer);
var
  block: packed record
    size: integer;
    head: TBlockHeader;
    data: TServerInfo;
  end;
begin
  block.size:= SizeOf(block);
  block.head.ID:= 0;
  block.head.Cmd:= scSpecialDatas;
  block.data.StartDate:= StartTime;
  block.data.SysDate:= CurrentTime;
  block.data.Influence:= Influence;
  block.data.ZoneTampon:= ZoneTampon;
  block.data.Speed:= Speed;
  Encode(block);
end;

procedure TADKSession.Send_RawData(const ID: Cardinal; const Cmd: TServCmd; Value: TStream);
var MemoryStream: TmemoryStream;
  SizeStream: Integer;
begin
  MemoryStream:= TMemoryStream.Create;
  try
    MemoryStream.WriteBuffer(SizeStream, Sizeof(SizeStream));
    MemoryStream.WriteBuffer(Id, Sizeof(Id));
    MemoryStream.WriteBuffer(Cmd, Sizeof(cmd));
    MemoryStream.CopyFrom(Value, 0);
    MemoryStream.Position:= 0;
    SizeStream:= MemoryStream.size;
    MemoryStream.WriteBuffer(SizeStream, Sizeof(SizeStream));
    Encode(MemoryStream.Memory^);
  finally
    MemoryStream.destroy;
  end;
end;

procedure TADKSession.Send_PlayerInfo(const ID: Cardinal; Value: TStream);
begin
  Send_RawData(ID, scUpdatePClient, Value);
end;

//Envoie d'un cardinal

procedure TADKSession.Send_Cardinal(const IDent: Cardinal; const Cmd: TServCmd; const Value: Cardinal);
var
  block: packed record
    size: integer;
    head: TBlockHeader;
    data: Cardinal;
  end;
begin
  block.size:= sizeof(block);
  block.head.ID:= IDent;
  block.head.Cmd:= Cmd;
  block.data:= Value;
  Encode(block);
end;

procedure TADKSession.Send_StartGame;
var
  block: packed record
    size: integer;
    head: TBlockHeader;
  end;
begin
  block.size:= SizeOf(Block);
  block.head.ID:= 0;
  block.head.Cmd:= scInitFinish;
  Encode(Block);
end;

procedure TADKSession.Send_TextAttr121(const Ident: Cardinal; const Cmd: TServCmd;
  const IDDestinataire: Cardinal; const TextAttr: TTextAttr; const Text: string);
var
  Block: packed record
    FixedPart: record
      size: integer;
      Head: TBlockHeader;
      IdToSend: Cardinal;
      Attr: TTextAttr;
      Len: integer;
    end;
    OtherPart: array[0..1024] of char;
  end;
begin
  with Block.FixedPart do begin
    Head.ID:= IDent;
    Head.Cmd:= Cmd;
    IdToSend:= IdDestinataire;
    Attr:= TextAttr;
    Len:= Length(Text);
    if len > 1024 then Len:= 1024;
    Move(Text[1], Block.OtherPart, len);
    Size:= SizeOf(Block.FixedPart) + 4 + Len;
  end;
  Encode(Block);
end;

procedure TADKSession.Send_TextInfo121(const ID: Cardinal; const Cmd: TServCmd;
  const IDDestinataire: Cardinal; const text: string);
var
  Block: packed record
    FixedPart: record
      size: integer;
      Head: TBlockHeader;
      IdToSend: Cardinal;
      Len: integer;
    end;
    OtherPart: array[0..MaxTxtInfo] of char;
  end;
begin
  with Block do begin
    with FixedPart do begin
      Size:= SizeOf(FixedPart) + 4 + Length(Text);
      Head.ID:= ID;
      Head.Cmd:= Cmd;
      IdToSend:= IDDestinataire;
      Len:= Length(Text);
      if len > (MaxTxtInfo + 1) then Len:= MaxTxtInfo + 1;
    end;
    Move(Text[1], OtherPart, FixedPart.Len);
  end;
  Encode(Block);
end;

procedure TADKSession.Send_Position(const ID: Cardinal; const Cmd: TServCmd;
  const PositionInfo: TPositionInfo);
var
  block: packed record
    size: integer;
    head: TBlockHeader;
    data: TPositionInfo;
  end;
begin
  block.size:= sizeof(block);
  block.head.ID:= ID;
  block.head.Cmd:= Cmd;
  block.data:= PositionInfo;
  Encode(block);
end;

procedure TADKSession.Send_NoDataCmd(const ID: Cardinal; const Cmd: TServCmd);
var
  block: packed record
    size: integer;
    head: TBlockHeader;
  end;
begin
  block.size:= sizeof(block);
  block.head.ID:= ID;
  block.head.Cmd:= Cmd;
  Encode(block);
end;


{TCustomADKServer}

procedure TCustomADKServer.Decode(Session: TADKSession; out Data);
var
  Block: TADKBlock absolute Data;
  RawData, Temp: TmemoryStream;
  SizeBlockHead: integer;

  procedure FillStream;
  begin
    Temp:= TmemoryStream.Create;
    try
      Temp.writebuffer(Block, Block.Size);
      SizeBlockHead:= sizeof(Block.Size) + sizeof(TBlockHeader);
      Temp.Position:= SizeBlockHead;
      RawData.CopyFrom(Temp, Block.size - SizeBlockHead);
    finally
      temp.free;
    end;
  end;

begin
  case Block.Head.Cmd of
    scNegociate: if assigned(EOnNegociate) then begin
        EOnNegociate(Self, Session, Block.Head.id, Block.Data.Negociate);
      end;
    scConnectMe: if Assigned(EOnLogon) then begin
        with Block.Data.LoginInfo do
          EOnLogon(Self, Session, Block.Head.id, name, pass);
      end;
    scNameExist, scChangeUrlUpdate, scShowMessage,
    scGetAllPerso, scGetNameAndPass: if Assigned(EOnString) then begin
        with Block.Data.TextInfo do
          EOnString(Self, Session, Block.Head.id, Block.Head.Cmd, Copy(Text, 1, Size));
      end;
    scCreatePerso2: if Assigned(EOnRecivePJ) then begin
        RawData:= TmemoryStream.Create;
        try
          FillStream; //Recupere dans le flux de donnes les elements pour le TmemoryStream RAWDATA
          EOnRecivePJ(Self, Session, Block.Head.Id, Block.Head.Cmd, RawData);
        finally
          RawData.Destroy;
        end;
      end;

    scText, scTextRp: if Assigned(EOnMessage) then begin
        with Block.Data.MessageInfo do
          EOnMessage(Self, Session, Block.Head.id, Block.Head.Cmd, Copy(Text, 1, Size), Attr);
      end;
    scChangeVitesse, // Modifier la vitesse de dplacement par dfaut
      scChangeInfluence, // Modifier la taille du rayon d'influence
      scErasePerso, // Effacer personnage de la base de donnes
      scChangeUpdate,
      scGetActifItem,
      scRotate,
      scGetNbrIncrit,
      scActifItemPNJ: if Assigned(EOnCardinal) then begin
        with Block do
          EOnCardinal(Self, Session, Head.id, Head.Cmd, data.CardinalInfo);
      end;
    scGetNbrPlayers..scGetIdAllPlayer: if assigned(EOnNoData) then
        EOnNoData(Self, Session, Block.Head.id, Block.Head.Cmd);
    scPing: begin
        Session.Encode(Block); // on renvoie le block immdiatement
      end;
    scString21: if assigned(EonString21) then begin //Texte Brut
        with Block.data.TextInfo21 do begin
          EonString21(Self, Session, Block.Head.id, Block.Head.Cmd, IdDestinataire, Copy(Text, 1, Size));
        end;
      end;
    scText21: if assigned(EOnMessage21) then begin //Texte avec Attributs
        with Block.data.MessageInfo21 do begin
          EOnMessage21(Self, Session, Block.Head.id, Block.Head.Cmd, IdDestinataire, Copy(Text, 1, Size), Attr);
        end;
      end;
    scPlaceAt,
      scTeleport,
      scMoveTo,
      scRunTo,
      scJump,
      scEndMove,
      scChangeMap: if assigned(EOnPosition) then begin
        with Block do
          EOnPosition(Self, Session, Head.id, Head.Cmd, Data.PositionInfo);
      end;



  else begin
      Session.Send_TextInfo(0, scCloseConnect, rsErr1507);
    //Session.Error('Ordre non gr '+IntToStr(Ord(Block.Head.Cmd)));
    end;
  end;
end;

end.

