unit ADKWSA;

interface

uses
  Windows, Winsock, Messages, Classes, SysUtils,
  ADKServer;

const
  RECV_BUFFER_SIZE = 4096; //2048; // 1024x2 (ouai..ou 512x4 ou 128x16 hein)
  MAX_BLOCK_SIZE = 4000; //1536; // 1024+512


type
  ESocketSendError = class(Exception);

  TWSABuffer = record
    case byte of
      0: (size: integer);
      1: (data: array[0..RECV_BUFFER_SIZE - 1] of byte);
  end;

// composant permettant d'accder  un serveur ADK en TCP
  TADKProvideWSA = class(TADKProvider)
  private
    fServer: string; // adresse ou nom IP du serveur
    fPort: word; // numro de port du serveur (disponible pour TADKProvideFF)
    fSocket: TSocket; // socket du serveur (disponible pour TADKProvideFF)
    fHandle: THandle; // fentre de synchronisation des threads
    fBuffer: TWSABuffer;
    fSize: integer;
    procedure SetServer(Value: string);
    procedure SetPort(Value: word);
  protected
    procedure Read(Socket: TSocket); // donnes entrante
    procedure Kill; override; // couper une connexion en avertissant le serveur
    procedure WndProc(var msg: TMessage); // traitement des messages de synchronisation
  public
    constructor Create(AOwner: TComponent); override;
    function Active: boolean; override;
    procedure Open; override; // ouverture d'une connexion
    procedure Close; override; // fermeture de la connexion
    procedure Encode(out Data); override; // envoie d'un block de donnes au serveur
  published
    property Server: string read fServer write SetServer;
    property Port: word read fPort write SetPort default 2001;
  end;

// composant recevant les connexions TCP du client ci-dessus
  TADKDispatchWSA = class(TADKDispatcher)
  private
    fPort: word; // numro de port du serveur
    fSocket: TSocket; // socket serveur
    fHandle: THandle; // fentre de synchronisation
    fClient: TSocket;
    fSockAddr: TSockAddr;
    procedure SetPort(Value: word);
  protected
    function GetActive: boolean; override; // dtermine si l'on est actif
    procedure WndProc(var msg: TMessage); // traitement des messages de synchronisation
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Start; override; // se mettre  l'coute des clients
    procedure Close; override; // arrt total (voir aussi TADKDispatch.Stop)
  published
    property Port: word read fPort write SetPort;
  end;

// session TCP ouverte sur le Dispatcher par le Client
  TADKSessionWSA = class(TCustomADKSession)
  private
    fAddr: string;
    fSocket: TSocket; // socket client
    fBuffer: TWSABuffer;
    fSize: integer;
  protected
    procedure Read;
    procedure Encode(out data); override; // envoie d'un block de donnes au client
  public
    constructor Create(AOwner: TADKDispatchWSA);
    destructor Destroy; override;
    property Addr: string read fAddr;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MySoft.ADK', [TADKProvideWSA, TADKDispatchWSA]);
end;

function INetAddr(host: string): integer;
var
  HostEnt: PHostEnt;
begin
  Result := inet_addr(pchar(host));
  if Result = INADDR_NONE then begin // ce n'est pas une adresse x.y.z.w
    HostEnt := gethostbyname(PChar(host));
    if HostEnt <> nil then
      Result := integer(pointer(HostEnt^.h_addr^)^)
    else
      Result := 0;
  end;
end;

procedure Disconnect(var ASocket: TSocket);
//var
// dummy:array[0..1024] of char;
begin
  if ASocket <> INVALID_SOCKET then begin
    shutdown(ASocket, SD_BOTH);
//  repeat until recv(ASocket,dummy,1024,0)<=0;
    closesocket(ASocket);
    ASocket := INVALID_SOCKET;
  end;
end;

function SendData(Socket: TSocket; out data; size: integer): boolean;
var
  p: pchar;
  i: integer;
begin
  result := true;
  if Size = 0 then exit;
  p := @Data;
  while Size > 0 do begin
    i := send(Socket, p^, Size, 0);
    if i = SOCKET_ERROR then begin
      result := false;
      exit;
    end;
    dec(Size, i);
    inc(p, i);
    sleep(0);
  end;
end;

//-- le provider WSA --------------------------------------------------------//

// initialisation

constructor TADKProvideWSA.Create(AOwner: TComponent);
begin
  inherited;
  fSocket := INVALID_SOCKET;
  fPort := 2001;
end;

// paramtrage du nom de serveur

procedure TADKProvideWSA.SetServer(Value: string);
begin
  if Value <> fServer then begin
    Close;
    fServer := Value;
  end;
end;

// paramtrage du n de port

procedure TADKProvideWSA.SetPort(Value: word);
begin
  if Value <> fPort then begin
    Close;
    fPort := Value;
  end;
end;

// la connexion s'est termine

procedure TADKProvideWSA.Kill;
begin
  if not Active then exit;
  Close;
  if Assigned(EOnClose) then EOnClose(Self);
end;

// dtermine si le provider est actif

function TADKProvideWSA.Active: boolean;
begin
  Result := (fHandle <> 0) or (fSocket <> INVALID_SOCKET);
end;

// ouverture de session

procedure TADKProvideWSA.Open;
var
  sa: TSockAddr;
begin
  if Active then exit;
  fSize := 0;
 // allocation d'une WndProc
  fHandle := AllocateHWnd(WndProc);
  if fHandle = 0 then FatalError('Erreur interne Windows');
 // allocation d'un socket TCP, le point de dpart, sans lui pas la peine d'aller plus loin :)
  fSocket := socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
  if fSocket = INVALID_SOCKET then FatalError('impossible d''ouvrir un socket IP');
  wsaAsyncSelect(fSocket, fHandle, wm_user, FD_CONNECT or FD_READ or FD_CLOSE);
 // connexion
  FillChar(sa, SizeOf(sa), 0);
  sa.sin_family := AF_INET;
  sa.sin_addr.S_addr := INetAddr(fServer); // adresse IP
  sa.sin_port := htons(fPort); // n de port
  connect(fSocket, sa, SizeOf(sa));
end;

// fermeture de la session

procedure TADKProvideWSA.Close;
begin
  Disconnect(fSocket);
  if fHandle <> 0 then begin
    DeallocateHWnd(fHandle);
    fHandle := 0;
  end;
end;

// mthode de synchronisation des sockets

procedure TADKProvideWSA.WndProc(var msg: TMessage);
begin
  try
    if Msg.msg <> WM_USER then begin
      with msg do DefWindowProc(fHandle, msg, wParam, lParam);
      exit;
    end;

    if Msg.LParamHi <> 0 then FatalError('Erreur de communication');

    case Msg.LParamLo of
      FD_READ: Read(Msg.wParam);
      FD_CONNECT: if Assigned(EOnConnect) then EOnConnect(Self);
      FD_CLOSE: Kill;
    end;

  except
    msg.Result := -1;
  end;
end;

procedure TADKProvideWSA.Read(Socket: TSocket);
var
  Count: integer;
begin
  Count := recv(Socket, fBuffer.Data[fSize], RECV_BUFFER_SIZE - fSize, 0);
  if Count = 0 then FatalError('Connexion perdue');
  if Count < 0 then FatalError('Erreur de lecture');
  if Assigned(EOnRead) then EOnRead(Self, Count);
  inc(fSize, Count);
  while (fSize > 4) and (fBuffer.Size <= fSize) do begin
    if fBuffer.Size > MAX_BLOCK_SIZE then FatalError('Message trop long');
    if Assigned(fClient) then fClient.Decode(fBuffer);
    Dec(fSize, fBuffer.Size);
    if fSize > 0 then Move(fBuffer.Data[fBuffer.Size], fBuffer, fSize);
  end;
  if fSize > MAX_BLOCK_SIZE then FatalError('Message invalide');
end;

// envoie d'un block de donnes au serveur
// NB: les premiers 32 bit de "data" est un integer indiquant la taille totale des donnes

procedure TADKProvideWSA.Encode(out Data);
begin
  if not SendData(fSocket, Data, integer(Data)) then FatalError('Ecriture sur le socket impossible');
end;

//-- Dispatcher --------------------------------------------------------------//

// initialisation

constructor TADKDispatchWSA.Create(AOwner: TComponent);
begin
  fHandle := AllocateHWnd(WndProc);
  fSocket := INVALID_SOCKET;
  inherited;
end;

// fin

destructor TADKDispatchWSA.Destroy;
begin
  inherited;
  DeallocateHWnd(fHandle);
end;

// paramtrage du port d'coute

procedure TADKDispatchWSA.SetPort(Value: word);
begin
  if Value <> fPort then begin
    Stop;
    fPort := Value;
  end;
end;

// dtermine s'il est actif

function TADKDispatchWSA.GetActive: boolean;
begin
  Result := (fSocket <> INVALID_SOCKET);
end;

// mise  l'coute des clients

procedure TADKDispatchWSA.Start;
var
  sa: TSockAddr;
begin
  if Active then exit;
// allocation d'un socket TCP, le point de dpart, sans lui pas la peine d'aller plus loin :)
  fSocket := socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
  if fSocket = INVALID_SOCKET then FatalError('impossible d''ouvrir un socket IP');
// gestion Windows
  wsaAsyncSelect(fSocket, fHandle, wm_user, FD_ACCEPT or FD_READ or FD_CLOSE);
// associer le n de port
  FillChar(sa, SizeOf(sa), 0);
  sa.sa_family := AF_INET;
  sa.sin_port := htons(fPort);
  if bind(fSocket, sa, SizeOf(sa)) <> 0 then FatalError('Impossible de dmarrer le service ' + IntToStr(fPort));
// mise  l'coute
  if listen(fSocket, 0) <> 0 then FatalError('Impossible de dmarrer le serveur');
// OnStart...
  if Assigned(EOnStart) then EOnStart(Self);
end;

// ne plus accepter de nouvelles connexions

procedure TADKDispatchWSA.Close;
begin
// fermer socket serveur
  Disconnect(fSocket);
end;

// mthode de synchronisation des tches

procedure TADKDispatchWSA.WndProc(var Msg: TMessage);
var
  i: integer;
begin
  {#todo1 modifier suivant recommandation de Paul}
  if Msg.msg <> WM_USER then begin
    with msg do DefWindowProc(fHandle, msg, wParam, lParam);
    exit;
  end;

  if msg.lParam = FD_ACCEPT then begin
    i := SizeOf(fSockAddr);
    fClient := accept(Msg.wParam, @fSockAddr, @i);
    TADKSessionWSA.Create(Self);
  end else begin
    for i := fSessions.Count - 1 downto 0 do
      with TADKSessionWSA(fSessions[i]) do begin
        if fSocket = msg.wParam then begin
          if msg.lParamHi <> 0 then begin
            CloseSession(TADKSessionWSA(fSessions[i])); //Close //  tester...
          end else begin
            case Msg.LParamLo of
              FD_READ: Read;
              FD_CLOSE: Kill;
            else CloseSession(TADKSessionWSA(fSessions[i])); //close; // ou autre chose...
            end;
          end;
          exit;
        end;
      end;
  end;

end;

//-- Sessions ----------------------------------------------------------------//

// init

constructor TADKSessionWSA.Create(AOwner: TADKDispatchWSA);
begin
  fSocket := AOwner.fClient;
  fAddr := inet_ntoa(AOwner.fSockAddr.sin_addr);
  fName := 'wsa@' + fAddr;
  inherited Create(AOwner);
end;

// done

destructor TADKSessionWSA.Destroy;
begin
  inherited;
  Disconnect(fSocket);
end;

// lecture du socket client

procedure TADKSessionWSA.Read;
var
  Count: integer;
begin
  Count := recv(fSocket, fBuffer.Data[fSize], RECV_BUFFER_SIZE - fSize, 0);
  if Count = 0 then Error('Connexion perdue');
  if Count < 0 then Error('Erreur de lecture');
  if Assigned(fDispatcher.OnRead) then fDispatcher.OnRead(fDispatcher, Self, Count);
  inc(fSize, Count);
  while (fSize > 4) and (fBuffer.Size <= fSize) do begin
    if fBuffer.Size > MAX_BLOCK_SIZE then Error('Message trop long');
    fDispatcher.Server.Decode(Self, fBuffer);
    Dec(fSize, fBuffer.Size);
    if fSize > 0 then Move(fBuffer.Data[fBuffer.Size], fBuffer, fSize);
  end;
  if fSize > MAX_BLOCK_SIZE then Error('Message invalide');
end;

// envoie de donnes au client
// NB: les premiers 32 bit de "data" est un integer indiquant la taille totale des donnes

procedure TADKSessionWSA.Encode(out Data);
begin
 {#todo1 Modifier par manuel}
 //if not SendData(fSocket,Data,integer(data)) then Error('Ecriture sur le socket impossible');
  if not SendData(fSocket, Data, integer(data)) then ESocketSendError.Create('Ecriture sur le socket impossible');
end;

//----- le code qui suit initialise les sockets Windows --------

procedure wsaStartup;
var
  wsa: TWSAData;
begin
  Winsock.wsaStartup($101, wsa);
end;

initialization
  wsaStartup;
finalization
  wsaCleanup;
end.
