{-----------------------------------------------------------------------------
 Unit Name: ObjetsActif
 Author:    Manuel
 Purpose:   Definit tout les objets actif sur une carte.
            Par exemple: Une porte, un personnage, une animation etc

 History:   30 Janvier 2003: cration de l'unit.
            Copie des Tclass Tminiperso qui se trouvait avant dans l'unit
            dfinition.
-----------------------------------------------------------------------------}


unit ObjetsActif;

interface
uses AdkNETdata, definitions, HandBook2, Contnrs, Classes, Types, personnage;

type
  TCaractActifChanged = (acId, acPosition, acMap, acVisible,
    acName, acRace, acSex, acSortActif, acObjets);

  TBaseActifItemClass = class of TBaseActifItem;
  TListActifItemClass = class of TListActifItem;
  TCompoActifItemClass = class of TCompoActifItem;

  TBaseActifItem = class;
  TListActifItem = class;
  TCompoActifItem = class;

  TonNotifyActifItem = procedure(Sender: TObject; ActifItem: TBaseActifItem) of object;
  TonActifItemCaractChange = procedure(ActifItem: TBaseActifItem; const Caract: TCaractActifChanged) of object;
  TonNotifyActifItemChange = procedure(Sender: TObject;
    ActifItem: TBaseActifItem; const Caract: TCaractActifChanged) of object;



  TBaseActifItem = class
  private
    fOnChange: TnotifyEvent;
    fonChangeCaract: TonActifItemCaractChange;
    FOwner: TListActifItem;
    fClassCardinal: Cardinal;
    fId: Cardinal;
    fPosition: TPositionInfo; //TR_Position;
    fvisible: byte;
    fReserved1: Shortint; // Futur Use
  protected
    procedure Change; //A appeler par toutes les proc qui modifie une valeur
    procedure ChangeCaract(const Value: TCaractActifChanged); // IDem que Change et ici on prcise ce qui change exactement
    procedure SetId(const Value: Cardinal);
    procedure SetPosition(const Value: TPositionInfo);
    procedure SetVisible(const Value: Byte);
    function GetName: Tstr15; virtual;
    procedure SetName(const Value: Tstr15); virtual;
    function GetMap: Word;
    procedure SetMap(const value: word);
    function GetCoord: Tpoint;
    procedure SetCoord(const Value: Tpoint);
  public
    constructor create(AOwner: TListActifItem);
    procedure assign(Source: TBaseActifItem); virtual;
    procedure Writedata(Datas: TStream); virtual; //renvoie le TStream pass en argument "remplie" avec les donnes
    procedure ReadData(Datas: TStream); virtual;
    function SamePos(XPos, YPos: integer): Boolean; // si XPo et Ypos= fPosition alors renvoie TRUE
    //Property's
    property ClassCardinal: Cardinal read fClassCardinal;
    property Id: Cardinal read fId write SetId;
    property Position: TPositionInfo read fPosition write SetPosition;
    property Map: word read GetMap write SetMap;
    property Coord: Tpoint read GetCoord write SetCoord;
    property visible: byte read fVisible write SetVisible;
    property Name: TStr15 read GetName write SetName;
    //Events
    property OnChange: TnotifyEvent read fOnChange write fOnChange;
    property OnChangeCaract: TonActifItemCaractChange read fonChangeCaract write fonChangeCaract;
  end;


  TNamedActifItem = class(TBaseActifItem)
  private
    Fname: Tstr15;
  protected
    function GetName: Tstr15; override;
    procedure SetName(const Value: Tstr15); override;
  public
    procedure Writedata(Datas: TStream); override;
    procedure ReadData(Datas: TStream); override;
    procedure assign(Source: TBaseActifItem); override;
    //property Name: Tstr15 read GetName write SetName;
  end;


  TminiObjet = record
    IdObjet: Word; // N d'identification de l'objet
    Emplacement: Templacement; // Emplacement sur le perso (Main, tete, sac etc)
  end;

  TArrayMiniObjets = array of TminiObjet;

  TarrayMiniSorts = array of TActiveSort;

  TMiniPerso = class(TNamedActifItem)
  private
    fRace: word;
    fSex: TSex;
    fSortsActifs: TarrayMiniSorts;
    fObjets: TArrayMiniObjets;
    //fMoveto: Tr_Pos;
    fReserved2: byte;     // Futur Use
    fWalker: Tobject;
  protected
    procedure SetRace(const Value: word);
    procedure SetSex(const Value: TSex);
    procedure SetSortsActifs(const Value: TarrayMiniSorts);
    procedure SetObjets(const Value: TArrayMiniObjets);
    procedure SetMoveto(const Value: Tr_Pos);
    procedure SetRunning(const Value: Boolean);
  public
    destructor destroy; override;
    procedure assign(Source: TBaseActifItem); override;
    procedure WriteData(Datas: TStream); override;
    procedure ReadData(Datas: TStream); override;
    // Property's
    property Race: word read fRace write SetRace;
    property Sex: TSex read fSex write SetSex;
    property SortsActifs: TarrayMiniSorts read fSortsActifs write SetSortsActifs;
    property Objets: TArrayMiniObjets read fObjets write SetObjets;
    //property Moveto: Tr_Pos read fMoveto write SetMoveto;
    //property Running: boolean read frunning write SetRunning;
    property Walker: Tobject read fWalker write fWalker;
  end;




  TListActifItem = class(Tobjectlist)
  private
    FOnAdd, FOnDelete, FOnChange: TonNotifyActifItem;
    FOnChangeCaract: TonNotifyActifItemChange;
    FParent: TCompoActifItem;
   // fAllowSameID: Permetre de rajouter un TminiPers dont l'ID est dja dans la liste
   // fKeepTheFirstID: Si True on Garde la premier ID comme bonne. Si False on remplace l'ancienne ID par la nouvelle
    fAllowSameID, fKeepTheFirstID: boolean;
  protected
    procedure ChangedItems(Sender: Tobject);
    procedure ChangedCaractItems(ActifItem: TBaseActifItem; const Caract: TCaractActifChanged);
    function GetItems(Index: Integer): TBaseActifItem;
    procedure SetItems(Index: Integer; BaseActifItem: TBaseActifItem);
  public
    constructor create;
    function Add(BaseActifItem: TBaseActifItem): Integer;
    function Remove(BaseActifItem: TBaseActifItem): Integer;
    procedure Insert(Index: Integer; BaseActifItem: TBaseActifItem);
    procedure Delete(Index: Integer);

    function IndexOf(BaseActifItem: TBaseActifItem): Integer;
    function GetItemByID(Id: Cardinal): TBaseActifItem;

    procedure DeleteByID(const Id: Cardinal);
    property Parent: TCompoActifItem read FParent write FParent;
    property Items[Index: Integer]: TBaseActifItem read GetItems write SetItems; default;
    property AllowSameID: boolean read fAllowSameID write fAllowSameID;
    property KeepTheFirstID: Boolean read fKeepTheFirstID write fKeepTheFirstID;
   //Events
    property OnAdd: TonNotifyActifItem read FOnAdd write FOnAdd; //<< Appel juste aprs l'ajout du perso
    property OnDelete: TonNotifyActifItem read FOnDelete write FOnDelete; //<< Appel juste AVANT l'effacement du perso
  end;


  // Composant final des ActifItem que l'on peut utiliser depuis la VCL
  TCompoActifItem = class(Tcomponent)
  private
    fListActifItem: TListActifItem;
  protected
    fOnAdd, fOnDelete, fOnChange: TonNotifyActifItem;
    fOnChangeCaract: TonNotifyActifItemChange;
    procedure ChangedItems(Sender: Tobject; Item: TBaseActifItem);
    procedure ChangedItemsCaratc(Sender: TObject; ActifItem: TBaseActifItem; const Caract: TCaractActifChanged);
    procedure AddedItem(Sender: TObject; BaseActifItem: TBaseActifItem);
    procedure DeletedItem(Sender: TObject; BaseActifItem: TBaseActifItem);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property ListItems: TListActifItem read fListActifItem;
  published
   //Events
    property OnAdd: TonNotifyActifItem read FOnAdd write FOnAdd;
    property OnDelete: TonNotifyActifItem read FOnDelete write FOnDelete;
    property OnChange: TonNotifyActifItem read FonChange write FonChange;
    property OnChangeCaract: TonNotifyActifItemChange read fOnChangeCaract write fOnChangeCaract;

  end;


procedure CustomPersoToRaw(const Id: Cardinal;
  const APerso: TCustomPerso; RawDatas: Tstream);


procedure FillTMiniPersoWithTPjPerso(const ID: Cardinal;
  const APerso: TCustomPerso; const BaseActifItem: TBaseActifItem);

procedure Register;

implementation
uses Globals, StrStuff;


procedure Register;
begin
  RegisterComponents('MiniChat', [TCompoActifItem]);
end;

procedure FillTMiniPersoWithTPjPerso(const ID: Cardinal;
  const APerso: TCustomPerso; const BaseActifItem: TBaseActifItem);
var i, j: integer;
  arrayMiniSorts: TarrayMiniSorts;
  ArrayMiniObjets: TArrayMiniObjets;
  //miniObjet: TminiObjet;
begin
  if assigned(APerso) and assigned(BaseActifItem) then begin
    BaseActifItem.ID:= Id;
    BaseActifItem.Position:= APerso.PositionInfo;
    BaseActifItem.fPosition.Vitesse:= Ord(APerso.TRPosition.dir);
    BaseActifItem.visible:= APerso.Visible;
    BaseActifItem.Name:= APerso.Name;

    if (BaseActifItem is TMiniPerso) and (APerso is TCustomPJPNJ) then begin
      with TCustomPJPNJ(Aperso) do begin
        TMiniPerso(BaseActifItem).race:= race;
        TMiniPerso(BaseActifItem).Sex:= Sex;
        {
        TMiniPerso(BaseActifItem).Moveto.x:= TRPosition.xyz.x;
        TMiniPerso(BaseActifItem).fMoveto.y:= TRPosition.xyz.y;
        TMiniPerso(BaseActifItem).fMoveto.z:= Ord(TRPosition.dir);
        TMiniPerso(BaseActifItem).Running:= False;
        }
        // SortsActifs
        //Problme ici car on accede  sorts actifs sans vrifier s'il est "dfini"
        //donc vrifi avant si high(SortsActifs) > -1
        j:= 0;
        for i:= 0 to high(SortsActifs) do
          if SortsActifs[i].IdObjet <> 0 then inc(j);
        if j > 0 then begin
          SetLength(arrayMiniSorts, j);
          j:= 0;
          for i:= 0 to high(SortsActifs) do begin
            if SortsActifs[i].IdObjet <> 0 then begin
              arrayMiniSorts[j]:= SortsActifs[i];
              inc(j);
            end;
          end;
        end;
        //TMiniPerso(BaseActifItem).SortsActifs:= arrayMiniSorts;
        //ici idem que pour sortsActifs
        // MiniInventaire
        j:= 0;
        for i:= 0 to High(Inventaire) do
          if Inventaire[i].Emplacement in TSetEmplacementMini then
            if Inventaire[i].IdObjet > 0 then inc(j);
        if j > 0 then begin
          SetLength(ArrayMiniObjets, j);
          j:= 0;
          for i:= 0 to High(Inventaire) do
            if Inventaire[i].Emplacement in TSetEmplacementMini then
              if Inventaire[i].IdObjet > 0 then begin
                ArrayMiniObjets[j].IdObjet:= Inventaire[i].IdObjet;
                ArrayMiniObjets[j].Emplacement:= Inventaire[i].Emplacement;
                inc(j);
              end;
          TMiniPerso(BaseActifItem).Objets:= ArrayMiniObjets;
        end;
      end;
    end;
  end;
end;

procedure CustomPersoToRaw(const Id: Cardinal;
  const APerso: TCustomPerso; RawDatas: Tstream);
var
  //NameOfClass: Cardinal;
  MiniPerso: TMiniPerso;
begin
  RawDatas.Position:= 0;
  MiniPerso:= TMiniPerso.Create(nil);
  try
    FillTMiniPersoWithTPjPerso(ID, APerso, MiniPerso);
      // Remplir le stream avec les donnes du Miniperso
    MiniPerso.WriteData(RawDatas);
  finally
    MiniPerso.Free;
  end;
end;



{TBaseActifItem}

procedure TBaseActifItem.assign(Source: TBaseActifItem);
begin
  Id:= Source.Id;
  Position:= Source.Position;
  visible:= Source.Visible;
  fReserved1:= Source.freserved1;
  change;
end;

procedure TBaseActifItem.ReadData(Datas: TStream);
begin
  Datas.position:= 0; //<< Normalement pas utile mais on ne sait jamais.
  with Datas do begin
    ReadBuffer(fClassCardinal, sizeof(fClassCardinal));
    ReadBuffer(fId, sizeof(fid));
    ReadBuffer(fPosition, sizeof(fPosition));
    ReadBuffer(fVisible, sizeof(fVisible));
    ReadBuffer(fReserved1, sizeof(fReserved1));
  end;
end;

procedure TBaseActifItem.Writedata(Datas: TStream);
begin
  Datas.position:= 0; //<< Normalement pas utile mais on ne sait jamais.
  with Datas do begin
    WriteBuffer(fClassCardinal, sizeof(fClassCardinal));
    WriteBuffer(fId, sizeof(fid));
    WriteBuffer(fPosition, sizeof(fPosition));
    WriteBuffer(fVisible, sizeof(fVisible));
    WriteBuffer(fReserved1, sizeof(fReserved1));
  end;
end;

constructor TBaseActifItem.create(AOwner: TListActifItem);
begin
  inherited create;
  FOwner:= AOwner;
  fClassCardinal:= CrcString2(Self.ClassName);
end;

procedure TBaseActifItem.Change;
begin
  if assigned(fOnChange) then fOnChange(self);
end;

procedure TBaseActifItem.ChangeCaract(const Value: TCaractActifChanged);
begin
  if assigned(fonChangeCaract) then fonChangeCaract(self, value);
end;

function TBaseActifItem.SamePos(XPos, YPos: integer): Boolean; // si XPo et Ypos= fPosition alors renvoie TRUE
begin
  result:= (fPosition.x = Xpos) and (fPosition.y = Ypos);
end;

procedure TBaseActifItem.SetId(const Value: Cardinal);
begin
  if fid <> Value then begin
    fId:= Value;
    Change;
    ChangeCaract(acId);
  end;
end;

procedure TBaseActifItem.SetPosition(const Value: TPositionInfo);
begin
  if (fposition.Map <> Value.map) then begin
    fPosition:= Value;
      { Quand on change de map on ne provoque qu'un Evenement
       ChangeCaract(acMap) et pas de ChangeCaract(acPosition) mme
       si la position a 99.99% de change d'tre modifier quand on chnage de map.
       Et, JUSTEMENT, comme c'est une certitude (que la position du perso
        changer puisqu'il  changer de Map) c'est  la procedure
       qui va traiter l'vnement ChangeCaract(acMap); de placer le perso
       sur sa nouvelle position (et son nouveau MAP bien sur).
       }
    ChangeCaract(acMap);
    change;
  end else begin
    if (fposition.x <> Value.x) or (fposition.y <> Value.y) or (fposition.Vitesse <> Value.Vitesse) then begin
      fPosition:= Value;
      ChangeCaract(acPosition);
      Change;
    end;
  end;
end;

procedure TBaseActifItem.SetVisible(const Value: Byte);
begin
  if fvisible <> Value then begin
    fvisible:= Value;
    Change;
    ChangeCaract(acVisible);
  end;
end;

function TBaseActifItem.GetName: Tstr15;
begin
  result:= 'TBaseActifItem';
end;

procedure TBaseActifItem.SetName(const Value: Tstr15);
begin
  exit;
end;

function TBaseActifItem.GetMap: Word;
begin
  result:= fPosition.map;
end;

procedure TBaseActifItem.SetMap(const value: word);
begin
  if (fposition.Map <> Value) then begin
    fPosition.map:= Value;
      { Quand on change de map on ne provoque qu'un Evenement
       ChangeCaract(acMap) et pas de ChangeCaract(acPosition) mme
       si la position a 99.99% de change d'tre modifie quand on change de map.
       Et, JUSTEMENT, comme c'est une certitude (que la position du perso
        changer puisqu'il  changer de Map) c'est  la procedure
       qui va traiter l'vnement ChangeCaract(acMap); de placer le perso
       sur sa nouvelle position (et son nouveau MAP bien sur).
       }
    ChangeCaract(acMap);
    change;
  end;
end;

function TBaseActifItem.GetCoord: Tpoint;
begin
  result:= Point(fPosition.x, fPosition.y);
end;

procedure TBaseActifItem.SetCoord(const Value: Tpoint);
begin
  if (fposition.x <> Value.x) or (fposition.y <> Value.y) then begin
    fPosition.x:= Value.x;
    fPosition.y:= Value.y;
    ChangeCaract(acPosition);
    Change;
  end;
end;


{TNamedActifItem}

procedure TNamedActifItem.assign(Source: TBaseActifItem);
begin
  if source is TNamedActifItem then fname:= Source.Name;
  inherited;
end;

procedure TNamedActifItem.ReadData(Datas: TStream);
begin
  inherited;
  fname:= ReadBufferString(Datas);
end;

procedure TNamedActifItem.Writedata(Datas: TStream);
begin
  inherited;
  WriteBufferString(Datas, Fname);
end;

function TNamedActifItem.GetName: Tstr15;
begin
  result:= fname;
end;

procedure TNamedActifItem.SetName(const Value: Tstr15);
begin
  if fname <> Value then begin
    fname:= Value;
    change;
  end;
end;




{TMiniPerso}

destructor TMiniPerso.destroy;
begin
  finalize(fSortsActifs);
  finalize(fObjets);
  if assigned(fWalker) then begin
    fWalker.Destroy;
    fWalker:= nil;
  end;
  inherited;
end;

procedure TMiniPerso.assign(Source: TBaseActifItem);
begin
  if source is TMiniPerso then begin
    fRace:= TMiniPerso(Source).race;
    fSex:= TMiniPerso(Source).Sex;
    fSortsActifs:= TMiniPerso(Source).SortsActifs;
    fObjets:= TMiniPerso(Source).Objets;
    fReserved2:= TMiniPerso(Source).fReserved2;
  end;
  inherited;
end;

procedure TMiniPerso.ReadData(Datas: TStream);
var lenArray: shortint;
  i: integer;
begin
  inherited;
  with datas do begin
    ReadBuffer(fRace, sizeof(fRace));
    ReadBuffer(fSex, sizeof(fSex));
    ReadBuffer(fReserved2, sizeof(fReserved2));
    ReadBuffer(lenArray, sizeof(lenArray));
    if lenArray > -1 then begin
      SetLength(fSortsActifs, lenArray + 1);
      for i:= 0 to lenArray do
        ReadBuffer(fSortsActifs[i], sizeof(TActiveSort));
    end;
    ReadBuffer(lenArray, sizeof(lenArray));
    if lenArray > -1 then begin
      SetLength(fObjets, lenArray + 1);
      for i:= 0 to lenArray do
        ReadBuffer(fObjets[i], sizeof(TminiObjet));
    end;

  end;
end;

procedure TMiniPerso.Writedata(Datas: TStream);
var lenArray: shortint;
  i: integer;
begin
  inherited;
  with datas do begin
    WriteBuffer(fRace, sizeof(fRace));
    WriteBuffer(fSex, sizeof(fSex));
    WriteBuffer(fReserved2, sizeof(fReserved2));
    lenArray:= High(fSortsActifs);
    WriteBuffer(lenArray, sizeof(lenArray));
    for i:= 0 to lenArray do
      WriteBuffer(fSortsActifs[i], sizeof(TActiveSort));
    lenArray:= High(fObjets);
    WriteBuffer(lenArray, sizeof(lenArray));
    for i:= 0 to lenArray do
      WriteBuffer(fObjets[i], sizeof(TminiObjet));
  end;
end;

procedure TMiniPerso.SetRace(const Value: word);
begin
  if frace <> Value then begin
    fRace:= Value;
    Change;
  end;
end;

procedure TMiniPerso.SetSex(const Value: TSex);
begin
  if fSex <> Value then begin
    fSex:= Value;
    Change;
  end;
end;

procedure TMiniPerso.SetSortsActifs(const Value: TarrayMiniSorts);
begin
  fSortsActifs:= Value;
  Change;
end;

procedure TMiniPerso.SetObjets(const Value: TArrayMiniObjets);
begin

end;

procedure TMiniPerso.SetMoveto(const Value: Tr_Pos);
begin

end;

procedure TMiniPerso.SetRunning(const Value: Boolean);
begin

end;


{TListActifItem}

procedure TListActifItem.ChangedItems(Sender: Tobject);
begin
  if assigned(FOnChange) then FOnChange(self, TBaseActifItem(sender));
end;

procedure TListActifItem.ChangedCaractItems(ActifItem: TBaseActifItem; const Caract: TCaractActifChanged);
begin
  if assigned(FOnChangeCaract) then FOnChangeCaract(self, ActifItem, Caract);
end;

constructor TListActifItem.create;
begin
  inherited;
  fAllowSameID:= false;
  fKeepTheFirstID:= false;
end;

function TListActifItem.GetItems(Index: Integer): TBaseActifItem;
begin
  Result:= TMiniPerso(inherited Items[Index]);
end;

procedure TListActifItem.SetItems(Index: Integer; BaseActifItem: TBaseActifItem);
begin
  inherited Items[Index]:= BaseActifItem;
  if assigned(FOnAdd) then FOnAdd(self, BaseActifItem);
end;

function TListActifItem.Add(BaseActifItem: TBaseActifItem): Integer;
var ExistMini: TBaseActifItem;
  procedure AddAndEvent;
  begin
    BaseActifItem.OnChange:= ChangedItems;
    BaseActifItem.OnChangeCaract:= ChangedCaractItems;
    Result:= inherited Add(BaseActifItem);
    if assigned(FOnAdd) then FOnAdd(self, BaseActifItem);
  end;
begin
  if not fAllowSameID then begin
    ExistMini:= GetItemByID(BaseActifItem.Id);
    if assigned(ExistMini) then begin
      if not fKeepTheFirstID then begin
        ExistMini.Assign(BaseActifItem);
        BaseActifItem.Free;
      end else BaseActifItem.Free;
    end else AddAndEvent;
  end else AddAndEvent;
end;

function TListActifItem.Remove(BaseActifItem: TBaseActifItem): Integer;
begin
  if assigned(FOnDelete) then FOnDelete(self, BaseActifItem);
  Result:= inherited Remove(BaseActifItem);
end;

function TListActifItem.IndexOf(BaseActifItem: TBaseActifItem): Integer;
begin
  Result:= inherited IndexOf(BaseActifItem);
end;

function TListActifItem.GetItemByID(Id: Cardinal): TBaseActifItem;
var i: integer;
begin
  result:= nil;
  for i:= 0 to Count - 1 do
    if ID = Items[i].Id then
    begin
      result:= TBaseActifItem(inherited Items[i]);
      exit;
    end;
end;

procedure TListActifItem.Insert(Index: Integer; BaseActifItem: TBaseActifItem);
begin
  inherited Insert(Index, BaseActifItem);
  if assigned(FOnAdd) then FOnAdd(self, BaseActifItem);
end;

procedure TListActifItem.Delete(Index: Integer);
begin
  if assigned(FOnDelete) then FOnDelete(self, Items[Index]);
  inherited;
end;

procedure TListActifItem.DeleteByID(const Id: Cardinal);
var i: integer;
begin
  for i:= 0 to Count - 1 do
    if ID = Items[i].Id then
    begin
      Delete(i);
      exit;
    end;
end;

{TCompoActifItem}

procedure TCompoActifItem.AddedItem(Sender: TObject; BaseActifItem: TBaseActifItem);
begin
  if assigned(FOnAdd) then FOnAdd(Self, BaseActifItem);
end;

procedure TCompoActifItem.ChangedItems(Sender: Tobject; Item: TBaseActifItem);
begin
  if assigned(fOnChange) then fOnChange(self, Item);
end;

procedure TCompoActifItem.ChangedItemsCaratc(Sender: TObject;
  ActifItem: TBaseActifItem; const Caract: TCaractActifChanged);
begin
  if assigned(fOnChangeCaract) then fOnChangeCaract(self, ActifItem, Caract);
end;

constructor TCompoActifItem.Create(AOwner: TComponent);
begin
  inherited;
  fListActifItem:= TListActifItem.create;
  fListActifItem.fOnAdd:= AddedItem;
  fListActifItem.fOnDelete:= DeletedItem;
  fListActifItem.fParent:= Self;
  fListActifItem.fOnChange:= ChangedItems;
end;

procedure TCompoActifItem.DeletedItem(Sender: TObject; BaseActifItem: TBaseActifItem);
begin
  if assigned(FOnDelete) then FOnDelete(Self, BaseActifItem);
end;

destructor TCompoActifItem.Destroy;
begin
  fListActifItem.free;
  inherited;
end;

initialization
  RegisterOwnClass(TBaseActifItem);
  RegisterOwnClass(TNamedActifItem);
  RegisterOwnClass(TMiniPerso);





end.

