{-----------------------------------------------------------------------------
 Unit Name: UDataServerNet
 Important: Cette unit doit tre la premiere fiche cr automatiquement
            dans votre projet.
            Voir menu Projet/Options et onglet Fiches
 Author   : Manuel
 Purpose  : Facilit la gestion des methodes de connexion et l'utilisation
            de AdkServeur et ADKDispatchWSA en fournissant des mthodes
            et proprit de "haut niveau"

 History: - 10 janvier 03: Ajout de sctext121: Permet l'envoie d'un texte d'un perso
          vers un seul autre
          - 09 Janvier 03: Mise en fonction e la version 1.0.0.3 du seveur
-----------------------------------------------------------------------------}


unit UDataServerNet;

interface

uses
  SysUtils, Classes, inifiles, ADKServer, ADKWSA, ADKSession, AdkNETData,
  definitions, Gest_file, Personnage, PathMap;

type
  TDataServerNet = class(TDataModule)
    ADKServer1: TADKServer;
    ADKDispatchWSA1: TADKDispatchWSA;
    MapsPathFind1: TMapsPathFind;
    procedure ADKDispatchWSA1Enter(Sender: TObject; Session: TADKSession;
      const ID: Cardinal);
    procedure ADKDispatchWSA1Start(Sender: TObject);
    procedure ADKDispatchWSA1Stop(Sender: TObject);
    procedure ADKDispatchWSA1Leave(Sender: TObject; Session: TADKSession;
      const ID: Cardinal);
    procedure ADKDispatchWSA1Read(Sender: TObject; Session: TADKSession;
      Count: Integer);
    procedure ADKDispatchWSA1Error(Sender: TObject; Msg: string);
    procedure ADKServer1Logon(Sender: TObject; Session: TADKSession;
      const ID: Cardinal; Name, Password: string);
    procedure ADKServer1Logout(Sender: TObject; Session: TADKSession;
      const ID: Cardinal);
    procedure ADKServer1Message(Sender: TObject; Session: TADKSession;
      const ID: Cardinal; const Cmd: TServCmd; Msg: string; Attr: TTextAttr);
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
    procedure ADKServer1Negociate(Sender: TObject; Session: TADKSession;
      const ID: Cardinal; Negociate: TNegociate);
    procedure ADKServer1String(Sender: TObject; Session: TADKSession;
      const ID: Cardinal; const Cmd: TServCmd; Msg: string);
    procedure ADKServer1Cardinal(Sender: TObject; Session: TADKSession;
      const ID: Cardinal; const Cmd: TServCmd; const Value: Cardinal);
    procedure ADKServer1Message21(Sender: TObject; Session: TADKSession;
      const ID: Cardinal; const Cmd: TServCmd;
      const IDDestinataire: Cardinal; Msg: string; Attr: TTextAttr);
    procedure ADKServer1String21(Sender: TObject; Session: TADKSession;
      const ID: Cardinal; const Cmd: TServCmd;
      const IDDestinataire: Cardinal; Msg: string);
    procedure ADKServer1Position(Sender: TObject; Session: TADKSession;
      const ID: Cardinal; const cmd: TServCmd;
      PositionInfo: TPositionInfo);
    procedure ADKServer1NoData(Sender: TObject; Session: TADKSession;
      const ID: Cardinal; const cmd: TServCmd);
    procedure ADKServer1RecivePJ(Sender: TObject; Session: TADKSession;
      const ID: Cardinal; const cmd: TServCmd; RawData: TStream);
  private
    { Dclarations prives }
    ServerIniFileName: string;
    fServerNegociate: TNegociate;
    fServerInfo: TServerInfo;
    FDisplayMes: Tstrings;
    fPort: word;
    fNameFilePj: string;
    FManagerFilePj: TManagerFile;
    IndexOfPIZI: integer;
    fVersionMiniClientAutorized: Word; //N de version minimale de client autoris  se connect
    fUrlUpdateDispo: string;


    procedure SetPort(const Value: Word);
    procedure ReadInifile;
    procedure WriteInifile;

    procedure Display(Msg: string);
    procedure LoadMapsInIniFile(Inifile: Tinifile);
    function GetTotalOctet: integer;
    function GetSessionPJ(const ID: Cardinal): TADKSession; overload;
    function GetSessionPJ(const Name: string): TADKSession; overload;

    procedure ControleCreation(Session: TADKSession; PJPerso: TPJPerso);
    {Trouver toute les TADKSession contenant des persos proche dans la zone d'influence de celui qui met la requete}
    procedure FirstPIZI;
    function EofPIZI: boolean;
    function NextPIZIByDist(const MyPos: TR_Position; PiziDist: integer): TADKSession;
    function NextPIZI(const MyPos: TR_Position): TADKSession; // Zone Influence
    function NextPIZIT(const MyPos: TR_Position): TADKSession;// Zone Influence+ZoneTampon
    {fin des methodes PIZI (Perso In my Zone Influence}
    function IsPositionFree(const IDMap, X, Y: Word): boolean;
    procedure PositionChangeTo(const IDent: Cardinal;
      const cmd: TServCmd; const APosition: TPositionInfo);

  public
    { Dclarations publiques }
    function GetCustomPerso(const IDPerso: integer): TCustomPerso;
    function IsConnected(Name: string): boolean; overload;
    function IsConnected(const IDPerso: cardinal): boolean; overload;
    //function GetSocketPJ(const Name: string): TWSocketPJ; overload;
    //function GetSocketPJ(const IDPerso: integer): TWSocketPJ; overload;

    property DefaultDisplay: Tstrings read FDisplayMes write FDisplayMes;
    property ServerNegociate: TNegociate read fServerNegociate write fServerNegociate;
    property NumUpdate: integer read fServerNegociate.update write fServerNegociate.update;
    property ServerInfo: TServerInfo read fServerInfo write fServerInfo;
    property Port: word read Fport write SetPort;
    property VersionAutorized: Word read fVersionMiniClientAutorized write fVersionMiniClientAutorized;
    property UrlUpdateDispo: string read fUrlUpdateDispo write fUrlUpdateDispo;
    //property TotalOctet: integer read GetTotalOctet;
  end;

var
  DataServerNet: TDataServerNet;

implementation
uses types, forms, ResStr, globals, StrStuff, Stuff,
  ObjetsActif;
{$R *.dfm}

const IniSectionOption = 'options';


procedure TDataServerNet.LoadMapsInIniFile(Inifile: Tinifile);
var AListMap: TStringlist;
  i: integer;
  BasicMapName, CompleteMapFilename: string;
begin
  //Chargement des maps
  try
    AListMap := TStringlist.create;
    //CARTES
    Inifile.ReadSectionValues('CARTES', AListMap);
    for i := 0 to AListMap.Count - 1 do begin
      CompleteMapFilename := ExtractfilePath(application.exename) + 'data\cartes\' + Extractword(2, AListMap[i], [chrEgal]) + '.adk';
      if fileExists(CompleteMapFilename) then
        MapsPathFind1.Maplist.AddMap(StrToInt(Extractword(1, AListMap[i], [chrEgal])), CompleteMapFilename)
    end;
  finally
    AListMap.Free;
  end;
end;

procedure TDataServerNet.ReadInifile;
var Inifile: Tinifile;
begin
  Inifile := Tinifile.create(ServerIniFileName);
  try
    fServerNegociate.Server := Inifile.ReadString(IniSectionOption, 'ServerName', '');
    fServerNegociate.Version := Inifile.ReadInteger(IniSectionOption, 'version', 0); // Version du serveur
    fServerNegociate.Update := Inifile.ReadInteger(IniSectionOption, 'update', 0); // Update Client disponible en tlchargement
    fServerInfo.Influence := Inifile.ReadInteger(IniSectionOption, 'RayonInflu', 14); // Rayon d'influence indiqu en nombre de tilde
    fServerInfo.ZoneTampon := Inifile.ReadInteger(IniSectionOption, 'ZoneTampon', 7); // Rayon d'influence indiqu en nombre de tilde
    fServerInfo.Speed := Inifile.ReadInteger(IniSectionOption, 'VitesseDepl', 100); // Vitesse de dplacement "standard"
    fNameFilePj := Inifile.ReadString(IniSectionOption, 'FileNamePj', 'Pj.rpj'); // Nom du fichier des persos
    fNameFilePj := ExtractFilePath(Application.exename) + fNameFilePj;
    fVersionMiniClientAutorized := Inifile.ReadInteger(IniSectionOption, 'MinVersionAutized', 1010);
    fUrlUpdateDispo := Inifile.Readstring(IniSectionOption, 'UrlUpdate', rsNoUrlUpdate);

    LoadMapsInIniFile(Inifile); // Chargement des maps.
  finally
    Inifile.destroy;
  end;
end;

procedure TDataServerNet.WriteInifile;
var Inifile: Tinifile;
begin
  Inifile := Tinifile.create(ServerIniFileName);
  try
    Inifile.WriteString(IniSectionOption, 'ServerName', fServerNegociate.Server);
    Inifile.WriteInteger(IniSectionOption, 'version', fServerNegociate.Version);
    Inifile.WriteInteger(IniSectionOption, 'update', fServerNegociate.Update);
    Inifile.WriteInteger(IniSectionOption, 'RayonInflu', fServerInfo.Influence);
    Inifile.WriteInteger(IniSectionOption, 'ZoneTampon', fServerInfo.ZoneTampon);
    Inifile.WriteInteger(IniSectionOption, 'VitesseDepl', fServerInfo.Speed);
    Inifile.WriteInteger(IniSectionOption, 'MinVersionAutized', fVersionMiniClientAutorized);
    Inifile.Writestring(IniSectionOption, 'UrlUpdate', fUrlUpdateDispo);
  finally
    Inifile.destroy;
  end;
end;

function TDataServerNet.GetCustomPerso(const IDPerso: integer): TCustomPerso;
var i: integer;
begin
  result := nil;
  for i := 0 to ADKDispatchWSA1.ListSessions.Count - 1 do begin
    with TADKSession(ADKDispatchWSA1.ListSessions.Items[i]) do
      result := ListPersos.GetPerso(IDPerso);
    if assigned(result) then exit;
  end;
end;

{Search Session With Perso In Zone Influence (PIZI)}

procedure TDataServerNet.FirstPIZI;
begin
  IndexOfPIZI := 0;
end;

function TDataServerNet.EofPIZI: boolean;
begin
  result := IndexOfPIZI >= ADKDispatchWSA1.ListSessions.count;
end;

function TDataServerNet.NextPIZIByDist(const MyPos: TR_Position; PiziDist: integer): TADKSession;
var i: integer;
begin
  result := nil;
  while not EofPIZI do begin
    with TADKSession(ADKDispatchWSA1.ListSessions.Items[IndexOfPIZI]) do begin
      for i := 0 to ListPersos.ListPerso.Count - 1 do begin //ListPersos est un TComponent qui contient une TList "ListPerso" qui gere les persos
        with ListPersos.ListPerso.Items[i] do begin
          if (MyPos.Map = MapNumber) and
            (Dist2d(point(MyPos.xyz.x, MyPos.xyz.y), Position) <= fServerInfo.Influence) then begin
            result := TADKSession(ADKDispatchWSA1.ListSessions.Items[IndexOfPIZI]);
            inc(IndexOfPIZI);
            exit;
          end;
        end;
      end;
    end;
    inc(IndexOfPIZI);
  end;
end;

//renvoie une sesion se trouvant dans la "zone d'influence" de MyPos
function TDataServerNet.NextPIZI(const MyPos: TR_Position): TADKSession;
begin
  result:= NextPIZIByDist(MyPos, fServerInfo.Influence);
end;

//renvoie une sesion se trouvant dans la "zone d'influence + Zone tampon" de MyPos
function TDataServerNet.NextPIZIT(const MyPos: TR_Position): TADKSession;
begin
  result:= NextPIZIByDist(MyPos, fServerInfo.Influence+ fServerInfo.ZoneTampon);
end;


{End of PIZI Methodes}

function TDataServerNet.IsConnected(Name: string): boolean;
begin
  result := IsConnected(GetIdFromName(Name));
end;

function TDataServerNet.IsConnected(const IDPerso: Cardinal): boolean;
var i, j: integer;
begin
  result := false;
  for i := 0 to ADKDispatchWSA1.ListSessions.Count - 1 do begin
    with TADKSession(ADKDispatchWSA1.ListSessions.Items[i]) do begin
      for j := 0 to ListPersos.ListPerso.Count - 1 do begin //ListPersos est un TComponent qui contient une TList "ListPerso" qui gere les persos
        if IDperso = ListPersos.ListPerso.Items[j].ID then begin
          result := true;
          exit;
        end;
      end;
    end;
  end;
end;

function TDataServerNet.GetSessionPJ(const ID: Cardinal): TADKSession;
var i, j: integer;
begin
  result := nil;
  for i := 0 to ADKDispatchWSA1.ListSessions.Count - 1 do begin
    with TADKSession(ADKDispatchWSA1.ListSessions.Items[i]) do begin
      for j := 0 to ListPersos.ListPerso.Count - 1 do begin //ListPersos est un TComponent qui contient une TList "ListPerso" qui gere les persos
        if ID = ListPersos.ListPerso.Items[j].ID then begin
          result := TADKSession(ADKDispatchWSA1.ListSessions.Items[i]);
          exit;
        end;
      end;
    end;
  end;
end;

function TDataServerNet.GetSessionPJ(const Name: string): TADKSession;
begin
  result := GetSessionPJ(GetIdFromName(Name));
end;


procedure TDataServerNet.Display(Msg: string);
var
  I: Integer;
begin
  if assigned(FDisplayMes) then begin
    FDisplayMes.BeginUpdate;
    try
      if FDisplayMes.Count > 250 then begin
        for I := 1 to 50 do
          FDisplayMes.Delete(0);
      end;
      FDisplayMes.Add(Msg);
    finally
      FDisplayMes.EndUpdate;
    end;
  end;
end;


function TDataServerNet.GetTotalOctet: integer;
begin
  result := Tag;
end;


{ * * * *  * * }

procedure TDataServerNet.DataModuleCreate(Sender: TObject);
begin
  ServerIniFileName := ExtractFilePath(Application.exename) + 'server.ini';
  ReadInifile;
  // Gestionnaire du fichier des PJ. Servira peut etre aussi au Pnj... On verra
  FManagerFilePJ := TManagerFile.create(fNameFilePj);
end;

procedure TDataServerNet.DataModuleDestroy(Sender: TObject);
begin
  try
    // FManagerFilePJ doit encore exist  l'appel de ADKDispatchWSA1.stop
    // si on veut pouvoir enregistr les persos.
    // Donc, ne pas modifier l'ordre de destruction.
    ADKDispatchWSA1.stop;
    WriteIniFile;
  finally
    FManagerFilePJ.Destroy;
  end;
end;

procedure TDataServerNet.SetPort(const Value: Word);
var IsActive: boolean;
begin
  // Change le n de port du serveur.
  // Tout deconnecter faire shutdown et reconnecter
  if Value <> ADKDispatchWSA1.Port then begin
    IsActive := ADKDispatchWSA1.Active;
    ADKServer1.Stop;
    ADKDispatchWSA1.Port := Value;
    if IsActive then ADKServer1.Start;
  end;
end;

procedure TDataServerNet.ADKDispatchWSA1Enter(Sender: TObject;
  Session: TADKSession; const ID: Cardinal);
begin
  with Session do begin
    Display(rsNewSessionOpened);
    //ET si je n'envoie rien ? Kk arrive sur le client ?
    //Parceque normalement c'est le client qui ngocie en 1er (a voir si c'tait vrai ca)
    //Send_Negociate('Serveur [' + fServerNegociate.Server + ']', fServerNegociate.Version , False);
    Send_ServerInfo(fServerInfo.StartDate, now, fServerInfo.Influence,
      fServerInfo.ZoneTampon, fServerInfo.Speed);
  end;
end;

procedure TDataServerNet.ADKDispatchWSA1Start(Sender: TObject);
begin
  fServerInfo.StartDate := Now;
  Display(format(rsServerIsOpen, [TimeToStr(Now), DateToStr(Now)]));
  Display(Format(rsNumPort, [ADKDispatchWSA1.port]));
  tag := 0; // Le tag du datamodule est utilis pour comptabilis le nombre d'octet reu.
end;

procedure TDataServerNet.ADKDispatchWSA1Stop(Sender: TObject);
begin
  Display(rsServerClosed);
end;

procedure TDataServerNet.ADKDispatchWSA1Leave(Sender: TObject;
  Session: TADKSession; const ID: Cardinal);
var i: integer;
  PIZISession: TADKSession;
  IDentification: Cardinal;
  MapFinder: TMapFinder;
begin
  if assigned(session) then begin
    try
      Session.Send_TextInfo(ID, scCloseConnect, '' {rsErr1501 + rsBreakLine+rsCallAdkMaster});

      for i := 0 to Session.ListPersos.Count - 1 do begin
        with Session.ListPersos.ListPerso[i] do begin
          IDentification := GetIdFromName(Name);
          FirstPIZI; //<< INDISPENSABLE !!!
          while not EofPIZI do begin
            PIZISession := NextPIZIT(TRPosition); // NextPIZIT car il faut supprim les perso de la zone TAMPON aussi.(au lieu de NextPIZI (sans le T) avant)
            if assigned(PIZISession) then
              if PIZISession <> Session then
                PIZISession.Send_Cardinal(Id, scMiniPersoDeco, IDentification);
          end;
        end;
      end;
    finally
      // Quoi qu'il arrive enregistrer les personnages que contient la Session qui se ferme.
      if assigned(session) then
        with Session.ListPersos do begin
          for i := 0 to Count - 1 do
            with ListPerso[i] do begin
              if ListPerso[i] is TPJPerso then begin
                with TPJPerso(ListPerso[i]) do
                  FManagerFilePj.writeRecord(RecordPerso);
              end else {#TODO1 Placer ici l'enrgeistrement des MONSTRES};
              Display(Name + ': Enregistr.');
            // Marqu la position du perso qui se dconnecte comme LIBRE
            // -> SUPPRIM Car on ne marque plus la position occup sur le Imap
            end;
        end;
    end;
  end;
end;

procedure TDataServerNet.ADKDispatchWSA1Read(Sender: TObject;
  Session: TADKSession; Count: Integer);
begin
  if (tag + count) < MaxInt then
    Tag := tag + count
  else Tag := 0; // On rinitialise sinon on risque de dpasser MAXINT.
 //display(IntToStr(Tag)+' octets reus');
end;

procedure TDataServerNet.ADKDispatchWSA1Error(Sender: TObject;
  Msg: string);
begin
  Display(msg);
end;


{ Events du Server * * * * * * * * * * * * * *}

procedure TDataServerNet.ADKServer1Negociate(Sender: TObject;
  Session: TADKSession; const ID: Cardinal; Negociate: TNegociate);
begin
  // ID n'est pas encore utiliser ici car il ne correspond  rien pour le moment
  // Dans Negociate.Server se trouve un des elements  vrifier
  // avant de donner acces au server.
  {#todo 2 A activer plus tard}
  //If Negociate.Server <> '0' then
  //  Session.Send_TextInfo(ID, scCloseConnect, rsErr501 + rsBreakLine+rsCallAdkMaster);

  { Vrifier si le n de version du client (qui est crit en "dur" dans
    l'exe du client est AU MOINS Egal ou Suprieur (= ou >) au n de version
    autoris (CAD: fVersionMiniClientAutorized). }


  if (Negociate.Update = 1) then begin // IL FAUT ARRETER les version 1 d'ADKCenter
    Session.Send_TextInfo(ID, TServCmd(5), rsOldVersion); //5= scDisconnect
  end else begin
    if (Negociate.Update < fServerNegociate.Update) then
      Session.Send_TextInfo(ID, scPatchDispo, fUrlUpdateDispo); //<< fUrlUpdateDispo contient l'Url complete ou tlcharger le fichier .Zip (ou .exe)
    if (Negociate.Version < fVersionMiniClientAutorized) then //<< Premiere version de ADKCenter
      Session.Send_TextInfo(ID, scDisConnect, rsOldVersion)
    else
      { Le false en fin de proc est a vrifier suivant la blackList }
      Session.Send_Negociate(fServerNegociate.Server, fServerNegociate.Update, False);
  end;
end;

procedure TDataServerNet.ADKServer1Logon(Sender: TObject;
  Session: TADKSession; const ID: Cardinal; Name, Password: string);
var GG_Personnage: TGG_Personnage;
  CustomPerso: TPJPerso;
  PIZISession: TADKSession;
  i: integer;
  MapName: string;
  MemoryStream: TmemoryStream;
  procedure SendFatalError(msg: string);
  begin
    Session.Send_TextInfo(GetIdFromName(Name), scDisConnect, msg);
  end;
begin
  //Verifier avant tout que le nom existe
  if not FManagerFilePJ.GetRecord(name, GG_Personnage) then begin
    SendFatalError(Format(rsErr1504, [name]));
    exit;
  end;
  // Vrifier si dja connect
  if IsConnected(GetIdFromName(Name)) then begin
    // Si le password est bon
    if ansisametext(GG_Personnage.Personnage.CaractAdd.Password, Password) then begin
      ADKDispatchWSA1.CloseSession(GetSessionPJ(Name));
      //Le perso  t enregistr donc on RErcupre son record pour tre sur d'tre bien  jour
      FManagerFilePJ.GetRecord(name, GG_Personnage);
    end else begin
      SendFatalError(Format(rsErr1506, [name]));
      exit;
    end;
  end;
  if not ansisametext(GG_Personnage.Personnage.CaractAdd.Password, Password) then begin
    SendFatalError(Format(rsErr1506, [Name]));
    exit;
  end;
  // Protections  mettre ici et en cas de banish ou BlackList
  // utiliser la proc SendFatalError
  //SendFatalError(Format(rsErr700,[GG_Personnage.Personnage.CaractAdd.name]));
  {#todo1 Enrigistrer les incident dans un fichier LOG}
  GG_Personnage.DateConnect := now; //On change la date de connexion
  //ajouter le perso  la session
  CustomPerso := TPJPerso.Create;
  CustomPerso.RecordPerso := GG_Personnage.Personnage;
  {#todo1 Avant d'envoyer le perso il faudra vrifier que Position.xyz.xy est libre
   sinon il faudra que le serveur lui trouve une place libre}
  MemoryStream := TMemoryStream.Create;
  try
    CustomPerso.writedata(MemoryStream);
    Session.Send_PlayerInfo(CustomPerso.ID, MemoryStream);
    Session.ListPersos.ListPerso.Add(CustomPerso);
  {Envoyer le nom de la carte (pas d'extension, rien, juste le nom)
    prcd par le n de la carte.
    Le n de la carte et le nom de la carte sont spar par un Point-virgule[;]}
    MapName := MapsPathFind1.Maplist.GetMapNameByID(CustomPerso.MapNumber);
    if MapName = '' then begin
      MapsPathFind1.Maplist.GetMapNameByID(0); // La carte 0 (zro) doit TOUJOURS exister}
      CustomPerso.MapNumber := 0;
    end;
    Session.Send_TextInfo(GetIdFromName(Name), scLoadMap,
      intTostr(CustomPerso.MapNumber) + ChrPvirgule + MapName);
    // Envoyer  toutes les sesions contenant un perso proche du mien
    // mon perso sous forme de TrecMiniPerso
    MemoryStream.Clear; //"Nettoyage" du TMemoryStream
    FirstPIZI; //<< INDISPENSABLE !!
    while not EofPIZI do begin
      PIZISession := NextPIZI(CustomPerso.TRPosition);
      if assigned(PIZISession) then begin
        if PIZISession <> Session then begin // Pour ne pas m'envoyer mon propre perso sous forme de miniPerso
          try
            MemoryStream.Clear;
            CustomPersoToRaw(CustomPerso.Id, CustomPerso, MemoryStream);
            PIZISession.Send_RawData(CustomPerso.Id, scActifItem, MemoryStream);
          except on ESocketSendError do begin
              ADKDispatchWSA1.CloseSession(PIZISession);
              break;
            end;
          end;
          // M'envoyer tout les persos de cette session qui sont proche de moi
          for i := 0 to PIZISession.ListPersos.ListPerso.Count - 1 do begin //ListPersos est un TComponent qui contient une TList "ListPerso" qui gere les persos
            with PIZISession.ListPersos.ListPerso.Items[i] do begin
              if (CustomPerso.MapNumber = MapNumber) and (Dist2d(CustomPerso.Position, Position) <= fServerInfo.Influence) then begin
                MemoryStream.Clear;
                CustomPersoToRaw(Id, PIZISession.ListPersos.ListPerso.Items[i], MemoryStream);
                Session.Send_RawData(Id, scActifItem, MemoryStream);
              end;
            end;
          end;
        end;
      end;
    end;
    session.PlayingGame := true; //<< La session  au moins un joueur qui joue
  finally
    MemoryStream.Free;
    // Envoie de la commande indiquant que le jeu peut commencer
    session.Send_StartGame;
  end;
end;

procedure TDataServerNet.ADKServer1Logout(Sender: TObject;
  Session: TADKSession; const ID: Cardinal);
begin
 //memo1.lines.Add('Bye '+Session.Name);
end;

procedure TDataServerNet.ADKServer1Message(Sender: TObject;
  Session: TADKSession; const ID: Cardinal; const Cmd: TServCmd;
  Msg: string; Attr: TTextAttr);
var i: integer;
  PIZISession: TADKSession;
begin
  case Cmd of
    scText: begin // texte sans limite de distance (donc envoy  toute les sessions
        for i := 0 to ADKDispatchWSA1.ListSessions.Count - 1 do begin
          with TADKSession(ADKDispatchWSA1.ListSessions.Items[i]) do
            Send_TextAttr(ID, Cmd, Attr, Msg);
        end;
      end;
    scTextRp: begin
        with Session.ListPersos.GetPerso(ID) do begin
          FirstPIZI; //<< INDISPENSABLE !!
          while not EofPIZI do begin
            PIZISession := NextPIZI(TRPosition); // GetTRPosition dcoule de "with Session.ListPersos.GetPerso(ID) do begin"
            if assigned(PIZISession) then
              PIZISession.Send_TextAttr(Id, cmd, Attr, msg);
          end;
        end;
      end;
  else end;
end;

(*
gerer l'arriv des commandes de 1 vers 1
Normalement on ne devrait avoir que du texte pour les 1 vers 1
c'est pour cela que j'ai prvu TTextInfo21 et TMessageInfo21
qui donne l'ID du perso a qui doit tre envoyer avec le texte
   sctext121: begin
     ASession:= GetSessionPJ(const ID: Cardinal): TADKSession; overload;
   end;
*)

procedure TDataServerNet.ADKServer1String(Sender: TObject;
  Session: TADKSession; const ID: Cardinal; const Cmd: TServCmd;
  Msg: string);
var i: integer;
  CustomPerso: TPJPerso;
  MemoryStream: TmemoryStream;
begin
  case Cmd of
    scNameExist: begin
        //Msg contient le nom du perso  vrifier
        //On renvoie True ou false et le nom vrifi
        // Le nom du perso ne peut pas contenir de @
        if pos('@', msg) <> 0 then
          Session.Send_TextInfo(ID, cmd,
            BoolToStr(true) + ChrPVirgule + msg)
        else
          Session.Send_TextInfo(ID, cmd,
            BoolToStr(FManagerFilePJ.RecordExist(Msg)) + ChrPVirgule + msg);
      end;
    scChangeUrlUpdate: fUrlUpdateDispo := Msg;
    scShowMessage:
      for i := 0 to ADKDispatchWSA1.ListSessions.Count - 1 do
        TADKSession(ADKDispatchWSA1.ListSessions.Items[i]).Send_TextInfo(ID, cmd, Msg);
    scGetAllPerso: begin // Renvoie tout les perso inscrit dans la base de donne.
        CustomPerso := TPJPerso.Create;
        MemoryStream := TMemoryStream.Create;
        try
          for i := 0 to FManagerFilePJ.Count - 1 do begin
            CustomPerso.RecordPerso := FManagerFilePJ.Items[i].Personnage;
            if Msg <> '' then begin
              if ansisametext(CustomPerso.RecordPerso.CaractAdd.Password, msg) then
                CustomPerso.writedata(MemoryStream);
              Session.Send_PlayerInfo(CustomPerso.ID, MemoryStream);
            end else begin
              CustomPerso.writedata(MemoryStream);
              Session.Send_PlayerInfo(CustomPerso.ID, MemoryStream);
            end;
            MemoryStream.clear;
          end;
        finally
          CustomPerso.free;
          MemoryStream.Free;
        end;
      end;
    scGetNameAndPass: begin // Renvoie les nom et pass sous la forme nom=password
                            // 'Msg' est utilis comme filtre pour ne renvoyer que
                            // les perso dant le pass lui sont identiques.
        for i := 0 to FManagerFilePJ.Count - 1 do
          if Msg <> '' then begin
            if ansisametext(FManagerFilePJ.Items[i].Personnage.CaractAdd.Password, msg) then
              Session.Send_TextInfo(ID, cmd, FManagerFilePJ.Items[i].Personnage.CaractAdd.Name + '=' + FManagerFilePJ.Items[i].Personnage.CaractAdd.Password);
          end else Session.Send_TextInfo(ID, cmd, FManagerFilePJ.Items[i].Personnage.CaractAdd.Name + '=' + FManagerFilePJ.Items[i].Personnage.CaractAdd.Password);
      end;
  end;
end;

procedure TDataServerNet.ControleCreation(Session: TADKSession; PJPerso: TPJPerso);
  procedure SendFatalError(msg: string);
  begin
    Session.Send_TextInfo(0, scCloseConnect, msg + rsBreakLine + rsCallAdkMaster);
  end;
begin
  //Verifier avant tout que le nom n'existe pas dja
  if FManagerFilePJ.RecordExist(PJPerso.Name) then begin
    Session.Send_TextInfo(0, scNameExist, Booltostr(true) + ChrPVirgule + PJPerso.Name);
    exit;
  end;
  // Protections  mettre ici et en cas de banish ou BlackList
  // utiliser la proc SendFatalError
  //SendFatalError(Format(rsErr700,[PlayerInfo.CaractAdd.Name]));
  {#todo1 Enrigistrer les incident dans un fichier LOG}
  case FManagerFilePJ.Add(PJPerso.RecordPerso) of
    -2: Session.Send_TextInfo(GetIdFromName(PJPerso.Name),
        scPersoWrited, BoolToStr(false) + ChrPVirgule +
        Format(rsErr1503, [PJPerso.Name]));
    -1: Session.Send_TextInfo(GetIdFromName(PJPerso.Name),
        scPersoWrited, BoolToStr(false) + ChrPVirgule +
        format(rsErr1502, [PJPerso.Name]));
  else
    Session.Send_TextInfo(GetIdFromName(PJPerso.Name), scPersoWrited, BoolToStr(true) + ChrPVirgule + PJPerso.Name);
  end;
end;

procedure TDataServerNet.ADKServer1RecivePJ(Sender: TObject;
  Session: TADKSession; const ID: Cardinal; const cmd: TServCmd;
  RawData: TStream);
var PJPerso: TPJPerso;
begin
  // demande de Cration d'un perso sur le serveur
  case Cmd of
    scCreatePerso2: begin //scCreatePerso (sans le 2) tait utilis pour la compatibilit avec adk 3.8)
        PJPerso := TPJPerso.create;
        try
          PJPerso.ReadData(RawData);
          ControleCreation(Session, PJPerso); //(voir proc au dessus)
        finally
          PJPerso.destroy;
        end;
      end;
  else end;
end;

// Cette procedure cherche le personnage avec l'ID= IDPersoAenvoyer
// et l'envoie sous forme de TminiPerso  la Session=Session

procedure RequestedMiniPerso(const Session: TADKSession;
  const IdDemandeur, IDPersoAenvoyer: Cardinal);
var CustomPerso: TCustomPerso;
  MemoryStream: TMemoryStream;
begin
  with DataServerNet do begin
    CustomPerso := GetCustomPerso(IDPersoAenvoyer);
    if assigned(CustomPerso) then begin
      MemoryStream := TMemoryStream.Create;
      try
        CustomPersoToRaw(CustomPerso.ID, CustomPerso, MemoryStream);
        Session.Send_RawData(CustomPerso.Id, scActifItem, MemoryStream);
        MemoryStream.clear; //<< On renvoie CustomPerso sous forme de Tminiperso au demandeur
        CustomPerso := GetCustomPerso(IdDemandeur);
        if assigned(CustomPerso) then begin
          CustomPersoToRaw(IdDemandeur, CustomPerso, MemoryStream);
          GetSessionPJ(IDPersoAenvoyer).Send_RawData(IdDemandeur, scActifItem, MemoryStream);
        end;
      finally
        MemoryStream.Free;
      end;
    end;
  end;
end;


{ Comme la prcedente mais on ne fait que renvoyer le MiniPerso au demandeur
  sans le double envoie que fait la procedure RequestedMiniPerso et qui est
  utilis par les PJ }
procedure RequestedMiniPNJ(const Session: TADKSession;
  const IdDemandeur, IDPersoAenvoyer: Cardinal);
var CustomPerso: TCustomPerso;
  MemoryStream: TMemoryStream;
begin
  with DataServerNet do begin
    CustomPerso := GetCustomPerso(IDPersoAenvoyer);
    if assigned(CustomPerso) then begin
      MemoryStream := TMemoryStream.Create;
      try
        CustomPersoToRaw(CustomPerso.ID, CustomPerso, MemoryStream);
        Session.Send_RawData(CustomPerso.Id, scActifItemPNJ, MemoryStream);
      finally
        MemoryStream.Free;
      end;
    end;
  end;
end;

procedure TDataServerNet.ADKServer1Cardinal(Sender: TObject;
  Session: TADKSession; const ID: Cardinal; const Cmd: TServCmd;
  const Value: Cardinal);
var PIZISession: TADKSession;
  CustomPerso: TCustomPerso;
  ATRPosition: TR_Position;
begin
  // Reception d'un Cardinal.
  // cette evenement est utilis par plusieurs commandes.
  case cmd of
    scChangeVitesse: fServerInfo.Speed := Value;
    scChangeInfluence: fServerInfo.Influence := Value; // Modifier la taille du rayon d'influence
    scErasePerso: begin
      // On efface que si le perso n'est pas connect. Sinon risque de problme
      // En cas d'extreme urgence il suffit d'envoyer une requete de dconexion
      // du perso suivi d'un erase. Value est l'ID du perso  ffacer.
        if not IsConnected(value) then FManagerFilePj.Erase(Value);
      end;
    scChangeUpdate: fServerNegociate.Update := value;
    scChangeVersionMiniClientAutorized: fVersionMiniClientAutorized := Value;
    // renvoy le Personnage (ou l'actifItem plutot) ayant l'iD = VALUE
    scGetActifItem: RequestedMiniPerso(Session, ID, Value);
    //ID est le PNJ  envoyer et Value est l'id du perso  qui on doit envoy le PNJ
    // Il faut se rappeler que le 1er ID est TOUJOURS l'ID de celui qui envoie la requete
    // Dans notre cas il s'agit d'un PNJ qui voit venir un PJ dans ca zone et qui
    // doit lui envoyer son signalement.
    // Comme la procedure RequestedMiniPNJ atend en premier l'ID du DEMANDEUR et
    // ensuite l'ID  ENVOYER, c'est le pourquoi de passer les parametre
    // tel quel et non pas comme on fait ailleur en passant dabord ID et ensuite Value.
    scActifItemPNJ: RequestedMiniPNJ(Session, Value, ID);
    scRotate: begin
        CustomPerso := GetCustomPerso(ID);
        if assigned(CustomPerso) then begin
          ATRPosition := CustomPerso.TrPosition;
          ATRPosition.Dir := TDirection(Value);
          FirstPIZI; //<< INDISPENSABLE !!
          while not EofPIZI do begin
            PIZISession := NextPIZI(ATRPosition);
            if assigned(PIZISession) then // Pas de verif pour savoir si on s'envoie a notre propre session la rotate car la session peut etre celle d'un Pnj
              PIZISession.Send_Cardinal(ID, cmd, Value);
          end;
        end;
      end;
    scGetNbrIncrit: Session.Send_Cardinal(ID, cmd, FManagerFilePJ.Count);
  else end;
end;

procedure TDataServerNet.ADKServer1Message21(Sender: TObject;
  Session: TADKSession; const ID: Cardinal; const Cmd: TServCmd;
  const IDDestinataire: Cardinal; Msg: string; Attr: TTextAttr);
var SessionToSend: TADKSession;
begin
  // Reception d'un texte avec attribut  rediriger
  // uniquement vers IDDestinataire
  SessionToSend := GetSessionPJ(IDDestinataire);
  if assigned(SessionToSend) then
    SessionToSend.Send_TextAttr121(ID, cmd, IdDestinataire, Attr, Msg);
end;

procedure TDataServerNet.ADKServer1String21(Sender: TObject;
  Session: TADKSession; const ID: Cardinal; const Cmd: TServCmd;
  const IDDestinataire: Cardinal; Msg: string);
var SessionToSend: TADKSession;
begin
  // Reception d'un texte brut (sans attribut)  rediriger
  // uniquement vers IDDestinataire
  SessionToSend := GetSessionPJ(IDDestinataire);
  if assigned(SessionToSend) then
    SessionToSend.Send_TextInfo121(ID, cmd, IdDestinataire, Msg);
end;

//Nombre de perso connected (total sans distinction PJ/PNJ)

function NumberOfPersonnages: cardinal;
var i, j: integer;
begin
  with DataServerNet do begin
    result := 0;
    try
      for i := 0 to ADKDispatchWSA1.ListSessions.Count - 1 do
        with TADKSession(ADKDispatchWSA1.ListSessions.Items[i]) do
          result := result + ListPersos.ListPerso.Count;
    except on exception do result := 0;
    end;
  end;
end;

procedure TDataServerNet.ADKServer1NoData(Sender: TObject;
  Session: TADKSession; const ID: Cardinal; const cmd: TServCmd);
begin
  { Reception de commande qui n'envoie rien d'autre que la commande 
    executer (sans aucune donn, d'ou le nom NODATA).
    Ses commandes sont gnralement des demandes de renseignement au
    serveur (combien de joueurs y at'il etc). }
  case cmd of
    scGetNbrPlayers: Session.Send_Cardinal(ID, cmd, NumberOfPersonnages); // Renvoie le nombre de joueurs connect
    scGetNbrSession: Session.Send_Cardinal(ID, cmd, ADKDispatchWSA1.ListSessions.Count); // Renvoie le nombre de session en cours
    scGetIdAllPlayer: Session.Send_TextInfo(ID, scTextServ, 'Commande non ralis!!!!'); // Renvoie les ID de tout les joueurs connects
  else end;
end;
(****
procedure TDataServerNet.ChangeMap(Session: TADKSession; const ID: Cardinal;
  CustomPerso: TCustomPerso; const APosition: TPositionInfo);
var MapName: string;
  MemoryStream: TmemoryStream;
begin
  MemoryStream := TMemoryStream.Create;
  try
    Ici c'est le nouveau pour sauter de map en map
    CustomPerso.MapNumber:= APosition.map;
    {Envoyer le nom de la carte (pas d'extension, rien, juste le nom)
    prcd par le n de la carte.
    Le n de la carte et le nom de la carte sont spar par un Point-virgule[;]}
    MapName := MapsPathFind1.Maplist.GetMapNameByID(APosition.map);
    if MapName = '' then begin
      MapsPathFind1.Maplist.GetMapNameByID(0); // La carte 0 (zro) doit TOUJOURS exister}
      CustomPerso.MapNumber := 0;
    end;
    Session.Send_TextInfo(GetIdFromName(Name), scLoadMap,
      intTostr(CustomPerso.MapNumber) + ChrPvirgule + MapName);
    // Envoyer  toutes les sesions contenant un perso proche du mien
    // mon perso sous forme de TrecMiniPerso
    MemoryStream.Clear; //"Nettoyage" du TMemoryStream
    FirstPIZI; //<< INDISPENSABLE !!
    while not EofPIZI do begin
      PIZISession := NextPIZI(CustomPerso.TRPosition);
      if assigned(PIZISession) then begin
        if PIZISession <> Session then begin // Pour ne pas m'envoyer mon propre perso sous forme de miniPerso
          try
            MemoryStream.Clear;
            CustomPersoToRaw(CustomPerso.Id, CustomPerso, MemoryStream);
            PIZISession.Send_RawData(CustomPerso.Id, scActifItem, MemoryStream);
          except on ESocketSendError do begin
              ADKDispatchWSA1.CloseSession(PIZISession);
              break;
            end;
          end;
          // M'envoyer tout les persos de cette session qui sont proche de moi
          for i := 0 to PIZISession.ListPersos.ListPerso.Count - 1 do begin //ListPersos est un TComponent qui contient une TList "ListPerso" qui gere les persos
            with PIZISession.ListPersos.ListPerso.Items[i] do begin
              if (CustomPerso.MapNumber = MapNumber) and (Dist2d(CustomPerso.Position, Position) <= fServerInfo.Influence) then begin
                MemoryStream.Clear;
                CustomPersoToRaw(Id, PIZISession.ListPersos.ListPerso.Items[i], MemoryStream);
                Session.Send_RawData(Id, scActifItem, MemoryStream);
              end;
            end;
          end;
        end;
      end;
    end;
    session.PlayingGame := true; //<< La session  au moins un joueur qui joue
  finally
    MemoryStream.Free;
    // Envoie de la commande indiquant que le jeu peut commencer
    session.Send_StartGame;
  end;
end;
***)

function TDataServerNet.IsPositionFree(const IDMap, X, Y: Word): boolean;
var MapFinder: TMapFinder;
begin
  MapFinder := MapsPathFind1.Maplist.GetMapByID(IDMap);
  if assigned(MapFinder) then
    result := MapFinder.CanWalk(point(x, y))
  else result := false;
end;

// Envoyer  tout les perso dans ma zone d'influence+ZoneTampon la commande
// de deplacement.
// APosition est la position (voir AdkDataNet) vers laquelle va se diriger ce perso.

procedure TDataServerNet.PositionChangeTo(const IDent: Cardinal;
  const cmd: TServCmd; const APosition: TPositionInfo);
var CustomPerso: TCustomPerso;
  i, CountSession: integer;
  MapFinder: TMapFinder;
  NewPos, OldPos: Tpoint;
  NewDist: integer;
begin
  CustomPerso := GetCustomPerso(IDent);
  if assigned(CustomPerso) then begin
    MapFinder := MapsPathFind1.Maplist.GetMapByID(APosition.map);
    if assigned(MapFinder) then begin
      NewPos := Point(APosition.x, APosition.y);
      OldPos := CustomPerso.Position;
      //* * * * * * Modification de la position du personnage  * * * *
      CustomPerso.Position := NewPos;
      // Marquer l'ancienne position du perso comme  nouveau libre
      // MapFinder.Colors[OldPos]:= clWalk; << On n'utilise plus cela
      // Et celle ou il va comme occup
      // MapFinder.Colors[NewPos]:= clDontWalk; << Ni ca
      for CountSession := 0 to ADKDispatchWSA1.ListSessions.Count - 1 do
        with TADKSession(ADKDispatchWSA1.ListSessions.Items[CountSession]) do begin
          for i := 0 to ListPersos.ListPerso.Count - 1 do //ListPersos est un TComponent qui contient une TList "ListPerso" qui gere les persos
            with ListPersos.ListPerso.Items[i] do
              if (APosition.map = MapNumber) then //PositionInfo.map= Le map ou se trouve mon perso - MapNumber= Map ou se trouve l'autre perso
                //envoy la commande si le perso tait AVANT dans notre zone d'influence
                // ou s'il y entre.
                if (Dist2d(OldPos, Position) <= (fServerInfo.Influence + fServerInfo.ZoneTampon)) then begin
                  Send_Position(IDent, Cmd, APosition);
                  // Pas la peine d'aller plus loin
                  // car si la session est une session PNJ et si elle contient 30 perso dans ma zone
                  // ca fait envoyer 30 fois la position, alors que c'est inutile
                  Break;
                end;
        end;
    end;
  end;
end;

procedure TDataServerNet.ADKServer1Position(Sender: TObject;
  Session: TADKSession; const ID: Cardinal; const cmd: TServCmd;
  PositionInfo: TPositionInfo);
var JepeutMeDeplacer: Boolean;
  CustomPerso: TCustomPerso;
  ATR_Position: TR_Position;
begin
  // Reception d'un TPositionInfo. Voir AdkData
  // cette evenement est utilis par plusieurs commandes.
  // verifier si la position ou veut aller le perso est libre sur son TMap
  case cmd of
    scPlaceAt,
      scTeleport,
      scMoveTo,
      scRunTo,
      scJump: begin
        JepeutMeDeplacer := IsPositionFree(PositionInfo.map, PositionInfo.x, PositionInfo.y);
        if JepeutMeDeplacer then PositionChangeTo(ID, cmd, PositionInfo)
        else Session.Send_NoDataCmd(ID, scStopmove);
      end;
    scEndMove: begin
        CustomPerso := GetCustomPerso(ID);
        if assigned(CustomPerso) then begin
          ATR_Position := CustomPerso.TRPosition;
          ATR_Position.Dir := TDirection(PositionInfo.Vitesse);
          CustomPerso.TRPosition := ATR_Position;
          PositionChangeTo(ID, cmd, PositionInfo);
        end;
      end;
    scChangeMap: begin
        beep;
      end;
  else end;
end;



end.

