unit Player;

interface

uses
  HandBook2;

type
  TDiceRoll = record
    dice: array[1..4] of byte;
    less: byte;
    total: byte;
    adjust: shortint;
  end;

  TCharacter = record
    Name: string[15];
    Password: string[15];
    Scores: TAbilities;
    ExStrength: byte;
    Race: TRace;
    Sex: TSex;
    Height: word;
    Weight: word;
    Age: word;
    MaxAge: word;
    Classes: TClasses;
    MagicSchool: TSchoolOfMagic;
    Thieving: TThievingSkills;
    Alignment: TAlignment;
    HitPoints: integer;
  end;

var
  DiceRoll: array[TAbility] of TDiceRoll;
  Character: TCharacter;

procedure RollDice;
function Roll(const Dice: TDice): integer;

function ExceptionalStrength: boolean;
function CanBeOfRace(ARace: TRace): boolean;
function CanBeOfClass(AClass: TClass): boolean;
function CanBeOfSchool(ASchool: TSchoolOfMagic): boolean;

implementation

procedure RollDice;
var
  a: TAbility;
  d: integer;
  r: integer;
begin
  FillChar(DiceRoll, SizeOf(DiceRoll), 0);
  FillChar(Character, SizeOf(Character), 0);
  Character.Alignment := NN;
  for a := Low(TAbility) to High(TAbility) do begin
    with DiceRoll[a] do begin
      less := 1;
      for d := 1 to 4 do begin
        r := Random(6) + 1;
        Dice[d] := r;
        if r < Dice[less] then less := d;
        inc(total, r);
      end;
      dec(total, Dice[less]);
    end;
  end;
  Character.ExStrength := Random(100) + 1;
end;

function Roll(const Dice: TDice): integer;
var
  d: integer;
begin
  result := 0;
  for d := 1 to Dice.Count do inc(Result, Random(Dice.faces) + 1);
end;

function ExceptionalStrength: boolean;
begin
  Result := (Character.Scores[abStrength] = 18)
    and (Character.Race <> Halfling)
    and ((Character.Classes * Warrior) <> []);
end;

function CanBeOfRace(ARace: TRace): boolean;
var
  a: TAbility;
begin
  Result := False;
  for a := Low(TAbility) to High(TAbility) do begin
    if (DiceRoll[a].Total < RacialAbilityRequirements[ARace, a].Min)
      or (DiceRoll[a].Total > RacialAbilityRequirements[ARace, a].Max) then exit;
  end;
  Result := True;
end;

function CanBeOfClass(AClass: TClass): boolean;
var
  c: TAbility;
  e: TSchoolOfMagic;
begin
  Result := False;
  if not (AClass in RaceClasses[Character.Race]) then exit;
  for c := Low(TAbility) to High(TAbility) do
    if (Character.Scores[c] < ClassAbilityMinimums[AClass, c]) then exit;
  Result := True;
// A mage need a School minimum requirement
  if AClass in Wizard then begin
    for e := Low(TSchoolOfMagic) to High(TSchoolOfMagic) do begin
      if CanBeOfSchool(e) then exit; // au moins une cole possible !
    end;
    Result := False;
  end;
end;

function CanBeOfSchool(ASchool: TSchoolOfMagic): boolean;
begin
  Result := False;
  with WizardRequirements[ASchool] do begin
    if not (Character.Race in Races) then exit;
    if Character.Scores[Ability] < Score then exit;
  end;
  Result := True;
end;


end.
