unit Gest_File;

interface
uses classes, definitions;
type

  // Surcharge la dfinition de TPersonnage (Unite Definitions)
  // Pour permettre d'ajouter des champs qui ne sont pas directement
  // en relation avec la dfinition du personnage.
  // Permet de ne pas toucher au record initial TPersonnage
  TGG_Personnage = record
    Personnage: TPersonnage; //TPersonnage est dans l'unit "Definitions" et il engloble aussi "TCharacter" qui  t dfini par Paul dans l'unit "Player"
    PositionInFile: Integer;
    Id: Cardinal;
    PNJ: boolean;
    DateCreate, DateConnect: TDateTime;
  end;

  // Record Utiliser avec Flist_GF_Index.
  // Garde un pointer vers chaque personnage dans le fichier
  // Ca bouffe de la memoire mais on devrait avoir un gain
  // de vitesse dans l'acces au dfinition des personnages dans
  // le fichier
  PIndexPersos = ^TIndexPersos;
  TIndexPersos = record
    ID: cardinal;
    PositionInFile: Integer;
    Trie: string[2]; //Heu.. ca devra servir pour tri la liste par nom. On verra ca plus tard
  end;

  TManagerFile = class(Tobject)
  private
    Flist_GF_Index: Tlist;
    FFilename: string;
    F: file of TGG_Personnage;
    FLastRecordAcceded: TGG_Personnage;
    FCompacteOnBackup: boolean;
    function GetCount: integer;
    function GetItems(Indice: integer): TGG_Personnage;
    procedure SetItems(Indice: integer; const Value: TGG_Personnage);
  protected
    procedure ClearAndUpdateList; // Efface et remet  jour la liste d'index

  public
    constructor Create(AFilename: string);
    destructor Destroy; override;

    function BackupFile(AFilename: string): cardinal;
    function compacte: cardinal; // Efface les record marqu comme tel.

    function GetRecord(Name: string): TGG_Personnage; overload;
    function GetRecord(const ID: Cardinal): TGG_Personnage; overload; //ID calculer a partir de la chaine de caracter du nom du perso
    function GetRecord(const ID: Cardinal; var GG_Personnage: TGG_Personnage): boolean; overload;
    function GetRecord(const Name: string; var GG_Personnage: TGG_Personnage): boolean; overload;
    function RecordExist(const ID: Cardinal): boolean; overload;
    function RecordExist(ByName: string): boolean; overload;

    function Add(GG_Personnage: TGG_Personnage): integer; overload;
    function Add(Personnage: TPersonnage): integer; overload;
    procedure Delete(const PositionInFile: integer); overload; // Delete et erase n'efface pas rellement le record
    procedure Delete(Name: string); overload; // Delete et erase n'efface pas rellement le record
    function Erase(const Id: cardinal): integer; // ils marquent le nom comme='' et le n Id comme = 0

    function writeRecord(const PosInfile: integer;
      const GG_Personnage: TGG_Personnage): integer; overload;
    function writeRecord(const GG_Personnage: TGG_Personnage): integer; overload;
    function writeRecord(Name: string; Personnage: TPersonnage): integer; overload;
    function writeRecord(Personnage: TPersonnage): integer; overload;

    property Filename: string read FFilename;
    property LastAcceded: TGG_Personnage read FLastRecordAcceded;
    property Count: integer read GetCount;
    property Items[Indice: integer]: TGG_Personnage read GetItems write setItems;
    property CompacteOnBackup: boolean read FCompacteOnBackup write FCompacteOnBackup;
  end;



implementation
uses Sysutils, Globals, StrStuff;



{ TManagerFile }

function TManagerFile.Add(GG_Personnage: TGG_Personnage): integer;
var IndexPersos: PIndexPersos;
begin
  result := -1;
  if trim(GG_Personnage.Personnage.CaractAdd.Name) = '' then exit
  else begin
    try
      Seek(f, FileSize(f));
      result := Filepos(f);
      with GG_Personnage do begin
        PositionInFile := result;
        Id := GetIdFromName(Personnage.CaractAdd.Name);
        DateCreate := Now;
        DateConnect := DateCreate;
      end;
      write(F, GG_Personnage);
      //Ajouter un element  la liste des index
      New(IndexPersos);
      IndexPersos^.PositionInFile := result;
      IndexPersos^.Id := GG_Personnage.Id;
      IndexPersos^.Trie := Uppercase(GG_Personnage.Personnage.CaractAdd.Name);
      Flist_GF_Index.Add(IndexPersos);
    except
      on EInOutError do result := -2;
    end;
  end;
end;

function TManagerFile.Add(Personnage: TPersonnage): integer;
var GG_Personnage: TGG_Personnage;
begin
  GG_Personnage.Personnage := Personnage;
  result := add(GG_Personnage);
end;

function TManagerFile.BackupFile(AFilename: string): cardinal;
var F_Backup: file of TGG_Personnage;
begin
  AssignFile(F_Backup, AFileName);
  try
    result := 0; //renvoie le nombre de record qui on t effacs.
    rewrite(F_Backup);
    Seek(f, 0);
    Seek(F_Backup, 0); //Pas besoin de a en principe
    while not eof(F) do begin
      read(F, FLastRecordAcceded);
      if FCompacteOnBackup then begin
        if (FLastRecordAcceded.Id = 0) or (FLastRecordAcceded.PositionInFile = -1) or (FLastRecordAcceded.Personnage.CaractAdd.Name = '') then
          inc(result)
        else Write(F_Backup, FLastRecordAcceded);
      end else Write(F_Backup, FLastRecordAcceded);
    end;
  finally
    CloseFile(F_Backup);
  end;
end;

procedure TManagerFile.ClearAndUpdateList;
var i: integer;
  IndexPersos: PIndexPersos;
  TruePosInFile: integer;
begin
  for i := Flist_GF_Index.count - 1 downto 0 do Dispose(Flist_GF_Index[i]);
  Flist_GF_Index.clear;
  Flist_GF_Index.capacity := FileSize(F) div Sizeof(TGG_Personnage) + 10; //+10 c'est juste pour se donner un peu de rab. Absolument pas necessaire
  Seek(F, 0); //On se met en debut de fichier
  TruePosInFile := 0;
  while not eof(f) do begin
    Read(F, FLastRecordAcceded);
    if FLastRecordAcceded.Id <> 0 then begin
      New(IndexPersos);
      IndexPersos^.PositionInFile := TruePosInFile;
      IndexPersos^.Id := GetIdFromName(FLastRecordAcceded.Personnage.CaractAdd.Name);
      IndexPersos^.Trie := FLastRecordAcceded.Personnage.CaractAdd.Name;
      Flist_GF_Index.Add(IndexPersos);
      TruePosInFile := FilePos(f);
    end;
  end;
end;

function TManagerFile.compacte: cardinal;
var SaveCompacteOnBackup: boolean;
begin
  if FileSize(F) > 0 then begin //Chargement dans la liste de tout les persos
    SaveCompacteOnBackup := FCompacteOnBackup;
    try
      result := BackupFile(ChangeFileExt(FFilename, '.~tm'));
      Closefile(F);
      DeleteFile(FFilename);
      RenameFile(ChangeFileExt(FFilename, '.~tm'), FFilename);
      AssignFile(F, FFileName);
      reset(F);
      ClearAndUpdateList;
    finally
      FCompacteOnBackup := SaveCompacteOnBackup;
    end;
  end else Result := 0;
end;

constructor TManagerFile.Create(AFilename: string);
begin
  inherited create;
  FCompacteOnBackup := true; //par defaut
  Flist_GF_Index := Tlist.Create;
  FFilename := AFilename;
  AssignFile(F, FFileName);
  if fileExists(FFilename) then reset(F) else rewrite(f);
  if FileSize(F) > 0 then ClearAndUpdateList; //Chargement dans la liste de tout les persos
end;

procedure TManagerFile.Delete(const PositionInFile: integer);
begin
  Seek(f, PositionInFile);
  read(F, FLastRecordAcceded);
  FLastRecordAcceded.Id := 0;
  FLastRecordAcceded.PositionInFile := -1;
  FLastRecordAcceded.Personnage.CaractAdd.Name := '';
  Seek(f, PositionInFile);
  Write(F, FLastRecordAcceded);
end;

procedure TManagerFile.Delete(Name: string);
var i: integer;
  Ident: cardinal;
begin
  Ident := GetIdFromName(Name);
  for i := 0 to Flist_GF_Index.count - 1 do
    if Ident = PIndexPersos(Flist_GF_Index[i])^.Id then
      Delete(PIndexPersos(Flist_GF_Index[i])^.PositionInFile);
end;

destructor TManagerFile.Destroy;
var i: integer;
begin
  for i := Flist_GF_Index.count - 1 downto 0 do Dispose(Flist_GF_Index[i]);
  Flist_GF_Index.free;
  CloseFile(F);
  inherited;
end;


function TManagerFile.Erase(const Id: Cardinal): integer;
var i: integer;
begin
  result := -1;
  for i := 0 to Flist_GF_Index.count - 1 do
    if Id = PIndexPersos(Flist_GF_Index[i])^.Id then begin
      Delete(PIndexPersos(Flist_GF_Index[i])^.PositionInFile);
      result := PIndexPersos(Flist_GF_Index[i])^.PositionInFile;
      exit;
    end;
end;

function TManagerFile.RecordExist(const ID: Cardinal): boolean;
var i: integer;
begin
  result := false;
  for i := 0 to Flist_GF_Index.count - 1 do
    if Id = PIndexPersos(Flist_GF_Index[i])^.Id then begin
      result := true;
      exit;
    end;
end;

function TManagerFile.RecordExist(ByName: string): boolean;
begin
  result := RecordExist(GetIdFromName(ByName));
end;

function TManagerFile.GetCount: integer;
begin
  //Nombre de personnage dans le fichier
  result := Flist_GF_Index.Count;
end;

function TManagerFile.GetItems(Indice: integer): TGG_Personnage;
begin
  if (indice > Flist_GF_Index.Count - 1) or (indice < 0) then exit;
  Seek(f, PIndexPersos(Flist_GF_Index[indice])^.PositionInFile);
  read(F, FLastRecordAcceded);
  FLastRecordAcceded.positionInFile := indice;
  result := FLastRecordAcceded;
end;

function TManagerFile.GetRecord(const ID: Cardinal): TGG_Personnage;
var i: integer;
begin
  for i := 0 to Flist_GF_Index.count - 1 do begin
    if Id = PIndexPersos(Flist_GF_Index[i])^.Id then begin
      Seek(f, PIndexPersos(Flist_GF_Index[i])^.PositionInFile);
      read(F, FLastRecordAcceded);
      //FLastRecordAcceded.PositionInFile:= i;
      result := FLastRecordAcceded;
      exit;
    end;
  end;
  //Si on arrive ici c'est que le Record n'existe pas
  result.personnage.CaractAdd.Name := '';
  result.Id := 0;
end;

function TManagerFile.GetRecord(Name: string): TGG_Personnage;
var crcName: integer;
begin
  CrcName := GetIdFromName(Name);
  result := GetRecord(CrcName);
end;

function TManagerFile.GetRecord(const ID: Cardinal; var GG_Personnage: TGG_Personnage): boolean;
begin
  GG_Personnage := GetRecord(ID);
  result := GG_Personnage.Id <> 0;
end;

function TManagerFile.GetRecord(const Name: string;
  var GG_Personnage: TGG_Personnage): boolean;
begin
  GG_Personnage := GetRecord(GetIdFromName(Name));
  result := GG_Personnage.Id <> 0;
end;

procedure TManagerFile.setItems(Indice: integer;
  const Value: TGG_Personnage);
begin
  if (indice > Flist_GF_Index.Count - 1) or (indice < 0) then exit;
  Seek(f, PIndexPersos(Flist_GF_Index[indice])^.PositionInFile);
  write(F, Value);
end;

function TManagerFile.writeRecord(const PosInfile: integer;
  const GG_Personnage: TGG_Personnage): integer;
begin
  result := -1;
  if (PosInfile > (FileSize(F) - 1)) or (PosInfile < 0) then
    exit;
  Seek(f, PosInfile);
  write(F, GG_Personnage);
  result := PosInfile;
end;

function TManagerFile.writeRecord(
  const GG_Personnage: TGG_Personnage): integer;
begin
  result := writeRecord(GG_Personnage.PositionInFile, GG_Personnage);
end;

function TManagerFile.writeRecord(Name: string;
  Personnage: TPersonnage): integer;
var GG_Personnage: TGG_Personnage;
begin
  GG_Personnage := GetRecord(Name);
  GG_Personnage.Personnage := Personnage;
  result := writeRecord(GG_Personnage);
end;

function TManagerFile.writeRecord(Personnage: TPersonnage): integer;
begin
  result := writeRecord(Personnage.CaractAdd.name, Personnage);
end;

end.
