unit Stuff;

interface
uses windows, classes;

(*
var VideoModes        : array[0..MaxVideoModes] of TVideoMode;
    NumberVideomodes  : Integer = 1; // 1 because we have a default mode

{ To Give FullScreenMode }
const MaxVideoModes = 200; // this isn't very much actually
  TVideoMode = record
    Width,
    Height,
    ColorDepth  : Word;
    Description : String[20];
  end;
*)

Function card (var tar; size: integer): integer; assembler;

function Dist2d(const pta, ptb: TPoint): integer;


implementation
uses Sysutils;



{$ifdef FULLSCREEN}
{ Give the possibility to have FullscreenMode }
function SetFullscreenMode(ModeIndex: Integer) : Boolean;
// changes to the video mode given by 'ModeIndex'
var DeviceMode : TDevMode;
begin
  with DeviceMode do
  begin
    dmSize:=SizeOf(DeviceMode);
    dmBitsPerPel:=VideoModes[ModeIndex].ColorDepth;
    dmPelsWidth:=VideoModes[ModeIndex].Width;
    dmPelsHeight:=VideoModes[ModeIndex].Height;
    dmFields:=DM_BITSPERPEL or DM_PELSWIDTH or DM_PELSHEIGHT;
    // if mode set failed, we'll just run in windowed mode
    Result:=ChangeDisplaySettings(DeviceMode,CDS_FULLSCREEN) = DISP_CHANGE_SUCCESSFUL;
    if Result then ScreenModeChanged:=True;
    if ModeIndex = 0 then ScreenModeChanged:=False;
  end;
end;

procedure TryToAddToList(DeviceMode: TDevMode);
// Adds a video mode to the list if it's not a duplicate and can actually be set.
var I : Integer;
begin
  // See if this is a duplicate mode (can happen because of refresh
  // rates, or because we explicitly try all the low-res modes)
  for I:=1 to NumberVideomodes-1 do
    with DeviceMode do
      if ((dmBitsPerPel = VideoModes[I].ColorDepth) and
          (dmPelsWidth  = VideoModes[I].Width)      and
          (dmPelsHeight = VideoModes[I].Height))    then Exit; // it's a duplicate mode

  // do a mode set test (doesn't actually do the mode set, but reports whether it would have succeeded).
  if ChangeDisplaySettings(DeviceMode,CDS_TEST or CDS_FULLSCREEN) <> DISP_CHANGE_SUCCESSFUL then Exit;

  // it's a new, valid mode, so add this to the list
  with DeviceMode do
  begin
    VideoModes[NumberVideomodes].ColorDepth:=dmBitsPerPel;
    VideoModes[NumberVideomodes].Width:=dmPelsWidth;
    VideoModes[NumberVideomodes].Height:=dmPelsHeight;
    VideoModes[NumberVideomodes].Description:=Format('%d x %d, %d bpp',[dmPelsWidth,dmPelsHeight,dmBitsPerPel]);
  end;
  Inc(NumberVideomodes);
end;

//This makes our example much larger as you will see, but you'll get a
//  very useful functionality by implementing it.
//If you still want you can replace VideoModes in the above function by fixed
//  values (say 640, 480, 16).
//Enumerating all video modes is done by EnumDisplaySettings:
procedure ReadVideoModes;
var I, ModeNumber : Integer;
    done          : Boolean;
    DeviceMode    : TDevMode;
    DeskDC        : HDC;

begin
  // prepare 'default' entry
  with VideoModes[0] do
  try
    DeskDC:=GetDC(0);
    ColorDepth:=GetDeviceCaps(DeskDC,BITSPIXEL);
    Width:=Screen.Width;
    Height:=Screen.Height;
    Description:='default';
  finally
    ReleaseDC(0,DeskDC);
  end;

  // enumerate all available video modes
  ModeNumber:=0;
  done:=False;
  repeat
    done:=not EnumDisplaySettings(nil,ModeNumber,DeviceMode);
    TryToAddToList(DeviceMode);
    Inc(ModeNumber);
  until (done or (NumberVideomodes >= MaxVideoModes));

  // low-res modes don't always enumerate, ask about them explicitly
  with DeviceMode do
  begin
    dmBitsPerPel:=8;
    dmPelsWidth:=42;
    dmPelsHeight:=37;
    dmFields:=DM_BITSPERPEL or DM_PELSWIDTH or DM_PELSHEIGHT;
    // make sure the driver doesn't just answer yes to all tests
    if ChangeDisplaySettings(DeviceMode,CDS_TEST or CDS_FULLSCREEN) <> DISP_CHANGE_SUCCESSFUL then
    begin
      I:=0;
      while (I < NumberLowResModes-1) and (NumberVideoModes < MaxVideoModes) do
      begin
        dmSize:=Sizeof(DeviceMode);
        dmBitsPerPel:=LowResModes[I].ColorDepth;
        dmPelsWidth:=LowResModes[I].Width;
        dmPelsHeight:=LowResModes[I].Height;
        dmFields:=DM_BITSPERPEL or DM_PELSWIDTH or DM_PELSHEIGHT;
        TryToAddToList(DeviceMode);
        Inc(I);
      end;
    end;
  end;
end;

procedure RestoreDefaultMode;
// restores default desktop video mode
var T : TDevMode absolute 0; // a little trick to create a nil pointer
begin
  // Since the first parameter must be a var, we cannot use nil directly. Instead
  //  we use a variable with an absolute address of 0.
  ChangeDisplaySettings(T,CDS_FULLSCREEN);
end;

{$endif}


FUNCTION card (var tar; size: integer): integer; assembler;
    (*PRE size <= sizeof(tar)
    POST returns number of on (1) bits in first size bytes of tar*)
    (*Reg usage: eax=result
                 ebx=work reg
                 ecx=nibble counter
                 edx=@last byte in tar, +1
                 esi=@next elm in tar
                 edi=@last dw in tar, +1
                 ebp=0 (to shorten opcodes)*)
    (*NB the main engine is countNibble. The setup gets tar into esi and
    sets two limits for it: full dw's, and bytes (i.e. full length).
    CountNibble is fed from dw's until the first limit is reached, then from
    the remaining bytes*)
ASM
(*setup*)
    push ebx
    push esi
    push edi
    push ebp
    mov  esi, eax       //file://set esi
    mov  ebx, edx
    and  ebx, 3
    add  edx, eax       //file://set edx
    mov  edi, edx
    sub  edi, ebx       //file://set edi
    xor  eax, eax       //file://set eax
    xor  ebp, ebp       //file://set ebp

@doDws:
    cmp  esi, edi
    jnb  @doBytes
    mov  ebx, [esi]
    add  esi, 4
    mov  ecx, 8         //file://eight nibbles per dw
@countNibble:           //file://repeats save three loop's
    shr  ebx, 1         //file://adding four more would only save one loop
    adc  eax, ebp
    shr  ebx, 1
    adc  eax, ebp
    shr  ebx, 1
    adc  eax, ebp
    shr  ebx, 1
    adc  eax, ebp
    loop @countNibble
    jmp  @doDws

@doBytes:
    cmp  esi, edx
    jz   @exit
    mov  bl, [esi]
    inc  esi
    mov  ecx, 2         //file://two nibbles
    jmp  @countNibble

@exit:
    pop  ebp
    pop  edi
    pop  esi
    pop  ebx
END;

var LocalX, LocalY: integer; //Pour accellerer

function Dist2d(const pta, ptb: TPoint): integer;
begin
  LocalX:= pta.X - ptb.X;
  LocalY:= pta.Y - ptb.Y;
  result:= trunc(Sqrt(LocalX * LocalX + LocalY * LocalY));
end;

end.
