{-----------------------------------------------------------------------------
 Unit Name: UDataClient
 Author   : Manuel
 Important: Cette unit doit tre la premiere fiche cr automatiquement
                dans votre projet.
                Voir menu Projet/Options et onglet Fiches
 Purpose  : Gestion des composants TADKProvide, TADKClient, TPersosComplet
                et TminiPersos pour fournir une interface facilement et
                rapidement exploitable pour la connexion au serveur ADK.

 History  : - 17 Janvier 03: Ajout de MapsPathFind pour le stockage du map
                ou se place notre personnage. Mme si le map est dans un
                Tlist et que l'on peut tres bien le dfinir tout seul sous
                la forme d'un TMapFinder, je prfre utiliser le Tcomponent
                TMapsPathFind qui permettra la gestion de plusieurs Maps
                quand il faudra rutiliser ce DataClient pour la gestion
                de plusieurs centaines de PNJ qui ne seront pas forcement
                sur la mme carte.
            - 11 Janvier 03: Ajout de la gestion de la commande scString21
                qui permet l'envoie de texte brut vers un perso bien prcis
            - 10 Janvier 03: Ajout de la gestion de la commande scText21 qui
                permet l'envoi d'un texte  une personne bien prcise
            - 09 Janvier 03: Mise en place du Cleint en version 1.0.0.7
-----------------------------------------------------------------------------}


unit UDataClient;

interface

uses
  SysUtils, Classes, ADKWSA, ADKClient,
  ADKNetData, ADKServer, PathMap, ObjetsActif, Personnage, Definitions;

type

  TOnServerInfo = procedure(Sender: TObject; const Value: TServerInfo) of object;
  TOnNameExist = procedure(Sender: TObject; const exist: boolean;
    const NamePerso: string) of object;
  TOnStringBrut = procedure(Sender: TObject; const ID: Cardinal; const Cmd: TServCmd;
    const StringBrut: string) of object;
  TOnTextAtribut = procedure(Sender: TObject; const Attr: TTextAttr; Text: string) of object;
  TOnTextAtributAndID = procedure(Sender: TObject; const ID: Cardinal;
    const Attr: TTextAttr; Text: string) of object;
  TOnCmdTextAtributAndID = procedure(Sender: TObject; const ID: Cardinal;
    const cmd: TServCmd; const Attr: TTextAttr; Text: string) of object;
  TOnTextAtribut21 = procedure(Sender: TObject; IDSender, IDDest: Cardinal;
    const Attr: TTextAttr; Text: string) of object;
  TOnStringBrut21 = procedure(Sender: TObject; IDSender, IDDest: Cardinal; Text: string) of object;
  TPositionChange = procedure(Sender: TObject; const ID: Cardinal;
    const Cmd: TServCmd; const Value: TPositionInfo) of object;
  TStopPosition = procedure(Sender: TObject; const ID: Cardinal) of object;
  TOnSpecifiqueCardinal = procedure(Sender: TObject; const ID: Cardinal; Value: cardinal) of object;
  TOnUpdatePNJPJ = procedure(Sender: TObject; const IDSender: Cardinal; const Cmd: TServCmd; const Value: TStream);


  TDataClientNet = class(TDataModule)
    ADKClient1: TADKClient;
    Provide: TADKProvideWSA;
    PersosComplets1: TPersosComplets;
    MapsPathFind1: TMapsPathFind;
    CompoActifItem1: TCompoActifItem;
    procedure ProvideConnect(Sender: TObject);
    procedure ProvideClose(Sender: TObject);
    procedure ProvideError(Sender: TObject; Msg: string);
    procedure ADKClient1Negociate(Sender: TObject;
      const Value: TNegociate);
    procedure ADKClient1ServerInfo(Sender: TObject;
      const Value: TServerInfo);
    procedure ADKClient1StartGame(Sender: TObject);
    procedure ADKClient1Message(Sender: TObject; const ID: Cardinal;
      const Cmd: TServCmd; Text: string; const Attr: TTextAttr);
    procedure ADKClient1LoadMap(Sender: TObject; Map: string);
    procedure ProvideRead(Sender: TObject; Count: Integer);
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
    procedure ADKClient1EndGame(Sender: TObject; const Reason: string);
    procedure ADKClient1String(Sender: TObject; const ID: Cardinal;
      Cmd: TServCmd; Text: string);
    procedure PersosComplets1Delete(Sender: TObject; Value: TCustomPerso);
    procedure ADKClient1Cardinal(Sender: TObject; const ID: Cardinal;
      const Cmd: TServCmd; Value: Cardinal);
    procedure ADKClient1Message21(Sender: TObject; const ID: Cardinal;
      const Cmd: TServCmd; const IdDestinataire: Cardinal; Text: string;
      const Attr: TTextAttr);
    procedure ADKClient1String21(Sender: TObject; const ID: Cardinal;
      Cmd: TServCmd; const IdDestinataire: Cardinal; Text: string);
    procedure ADKClient1Position(Sender: TObject; const ID: Cardinal;
      const Cmd: TServCmd; const Value: TPositionInfo);
    procedure ADKClient1StopMove(Sender: TObject; const ID: Cardinal;
      const Cmd: TServCmd);
    procedure ADKClient1ActifItem(Sender: TObject; const ID: Cardinal;
      const Cmd: TServCmd; const Value: TStream);
    procedure CompoActifItem1Add(Sender: TObject;
      ActifItem: TBaseActifItem);
    procedure CompoActifItem1Delete(Sender: TObject;
      ActifItem: TBaseActifItem);
    procedure ADKClient1PlayerInfo(Sender: TObject; const ID: Cardinal;
      const Cmd: TServCmd; const Value: TStream);
  private
    { Dclarations prives }
    FMemoClient, FMemoServer: Tstrings;
    FOctetsRecus: Cardinal;
    fConnected: boolean;
    fIamANumber: string;
    fCurrentUpdate: integer; // n d'update en cour sur le client
    fValueNegociate: TNegociate; // Donnes reu du serveur aprs le NEGOCIATE;
    fServerInfo: TServerInfo; // Donns "spcial" (Heure dmarrage serveur, Heure ADk, Zone influence etc)
    fSynchroniseTime: Integer; //permet de savoir quand on a recu le Time du serveur et de s'y synchroniser

    EOnServerInfo: TOnServerInfo;
    EOnConnected, EOnCloseConnect, EOnLogedPerso,
      EOnPersoDoNotExist: Tnotifyevent;
    EOnNameExist, EOnPersoWrited: TOnNameExist;
    EOnStringBrut: TOnStringBrut;
    EOnTextAtribut: TOnTextAtribut;
    EOnTextAtributAndID: TOnTextAtributAndID;
    EOnCmdTextAtributAndID: TOnCmdTextAtributAndID;
    EOnActifItemIn, EOnActifItemOut,
      EOnMinipersoIn, EOnMinipersoOut: TonNotifyActifItem;
    EOnStringBrut21: TOnStringBrut21;
    EOnTextAtribut21: TOnTextAtribut21;
    EOnPositionChange: TPositionChange;
    EOnStopPosition: TStopPosition;
    ERotatePosition, EOnTotalInscrit: TOnSpecifiqueCardinal;

    procedure ReadInifile;
    procedure WriteInifile;
    procedure ActifItemMoveto(const ID: Cardinal; const Value: TPositionInfo);

    function GetMemoClient: Tstrings;
    function GetMemoServer: Tstrings;
    procedure SetMemoClient(const Value: Tstrings); // Tstrings des messages entre personnages
    procedure SetMemoServer(const Value: Tstrings); // Tstrings des messages en provenance du serveur
    function GetLastLoggedPerso: TCustomPerso;
    function GetActivePerso: TCustomPerso;
  public
    { Dclarations publiques }
    // Dfinition de l'Ip et du n de port de connexion
    procedure SetLocalHost; // Place le client en mode connexion locale.
    procedure SetHost(const Adr: string; const Port: word); // Fournir au cliet l'Ip et le n de port de connexion
    // Dfinition d'un affichage basic
    procedure DisplayMessage(const StrMessage: string); // Affiche les messages entre Clients
    procedure DisplayServer(const StrMessage: string); // Affiche les messages du serveur
    procedure DisplayError(const StrMessage: string); // Affiche les messages d'erreur (dans le Tstrings du serveur)

    // * * * * * * * * * Procedures de "confort" * * * * * * * * *
    // Ouvrir la session avec serveur.
    // StrDcodeur est un string permettant de dcoder le fichier que recupere la procedure.
    // Ce fichier contient l'Ip, le n de port et des donnes
    // Additionnelles permettant la connexion au serveur
    procedure CallServer(StrDecodeur: string);
    // Fermer la session avec le serveur
    procedure CloseSession;
    //Deconnecter un perso sans fermer la session en cour "in Many Flavours"
    //*Utile pour le Manager de PNJ*
    procedure DisconnectOne(const DeconnecteurID, AdeconnecterID: Cardinal); overload;
    procedure DisconnectOne(const DeconnecteurID: Cardinal; const NamePerso: string); overload;
    procedure DisconnectOne(const NamePerso: string); overload;
    procedure DisconnectOne(const ID: Cardinal); overload;
    // Enregistre "Personnage" sur le serveur. Ident n' pas d'importance dans cette proc.
    procedure CreatePersoOnServer(const Ident: Cardinal; PersoStream: TStream);
    // Log un perso existant. NAME= Nom du personnage.  Password= Son mot de passe
    procedure LogOn(const Name, Password: string);
    // Demande au serveur si le nom "PersoName" existe sur le serveur
    // La rponse est a "recuprer" grace  l'venement *property OnNameExist*
    procedure NameExist(const Ident: Cardinal; const PersoName: string);
    // Envoie un Ping interne (Just for test)
    procedure Ping(const Ident: Cardinal);

    // Property's
    property IamANumber: string read fIamANumber write fIamANumber; //Utilisation interne
    // LastPerso donne acces au dernier personage connect
    // On peut pass par DataClientNet.PersosComplets1.ListePerso.Items[Count-1]
    // cette property est la pour simplifi l'appel, tout comme la  property
    // PERSO est la pour simplifier l'appel au perso actif (celui que l'on dirige en principe)
    // ATTENTION:
    // LastPerso est le dernier personnage connect et non pas le personnage actif
    // On peut changer le perso actif avec DataClientNet.PersosComplets1.ActivePersoByName
    // ou DataClientNet.PersosComplets1.ActivePersoByID
    // et on accede au perso actif avec la property PERSO (un peu plus bas)
    // ou par: DataClientNet.PersosComplets1.PersoActif
    // Si la liste des perso est vide LastPerso et PERSO renvoient NIL.
    // Au premier ajout d'un personnage celui-ci devient le LASTPERSO et
    // le perso actif (PERSO) par dfaut.
    // Il est de la responsabilit du programmeur de modifi
    // PERSO (avec DataClientNet.PersosComplets1.ActivePersoByName) ou
    // DataClientNet.PersosComplets1.ActivePersoByID avant de l'utilis.
    // (dans le cas ou le perso actif en cours n'est pas celui  utiliser);
    // *Utile pour le Manager de PNJ*
    property LastPerso: TCustomPerso read GetLastLoggedPerso; //renvoie le DERNIER perso logguer
    property Perso: TCustomPerso read GetActivePerso; //renvoie le perso ACTIF (pas indispensable juste pour simplifier l'acces au personnage actif)

    property MemoClient: Tstrings read GetMemoClient write SetMemoClient; // Property pour acceder  l'affichage basic du texte
    property MemoServer: Tstrings read GetMemoServer write SetMemoServer;
    property OctetsRecus: Cardinal read FOctetsRecus; // Total des octets reu depuis la connexion
    property Connected: boolean read fConnected; //True = On est connect... normal non ?
    property ValueNegociate: TNegociate read fValueNegociate; //Donnes reues aprs le NEGOCIATE avec le serveur
    property ValueServerInfo: TServerInfo read fServerInfo; // Donnes "spcial" du serveur (Heur de marrage serveur, Heure Adk, Zone Influence etc)
    property SynchroniseTime: integer read fSynchroniseTime write fSynchroniseTime;
    //Events
    property OnServerInfo: TOnServerInfo read EOnServerInfo write EOnServerInfo;
    property OnPersoDoNotExist: Tnotifyevent read EOnPersoDoNotExist write EOnPersoDoNotExist;
    property OnConnected: Tnotifyevent read EOnConnected write EOnConnected;
    property OnCloseConnect: Tnotifyevent read EOnCloseConnect write EOnCloseConnect;
    property OnLogedPerso: Tnotifyevent read EOnLogedPerso write EOnLogedPerso;
    property OnNameExist: TOnNameExist read EOnNameExist write EOnNameExist;
    property OnStringBrut: TOnStringBrut read EOnStringBrut write EOnStringBrut;
    property OnPersoWrited: TOnNameExist read EOnPersoWrited write EOnPersoWrited;
    property OnTextAtribut: TOnTextAtribut read EOnTextAtribut write EOnTextAtribut;
    property OnTextAtributAndID: TOnTextAtributAndID read EOnTextAtributAndID write EOnTextAtributAndID;
    property OnCmdTextAtributAndID: TOnCmdTextAtributAndID read EOnCmdTextAtributAndID write EOnCmdTextAtributAndID;
    property OnActifItemIn: TonNotifyActifItem read EOnActifItemIn write EOnActifItemIn;
    property OnActifItemOut: TonNotifyActifItem read EOnActifItemOut write EOnActifItemOut;
    property OnMinipersoIn: TonNotifyActifItem read EOnMinipersoIn write EOnMinipersoIn;
    property OnStringBrut21: TOnStringBrut21 read EOnStringBrut21 write EOnStringBrut21;
    property OnTextAtribut21: TOnTextAtribut21 read EOnTextAtribut21 write EOnTextAtribut21;
    property OnPositionChange: TPositionChange read EOnPositionChange write EOnPositionChange;
    property OnStopPosition: TStopPosition read EOnStopPosition write EOnStopPosition;
    property OnRotatePosition: TOnSpecifiqueCardinal read ERotatePosition write ERotatePosition;
    property OnTotalInscrit: TOnSpecifiqueCardinal read EOnTotalInscrit write EOnTotalInscrit;
  end;

var
  DataClientNet: TDataClientNet;

implementation
uses windows, HttpSock, Inifiles, Stuff, StrStuff, Globals,
  resStr, forms, Dialogs;
{$R *.dfm}

const IniSectionOption = 'options';

procedure TDataClientNet.ReadInifile;
var Inifile: Tinifile;
begin
  Inifile:= Tinifile.create(ExtractFilePath(Application.exename) + 'client.ini');
  try
    fCurrentUpdate:= Inifile.ReadInteger(IniSectionOption, 'CurrentUpdate', 1);
  finally
    Inifile.destroy;
  end;
end;

procedure TDataClientNet.WriteInifile;
var Inifile: Tinifile;
begin
  Inifile:= Tinifile.create(ExtractFilePath(Application.exename) + 'server.ini');
  try
  finally
    Inifile.destroy;
  end;
end;

//Dfinition d'une interface basique d'afrfichage des messages  affich

function TDataClientNet.GetMemoClient: Tstrings;
begin
  result:= FMemoClient;
end;

function TDataClientNet.GetMemoServer: Tstrings;
begin
  result:= FMemoServer;
end;

procedure TDataClientNet.SetMemoClient(const Value: Tstrings);
begin
  FMemoClient:= Value;
end;

procedure TDataClientNet.SetMemoServer(const Value: Tstrings);
begin
  FMemoServer:= Value;
end;

procedure TDataClientNet.DisplayMessage(const StrMessage: string);
begin
  //Affiche les messages entre Clients
  if assigned(FMemoClient) then
    with FMemoClient do begin
      BeginUpdate;
      Add(StrMessage);
      while Count > 200 do Delete(0);
      EndUpdate;
    end;
end;

procedure TDataClientNet.DisplayServer(const StrMessage: string);
begin
  //Affiche les messages en provenance du serveur
  if assigned(FMemoServer) then
    with FMemoServer do begin
      BeginUpdate;
      Add(StrMessage);
      while Count > 50 do Delete(0);
      EndUpdate;
    end;
end;

procedure TDataClientNet.DisplayError(const StrMessage: string);
begin
  // Affiche les messages d'erreur (dans le Tstrings du serveur)
  DisplayServer(StrMessage);
end;

// Procedure pour les property d'acces aux personnages

function TDataClientNet.GetLastLoggedPerso: TCustomPerso;
begin
  result:= nil;
  with PersosComplets1 do
    if ListPerso.Count > 0 then result:= ListPerso[ListPerso.count - 1];
end;

function TDataClientNet.GetActivePerso: TCustomPerso; // Pour la property PERSO
begin
  result:= PersosComplets1.PersoActif;
end;

(*
function TDataClientNet.GetRecordPerso: TPersonnage; // Pour la property RPerso
begin
  if assigned(PersosComplets1.PersoActif) then
    result := PersosComplets1.PersoActif.RecordPerso
  else raise Exception.create('Aucun perso actif');
end;
*)

// Dfinition des IP et n de Port de connexion

procedure TDataClientNet.SetLocalHost;
begin
  Provide.Port:= 2010;
  Provide.server:= '127.0.0.1';
end;

procedure TDataClientNet.SetHost(const Adr: string; const Port: word);
begin
  Provide.server:= Adr;
  Provide.Port:= port;
end;


// Metodes du Datamodules et des composants pos dessus

procedure TDataClientNet.DataModuleCreate(Sender: TObject);
begin
  ReadInifile;
  fServerInfo.Influence:= 12; // Juste pour avoir des valeurs par dfaut en cas de problme
  fServerInfo.ZoneTampon:= 7; // Idem
  MapsPathFind1.Maplist.OwnsObjects:= false; //<< Indispensable
end;

procedure TDataClientNet.DataModuleDestroy(Sender: TObject);
begin
  writeInifile;
end;


procedure TDataClientNet.ProvideConnect(Sender: TObject);
begin
  DisplayServer(rsClientConected);
  fConnected:= true;
 // if assigned(EOnConnected) then EOnConnected(self); << Pas ici. Voir dans la reception du NEGOCIATE
 //On renvoie tout de suite une demande de ngociation
{$IFDEF Sleep50}
  sleep(50);
{$ENDIF}
  //fIamANumber=  -1 c'est que c'est un "serveur" de Pnj qui se connecte
  // Le 5 est n d'update de la version client.
  // Je l'ecrit en DUR car ce n doit changer suivant les versions fournit
  // et ne doit pas tre modifier via un .ini ou autre
  AdkClient1.SendNegociate(fIamANumber, 6);
end;

procedure TDataClientNet.ProvideClose(Sender: TObject);
begin
  DisplayServer(rsEndConnection);
  fConnected:= false;
  CompoActifItem1.ListItems.Clear;
  if assigned(EOnCloseConnect) then EOnCloseConnect(self);
end;

procedure TDataClientNet.ProvideError(Sender: TObject; Msg: string);
begin
  DisplayError(msg);
end;

procedure TDataClientNet.ADKClient1Negociate(Sender: TObject;
  const Value: TNegociate);
begin
  DisplayServer(format(rsNegociate, [Value.server, Value.update]));
  if Value.Banish then begin
    DisplayError(rsErr879 + rsBreakline + rsCallAdkMaster);
    CloseSession;
    exit;
  end;
  fValueNegociate:= Value;
  //Ici on peut dire que le client (le client, ET PAS forcement un personnage) est vraiment connect. Pas avant.
  if assigned(EOnConnected) then EOnConnected(self);
end;

procedure TDataClientNet.ADKClient1ServerInfo(Sender: TObject;
  const Value: TServerInfo);
begin
  //DisplayServer('Serveur dmarr le ' + DateTimeToStr(Value.StartDate));
  fServerInfo:= Value;
  fSynchroniseTime:= GetTickcount;
  if assigned(EOnServerInfo) then OnServerInfo(self, fServerInfo);
end;

procedure TDataClientNet.ADKClient1StartGame(Sender: TObject);
begin
  DisplayServer('le jeu a commenc');
  if assigned(EOnLogedPerso) then EOnLogedPerso(self);
end;

procedure TDataClientNet.ADKClient1Message(Sender: TObject; const ID: Cardinal;
  const Cmd: TServCmd; Text: string; const Attr: TTextAttr);
begin
  case Cmd of
    scText, scTextRp: begin
        if assigned(EOnCmdTextAtributAndID) then
          EOnCmdTextAtributAndID(Self, ID, cmd, Attr, Text)
        else DisplayMessage(Text);
      end;
  else end;
end;

procedure TDataClientNet.ADKClient1LoadMap(Sender: TObject; Map: string);
begin
  // Le string MAP se compose de deux partie spar par un Point-virgule [;]
  // 'NID-de-la-Carte;Nom-de-la-carte'
  // La premire partie est le n ID de la carte et la deuxime partie est
  // le nom exact de cette carte
  LastPerso.MapNumber:= StrToint(extractword(1, map, [ChrPVirgule]));
  MapsPathFind1.Maplist.AddMap(StrToint(extractword(1, map, [ChrPVirgule])),
    ExtractFilePath(Application.exename) + 'data\cartes\' +
    extractword(2, map, [ChrPVirgule]) +
    '.adk'
    );
  if LastPerso is TCustomPJPNJ then begin
    TCustomPJPNJ(LastPerso).WalkableMap:= MapsPathFind1.Maplist.GetMapByID(LastPerso.MapNumber);
  end;
end;

procedure TDataClientNet.ProvideRead(Sender: TObject; Count: Integer);
begin
  FOctetsRecus:= FOctetsRecus + count; // Cumul des octets reu
end;

// * * * * * * * * * Procedures de "confort" * * * * * * * * *

function GetADKServer(const StrDecodeur: string; out ip: string;
  out port: word; out textAdditionnel: string): boolean;
var
  rq, DefaultIniFileName: string;
  Inifile: Tinifile;
begin
  result:= false;
  //s:= ''; // http header
  Getdir(0, DefaultIniFileName);
  Inifile:= Tinifile.create(DefaultIniFileName + '\path.ini');
  try
    with Inifile do
      rq:= HttpGetLine(readstring('URL', 'SERVEUR', ''), readstring('URL', 'File', ''));
      //rq:= HttpGet(readstring('URL', 'SERVEUR', ''), readstring('URL', 'File', ''), s);
  finally
    Inifile.free;
  end;
  rq:= XorDecode(StrDecodeur, rq);
  if rq = '' then exit;
  Ip:= extractword(2, rq, [ChrPvirgule]);
  if Ip = '' then exit;
  Port:= 0; // Si la valeur n'est pas modifi c'est que c'est rap
  Port:= StrToInt(extractword(3, rq, [ChrPvirgule]));
  if port = 0 then exit;
  textAdditionnel:= extractword(4, rq, [ChrPvirgule]);
  // textAdditionnel peut tre vide ou contenir  peut pres
  // nimporte quoi et spcialement certain ordres  executer
  result:= true;
end;

procedure TDataClientNet.CallServer(StrDecodeur: string);
var StrIp, TxtAdditionnel: string;
  Port: Word;
begin
  //fermer la connexion avant tout.
  //while DataConnect.WSocket1.State <> wsClosed do DataConnect.WSocket1.Close;
  Provide.Close;
  //recup des donnes de connexion
  if GetADKServer(StrDecodeur, StrIp, port, TxtAdditionnel) then begin
    SetHost(StrIp, Port);
    DisplayServer('Adresse IP du serveur trouve. Tentative de connection en cours.');
    Provide.Open; //Connecter le socket
  end else begin
    DisplayError(rsErr9001);
    DisplayError(rsConnectToLocal);
    SetLocalHost;
    Provide.Open; //Connecter le socket en mode local
  end;
end;

procedure TDataClientNet.CloseSession;
begin
  Provide.close;
  ProvideClose(Self);
end;

// deconnecter le perso avec l'ID "AdeconencterID" sans fermer la session
// "DeconnecteurID" est l'ID de celui qui fait la demande de dconnection.
// Ce sera 0 (zero) si c'est le serveur qui execute la commande
// Ce sera 1 (un) si c'est un programme auto de gestion de PNJ
// Sinon ce sera l'ID relle du perso s'il s'agit d'un PJ autoriser
// a dconnecter d'autre persos.

procedure TDataClientNet.DisconnectOne(const DeconnecteurID, AdeconnecterID: Cardinal);
begin
  DataClientNet.ADKClient1.DisconnectOne(DeconnecteurID, AdeconnecterID);
end;

// deconnecter le perso 'NamePerso' sans fermer la session
// le 'deconnecteur' envoie son ID dans "DeconnecteurID"

procedure TDataClientNet.DisconnectOne(const DeconnecteurID: Cardinal; const NamePerso: string);
begin
  DisconnectOne(DeconnecteurID, GetIdFromName(NamePerso));
end;

// deconnecter le perso 'NamePerso' sans fermer la session
// deconnecte le perso en balancant son ID

procedure TDataClientNet.DisconnectOne(const NamePerso: string);
begin
  DisconnectOne(GetIdFromName(NamePerso), GetIdFromName(NamePerso));
end;

// deconnecter le perso avec l'ID sans fermer la session
// deconnecte le perso en balancant son ID

procedure TDataClientNet.DisconnectOne(const ID: Cardinal);
begin
  DisconnectOne(ID, ID);
end;

(*
procedure TDataClientNet.CreatePersoOnServer(const Ident: Cardinal; Personnage: TPlayerInfo);
begin
  ADKClient1.SendCreatePerso(Ident, Personnage);
end;
*)

procedure TDataClientNet.CreatePersoOnServer(const Ident: Cardinal; PersoStream: TStream);
begin
  ADKClient1.SendCreatePerso(Ident, PersoStream);
end;

procedure TDataClientNet.LogOn(const Name, Password: string);
begin
  ADKClient1.SendLogon(name, Password);
  // LogOn provoque de la part du serveur (si le perso existe... bien entendu)
  // le renvoie du personnage au complet, suivi de l'envoie
  // du nom de la carte (La ou se trouve notre perso. Par exemple:'Cour exterieure de chessy'
  // Le nom de la carte est envoy sans extension.
  // Ensuite on recoit tout les personnages proche du ntre sous forme
  // de TrecMiniPerso.
  // Pour finir le serveur envoie la commande "scInitFinish"
  // qui signale que l'on peut utiliser le perso et, surtout,  charger le Map
  // et demander sa mise  jour ( Sinon impossible de voir les autres joueurs)
  // Donc la partie Interface Client ne fait rien tant que le la commande
  // scInitFinish n'est pas envoy.
  // On peut surcharger EOnLogedPerso pour savoir quant le message scInitFinish
  // est arriv. Ensuite suffit d'utiliser LastPerso ou Perso pour avoir accs
  // au donnes du perso fraichement arriv (incluant son map).
end;

procedure TDataClientNet.Ping(const Ident: Cardinal);
begin
  // Envoie un ping au serveur
  ADKClient1.Ping(Ident);
end;

procedure TDataClientNet.NameExist(const Ident: Cardinal; const PersoName: string);
begin
  ADKClient1.SendNameExist(Ident, PersoName);
end;


procedure TDataClientNet.ADKClient1EndGame(Sender: TObject;
  const Reason: string);
begin
  //Laisser deux ligne en blanc pour sparer le texte d'erreur des autres
  DisplayError(rsBreakline);
  //afficher la raison de la dconnexion
  DisplayError(reason);
  //et fermer la connexion.
  ProvideClose(self);
  Provide.Close; //Close(self);
end;

procedure TDataClientNet.ADKClient1PlayerInfo(Sender: TObject;
  const ID: Cardinal; const Cmd: TServCmd; const Value: TStream);
var CustomPerso: TPJPerso;
  MiniPerso: TMiniPerso;
begin
  // Rcption des donns completes d'un perso. Il est possible que cela serve
  //  plusieurs commandes (autre que scUpdatePClient) d'ou le "Cmd: TServCmd"
  // dans la liste d'arguments de la procedure pour savoir qu'elle commande
  // nous  fait atterir ici
  case Cmd of
    scUpdatePClient: begin
        CustomPerso:= TPJPerso.create;
        CustomPerso.ReadData(Value);
        PersosComplets1.ListPerso.Add(CustomPerso);
        {#todo1 j'ajoute mon propre perso en tant que MiniPerso
        Je ne suis pas sur que cela soit plus simple. A voir  l'usage}
        MiniPerso:= TMiniPerso.create(CompoActifItem1.ListItems);
        FillTMiniPersoWithTPjPerso(Id, CustomPerso, MiniPerso);
        CompoActifItem1.ListItems.Add(MiniPerso);
      end;
  else end;
  //halt; //<< Ca c'est pour tester avec Memproff
end;

procedure TDataClientNet.ADKClient1String(Sender: TObject;
  const ID: Cardinal; Cmd: TServCmd; Text: string);

  procedure ExecutescDisconnect;
  begin
    PersosComplets1.ListPerso.DeleteByID(Id);
    displayServer(text);
    if pos('Error 1504', Text) <> -1 then
      if assigned(EOnPersoDoNotExist) then EOnPersoDoNotExist(self);
  end;
begin
  // reception d'un string 'simple' (sans attribut de texte)
  // Cet vnement est utilis par plusieurs commmande.
  // Il faut vrifier quelle est la commande qui nous a ammen ici
  case Cmd of
    scDisConnect: ExecutescDisconnect;
    scPersoWrited: if assigned(EOnPersoWrited) then
        EOnPersoWrited(self, StrToBool(extractword(1, Text, [ChrPVirgule])), extractword(2, Text, [ChrPVirgule])); //Texte spar par un point virgule (;)Premier champ=True ou False. Si true le perso a t enregistr. Sinon c'est qu'il y a eu un problme
    scNameExist: if assigned(EOnNameExist) then //retour d'une demande de verif pour savoir si un perso existe sur le serveur
        EOnNameExist(self, StrToBool(extractword(1, Text, [ChrPVirgule])), extractword(2, Text, [ChrPVirgule])); //Texte spar par un point virgule (;)Premier champ=True ou False. Si true le perso existe sur le serveur. 2eme champ= Nom du perso
    scTextServ: DisplayServer(text); //Afficher simplement le text en provenance du serveur
    scPatchDispo, scShowMessage,
      scGetNameAndPass: if assigned(EOnStringBrut) then EOnStringBrut(Self, Id, Cmd, Text);
  else end;
end;

procedure TDataClientNet.ADKClient1ActifItem(Sender: TObject;
  const ID: Cardinal; const Cmd: TServCmd; const Value: TStream);
var ClassCardinal: Cardinal;
  AClass: TBaseActifItemClass;
  ActifItem: TBaseActifItem;
begin
  // Reception d'un elements actif d'une carte(MiniPerso par exemple)
  // Value contient les elements permetant de retrouver la Class de l'objet
  Value.Position:= 0;
  Value.ReadBuffer(ClassCardinal, Sizeof(ClassCardinal));
  AClass:= TBaseActifItemClass(ClassFromId(ClassCardinal)); //On recuperer sa Class
  ActifItem:= AClass.create(CompoActifItem1.ListItems);
  ActifItem.ReadData(Value);
  CompoActifItem1.ListItems.add(ActifItem);
end;

procedure TDataClientNet.ADKClient1Cardinal(Sender: TObject;
  const ID: Cardinal; const Cmd: TServCmd; Value: Cardinal);
begin
  // reception d'un cardinal
  // Cet vnement est utilis par plusieurs commmande.
  // Il faut vrifier quelle est la commande qui nous a ammen ici
  case Cmd of
    scMiniPersoDeco: begin
      // Deconecter le MiniPerso ayant son ID = VALUE
      // ATTENTION:
      // L'ID de la procedure est l'ID du perso qui envoie la commande.
      // Il est possible que ce ne soit pas forcement le perso  deconnecter.
      // Donc il faut bel et bien utiliser VALUE pour chercher le MiniPerso
      //  supprim.
        with CompoActifItem1.ListItems do
          DeleteById(Value)
      end;
    scRotate: if assigned(ERotatePosition) then ERotatePosition(self, Id, Value);
    scGetNbrIncrit: if Assigned(EOnTotalInscrit) then EOnTotalInscrit(self, Id, Value);

    scPing: exit;
  else end;
end;


procedure TDataClientNet.PersosComplets1Delete(Sender: TObject;
  Value: TCustomPerso);
begin
  // Le OnDelete de PersosComplets1 se produit AVANT l'effacement rel du
  // perso dans la liste. Cela permet de controler ici quel perso est
  // effacer (Value est le TCustomPerso qui va tre effacer)
  // et de faire quelque modif ou enregistrement ou autre avant son effacement
  // Donc Si Count =1 cela veut dire que c'est le dernier perso de la liste
  // qui va tre effacer. Dans ce cas on ferme la session
  if PersosComplets1.count = 1 then CloseSession;
end;

procedure TDataClientNet.CompoActifItem1Add(Sender: TObject;
  ActifItem: TBaseActifItem);
begin
  if ActifItem is TminiPerso then
    if assigned(EOnMinipersoIn) then EOnMinipersoIn(self, ActifItem);
  if assigned(EOnActifItemIn) then EOnActifItemIn(self, ActifItem);
  //DisplayServer(Value.Perso.name+ ' est entr(e).');
end;

procedure TDataClientNet.CompoActifItem1Delete(Sender: TObject;
  ActifItem: TBaseActifItem);
begin
  if ActifItem is TminiPerso then
    if assigned(EOnMinipersoOut) then EOnMinipersoOut(self, ActifItem);
  if assigned(EOnActifItemOut) then EOnActifItemOut(self, ActifItem);

  //DisplayServer(Value.Perso.name+ ' est sorti(e).');
end;

procedure TDataClientNet.ADKClient1Message21(Sender: TObject;
  const ID: Cardinal; const Cmd: TServCmd; const IdDestinataire: Cardinal;
  Text: string; const Attr: TTextAttr);
begin
  // Reception d'un message, avec attributs, envoy par un perso
  // vers le perso IdDestinataire
  case Cmd of
    scText21: if assigned(EOnTextAtribut21) then EOnTextAtribut21(self, Id, IDDestinataire, Attr, Text);
  else end;
end;

procedure TDataClientNet.ADKClient1String21(Sender: TObject;
  const ID: Cardinal; Cmd: TServCmd; const IdDestinataire: Cardinal;
  Text: string);
begin
  // Reception d'un string brut (sans attributs) envoy par un perso
  // vers le perso IdDestinataire
  case Cmd of
    scString21: if assigned(EOnStringBrut21) then EOnStringBrut21(self, Id, IDDestinataire, Text);
  else end;
end;

// Supprime tous les TBaseActifItem trop loin de MyPosition

procedure RemoveSoFar(MyPosition: Tpoint);
var i: integer;
begin
  with DataClientNet do begin
    for i:= CompoActifItem1.ListItems.Count - 1 downto 0 do begin
      with CompoActifItem1.ListItems[i] do begin
        if (Dist2d(MyPosition, Point(Position.x, Position.y)) >= (fServerInfo.Influence + fServerInfo.ZoneTampon) - 2) then
          CompoActifItem1.ListItems.Delete(i);
      end;
    end;
  end;
end;

procedure TDataClientNet.ActifItemMoveTo(const ID: Cardinal; const Value: TPositionInfo);
var BaseActifItem: TBaseActifItem;
  i: integer;
begin
  BaseActifItem:= CompoActifItem1.ListItems.GetItemByID(Id);
{$IFDEF PNJ}
  if assigned(BaseActifItem) then begin
    OldPoint:= Point(BaseActifItem.Position.x, BaseActifItem.Position.y);
    for i:= 0 to PersosComplets1.ListPerso.Count - 1 do begin
      BaseActifItem.Position:= Value;
      if PersosComplets1.ListPerso.Items[i].ID <> ID then begin
        with PersosComplets1.ListPerso.Items[i] do begin
          oldDist:= Dist2d(OldPoint, Position);
          NewDist:= Dist2d(Point(Value.x, Value.y), Position);
          //DisplayMessage('Valeur de ServerInfo.Influence:' + intTostr(fServerInfo.Influence));
          DisplayMessage(PersosComplets1.ListPerso.Items[i].name + ': OldDist: '+
            IntToStr(oldDist)+
            'NewDist: ' +
            IntToStr(NewDist)+
            'Resultat boolean: '+ BoolToStr((oldDist > fServerInfo.Influence) and (NewDist <= fServerInfo.Influence), true)
            );

//          if (oldDist > fServerInfo.Influence) and (NewDist <= fServerInfo.Influence) then
            ADKClient1.SendCardinal(PersosComplets1.ListPerso.Items[i].ID, scGetActifItem, ID);

        end;
      end else PersosComplets1.ListPerso.Items[i].Position:= Point(Value.x, Value.y);
    end;
  end else
    ADKClient1.SendCardinal(Perso.ID, scGetActifItem, ID);
{$ELSE}
  //On centralize ici les actions  faire par dfaut quand un perso bouge.
  if assigned(BaseActifItem) then begin
    BaseActifItem.Position:= Value;
    if Id = Perso.ID then begin
      Perso.Position:= Point(Value.x, Value.y); //Indispensable sinon c'est le bordel.
      RemoveSoFar(Point(Value.x, Value.y));
    end else begin
      //Si l'actifitem sort de notre zone d'influence+zone tampon on l'efface
      with BaseActifItem.Position do begin
        if (Dist2d(Point(x, y), Perso.Position) >= (fServerInfo.Influence + fServerInfo.ZoneTampon) - 2) then
          CompoActifItem1.ListItems.Remove(BaseActifItem)
      end;
    end;
  end else begin
    // On ne demande les donnes du miniperso que s'il rentre dans notre zone d'influence
    // Le serveur envoie les donnes quand le perso est dans notre "zone indluence+ZoneTampon".
    for i:= 0 to PersosComplets1.ListPerso.Count - 1 do
      if (Dist2d(Point(Value.x, Value.y), PersosComplets1.ListPerso.Items[i].Position) <= (fServerInfo.Influence)) then
        ADKClient1.SendCardinal(PersosComplets1.ListPerso.Items[i].ID, scGetActifItem, ID);
  end;
{$ENDIF}
end;

procedure TDataClientNet.ADKClient1Position(Sender: TObject;
  const ID: Cardinal; const Cmd: TServCmd; const Value: TPositionInfo);
begin
    // Suite  une demande de dplacement soit le point d'arriv est valid, soit le server
    // nous en fournis un valable dans Value: TPositionInfo,
    // soit on n'arrive PAS ici et c'est une commande scStopmove qui est envoy
    // ce qui signifie que le tlport n'est pas possible.

  case cmd of
    scPlaceAt,
      scTeleport,
      scMoveTo,
      scRunTo,
      scJump,
      scEndMove: ActifItemMoveTo(ID, Value);
  else
    showmessage('Commande inconnue dans ADKClient1Position');
  end;
  if assigned(EOnPositionChange) then EOnPositionChange(self, ID, Cmd, Value);
   //property OnPositionChange: TPositionChange read EOnPositionChange write EOnPositionChange;
end;


procedure TDataClientNet.ADKClient1StopMove(Sender: TObject;
  const ID: Cardinal; const Cmd: TServCmd);
begin
  case Cmd of
    scStopmove: if assigned(OnStopPosition) then OnStopPosition(self, Id);
  else end;
end;


end.

